c********************************************************************************************
C
      SUBROUTINE TA_2015_N (IOPT,PARMOD,PS,X,Y,Z,BX,BY,BZ)    !  A DOUBLE-PRECISION SUBROUTINE
C
C  RETURNS COMPONENTS OF THE EXTERNAL MAGNETIC FIELD VECTOR (I.E. DUE TO ONLY MAGNETOSPHERIC 
C  CURRENTS, WITHOUT CONTRIBUTION FROM THE EARTH'S SOURCES), ACCORDING TO THE DATA-BASED MODEL 
C  DRIVEN BY INTERPLANETARY PARAMETERS.
C
C  VERSION OF 09/02/2015, BASED ON FITTING THE MODEL TO THE ENTIRE SET WITH 3 372 032 DATA RECORDS
C
C  REFERENCE:  N. A. TSYGANENKO AND V. A. ANDREEVA, "A FORECASTING MODEL OF THE MAGNETOSPHERE
C  DRIVEN BY AN OPTIMAL SOLAR-WIND COUPLING FUNCTION", ACCEPTED BY JGRA ON SEPTEMBER 15, 2015.
C
C  INPUT PARAMETERS: 
C
C  IOPT              A DUMMY PARAMETER
C  PARMOD            A 10-element array, with the first 4 elements to be specified as follows:
c
C  PARMOD(1)  PDYN, the solar wind dynamic pressure in nPa
C  PARMOD(2)  IMF BY in nT (average over 30-min trailing interval)                                 
C  PARMOD(3)  IMF BZ in nT (average over 30-min trailing interval)                                            
C  PARMOD(4)  XIND, the solar-wind-magnetosphere driving index, based on Newell et al. [2007], 
c             coupling function, calculated as arithmetic mean of its seven 5-min averages, including 
c             a preceding 30-min trailing interval, see documentation file TA15_Model_description.pdf 
C              for further details. Typical values of XIND: between 0 (quiet) and 2 (strongly disturbed)
c
C  PS                GEODIPOLE TILT ANGLE (RADIANS!)
C  X, Y, Z           Cartesian GSM (or GSW) position (in Earth's radii, Re=6371.2km)
C
C---------------------------------------------------------------------------------------
C  OUTPUT:   
C
C  BX, BY, BZ        GSM (GSW) components of the model field (nanoTesla)
C
C  CODED BY N. A. TSYGANENKO, 2015
C
c----------------------------------------------------------------------------------------
      IMPLICIT  REAL * 8  (A - H, O - Z)
      EXTERNAL TAIL15_SHIELDED,SRC_SHIELDED,PRC_SHIELDED
      DIMENSION PARMOD(10), PARAMETERS(-10:30), A(24)
C
      PDPM =PARMOD(1) 
      BYIMF=PARMOD(2) 
      BZIMF=PARMOD(3)
      XIND =PARMOD(4)

      IF (XIND.GT.2.D0) PRINT*,'  WARNING: N-INDEX OUT OF ALLOWED RANGE'

      PS2=PS*PS
      PARAMETERS(-1) = PS
      PARAMETERS(-2) = PDPM
      PARAMETERS(-3) = BZIMF
      PARAMETERS(-4) = BYIMF
      PARAMETERS(-5) = XIND 
C
      A(1) =1.D0
      A(2) =    1.07720     +0.800154  *XIND -0.392647 *XIND**2
      A(3) =  -0.180591E-01 -0.555277  *XIND +0.264679 *XIND**2
      A(4) =   0.740594     +0.859522  *XIND -0.404738 *XIND**2
      A(5) =  -0.123505      -2.95940  *XIND  +1.75654 *XIND**2
      A(6) =   0.129368E-01  +2.83339  *XIND  -1.06924 *XIND**2
      A(7) =  -0.165048      -1.05081  *XIND  +1.03674 *XIND**2
      A(8) =   0.197096      +2.64224  *XIND -0.699619 *XIND**2
      A(9) =   0.579682      -1.65251  *XIND +0.260826 *XIND**2
      A(10)=  -0.362541      +1.26780  *XIND -0.426201 *XIND**2
      A(11)=  -0.552823E-01  +1.03528  *XIND -0.362947 *XIND**2
      A(12)=   0.300880     +0.416150  *XIND -0.318769 *XIND**2
      A(13)=   0.627809     -0.173608  *XIND+0.0129412 *XIND**2
      A(14)=    3.23899      -2.00868  *XIND  +1.23972 *XIND**2
      A(15)=    8.54211      +1.00750  *XIND  -1.78497 *XIND**2
      A(16)=    6.41747      +1.10915  *XIND -0.548092 *XIND**2
      A(17)=    1.20270     +0.874565  *XIND -0.294531 *XIND**2
      A(18)=   0.652163     +0.125087  *XIND -0.114611 *XIND**2
      A(19)=   0.812527     +0.950255  *XIND -0.398697 *XIND**2
      A(20)=    1.36762      -3.14119  *XIND  +1.44749 *XIND**2
      A(21)=    3.20611     -0.918283  *XIND +0.356052 *XIND**2
      A(22)=   0.131783E-02 +0.516695  *XIND -0.248840 *XIND**2
      A(23)=   0.373055     -0.613403  *XIND +0.311073 *XIND**2
      A(24)=    3.46828      -2.99281  *XIND  +1.17698 *XIND**2
C
CCC   PARAMETERS( 1) =  RESERVED FOR THE DIPOLE SHIELDING MODULE, BUT ACTUALLY UNUSED DUMMY PARAMETER
      PARAMETERS( 2) =  A(2)+A(3)*PS2    ! MAGNITUDE OF THE NEW TAIL15 SOURCE       
      PARAMETERS( 3) =  A(4)+A(5)*PS2    ! MAGNITUDE OF THE SRC                     
      PARAMETERS( 4) =  A(6)+A(7)*PS2    ! MAGNITUDE OF THE PRC                     
      PARAMETERS( 5) =  A(8)+A(9)*PS2    ! MAGNITUDE OF THE R1 (R) FAC SYSTEM       
      PARAMETERS( 6) =  A(10)*PS         ! MAGNITUDE OF THE R1 (A) FAC SYSTEM       
      PARAMETERS( 7) =  A(11)            ! IMF BY & BZ PENETRATION COEFFICIENT      
C
      PARAMETERS( 8) =  A(12)  ! PWR: EXPONENT OF THE TAIL15 RADIAL FALL-OFF RATE OF THE ELECTRIC CURRENT FUNCTION (0.5<PWR<2)
      PARAMETERS( 9) =  A(13)  ! PWP: EXPONENT IN THE MAGNITUDE FACTOR OF TAIL15 AS A FUNCTION OF S.W. RAM PRESSURE (PWP~0.3)
      PARAMETERS(10) =  A(14)  ! XC: LOCATION OF THE CURVATURE CENTER OF THE TAIL15 CURRENT FLOW LINES  (0<XC<10)
      PARAMETERS(11) =  A(15)  ! RN: GEOCENTRIC RADIAL DISTANCE TO THE INNER EDGE OF THE TAIL15 CURRENT SHEET (8<RN<12)
      PARAMETERS(12) =  A(16)  ! RH: HINGING DISTANCE (COMMON PARAMETER OF ALL EQUATORIAL SOURCES)
      PARAMETERS(13) =  A(17)  ! SRCSCL: SRC SCALE FACTOR
      PARAMETERS(14) =  A(18)  ! SRCEPS: SRC EPS PARAMETER (CONTROLS NOON-MIDNIGHT ASYMMETRY OF SRC)
      PARAMETERS(15) =  A(19)  ! PRCSCL: PRC SCALE FACTOR 
      PARAMETERS(16) =  A(20)  ! PRCEPS: PRC EPS PARAMETER (CONTROLS NOON-MIDNIGHT ASYMMETRY OF PRC) 
      PARAMETERS(17) =  A(21)  ! PRC LONGITUDE ANGLE (RADIANS)
      PARAMETERS(18) =  A(22)  ! R1_Theta00:  R1 FAC NOON FOOTPOINT COLATITUDE IN RADIANS (BETWEEN 0.20 AND 0.35) 
      PARAMETERS(19) =  A(23)  ! R1_DTheta00: R1 FAC FTPT COLATITUDE INCREMENT FROM NOON TO MIDNIGHT (BETWEEN 0.07 AND 0.17)
      PARAMETERS(20) =  A(24)  ! DMIDN:  TAIL SHEET HALFTHICKNESS AT MIDNIGHT  (BETWEEN 1.0 AND 4.0)
C
      CALL DIPOLE_SHIELD (X,Y,Z,PS,PDPM,BZIMF,CFX,CFY,CFZ)   !  THE DIPOLE SHIELDING FIELD IS PRESSURE-SCALED INSIDE THE DIPOLE_SHIELD(X,Y,Z,...)
C                                                            !   ALL OTHER MODULES NEED IT TO BE DONE IN THIS S/R (BELOW).
C
C  NOW CALCULATE CONTRIBUTIONS FROM INTRA-MAGNETOSPHERIC FIELD SOURCES:
C  BUT, FIRST OF ALL, PRESSURE-SCALE THE COORDINATES TO THE CURRENT VALUE OF Pdyn (here designated as PDPM):
C
      FPDPM=(PDPM/2.01D0)**0.194D0   !  RAM PRESSURE SCALING FACTOR THE POWER INDEX ACCORDING TO LIN ET AL. [2010]
      XX=X*FPDPM
      YY=Y*FPDPM
      ZZ=Z*FPDPM 
C
c----------------------------------------------------------------------------------------------
C   (2) TAIL15 MODULE 
C
      CALL DEFORM_XZ_YZ (PS,PARAMETERS,XX,YY,ZZ,TBX,TBY,TBZ,
     *  TAIL15_SHIELDED) 
      TX=PARAMETERS(2)*TBX
      TY=PARAMETERS(2)*TBY
      TZ=PARAMETERS(2)*TBZ
c------------------------------------------------------------------------------------------------
C
C   (3) AXISYMMETRIC RING CURRENT (SRC)
C
      CALL DEFORM_XZ_YZ(PS,PARAMETERS,XX,YY,ZZ,SBX,SBY,SBZ,SRC_SHIELDED) 
      SX=PARAMETERS(3)*SBX
      SY=PARAMETERS(3)*SBY
      SZ=PARAMETERS(3)*SBZ
C
c------------------------------------------------------------------------------------------------
C
C   (4) PARTIAL RING CURRENT (PRC)
C
      CALL DEFORM_XZ_YZ(PS,PARAMETERS,XX,YY,ZZ,PBX,PBY,PBZ,PRC_SHIELDED) 
      PX=PARAMETERS(4)*PBX
      PY=PARAMETERS(4)*PBY
      PZ=PARAMETERS(4)*PBZ
C
C------------------------------------------------------------------------------------------------
C
C   (5) REGION 1 FACs -  FIRST NORTH-SOUTH-SYMMETRIC MODE (R): 
C
       Theta00 =PARAMETERS(18)
       DTheta00=PARAMETERS(19)
C
       CALL R1_FAC_R(Theta00,DTheta00,PS,XX,YY,ZZ,BXR1R_UNSH,BYR1R_UNSH,
     *      BZR1R_UNSH)                                      ! THE SUFFIX "R" CORRESPONDS TO REGULAR MODE OF R1 FACs
       CALL R1_R_SHLD (XX,YY,ZZ,PS,BZIMF,Theta00,DTheta00,   !  SHIELDING FIELD FOR THE "R" MODE
     *                 BXR1R_SHLD,BYR1R_SHLD,BZR1R_SHLD)
C
       R1RX=PARAMETERS(5)*(BXR1R_UNSH+BXR1R_SHLD)*FPDPM
       R1RY=PARAMETERS(5)*(BYR1R_UNSH+BYR1R_SHLD)*FPDPM
       R1RZ=PARAMETERS(5)*(BZR1R_UNSH+BZR1R_SHLD)*FPDPM
C
C-----------------------------------------------------------------------------------------------------
C
C   (6) REGION 1 FACs -  NOW NORTH-SOUTH-ANTISYMMETRIC MODE (A): 
C
       CALL R1_FAC_A (PS,XX,YY,ZZ,BXR1A_UNSH,BYR1A_UNSH,BZR1A_UNSH) ! THE SUFFIX "A" CORRESPONDS TO TILT-ANTISYMMETRIC MODE OF R1 FACs
       CALL R1_A_SHLD (XX,YY,ZZ,PS,BZIMF,Theta00,DTheta00,          !  SHIELDING FIELD FOR THE "A" MODE
     *                 BXR1A_SHLD,BYR1A_SHLD,BZR1A_SHLD)
C
       R1AX=PARAMETERS(6)*(BXR1A_UNSH+BXR1A_SHLD)*FPDPM
       R1AY=PARAMETERS(6)*(BYR1A_UNSH+BYR1A_SHLD)*FPDPM
       R1AZ=PARAMETERS(6)*(BZR1A_UNSH+BZR1A_SHLD)*FPDPM
C
C-------------------------------------------------------------------------------------------------
C
C   (7) PENETRATED TRANSVERSE PART OF THE IMF:
C
       BYPEN=A(11)*BYIMF
       BZPEN=A(11)*BZIMF
C
C----------------------------------------------------------------------------
C
       BX=CFX+TX+SX+PX+R1RX+R1AX
       BY=CFY+TY+SY+PY+R1RY+R1AY+BYPEN
       BZ=CFZ+TZ+SZ+PZ+R1RZ+R1AZ+BZPEN
C      
      RETURN
      END

cOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
c
      SUBROUTINE TAIL15_SHIELDED (X,Y,Z,PARAMETERS,BX,BY,BZ)   
C
C  INPUT:  X, Y, Z  - CARTESIAN GSM POSITION (IN RE)
C  OUTPUT: BX,BY,BZ - CARTESIAN GSM B-FIELD COMPONENTS
C
C       AUTHOR:  N. A. TSYGANENKO, 2015

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION PARAMETERS(-10:30)
C
       PDPM   =PARAMETERS(-2)
       BZIMF  =PARAMETERS(-3)
C
       PWR    = PARAMETERS( 8) !   PWR: EXPONENT OF THE TAIL15 RADIAL FALL-OFF RATE OF THE ELECTRIC CURRENT FUNCTION (0.5<PWR<2)
       PWP    = PARAMETERS( 9) !   PWP: EXPONENT IN THE MAGNITUDE FACTOR OF TAIL15 AS A FUNCTION OF S.W. RAM PRESSURE (PWP~0.3)
       XC     = PARAMETERS(10) !   XC: LOCATION OF THE CURVATURE CENTER OF THE TAIL15 CURRENT FLOW LINES  (0<XC<10)
       RN     = PARAMETERS(11) !   RN: GEOCENTRIC RADIAL DISTANCE TO THE INNER EDGE OF THE TAIL15 CURRENT SHEET (8<RN<12)
       RH     = PARAMETERS(12) !   RH: HINGING DISTANCE (COMMON PARAMETER OF ALL EQUATORIAL SOURCES)
       DMIDN  = PARAMETERS(20) !   DMIDN:  TAIL SHEET HALFTHICKNESS AT MIDNIGHT  (BETWEEN 1.0 AND 4.0)                         
C
       T15FACT =(PDPM/2.D0)**PWP

       CALL TAIL15_UNSHIELDED (DMIDN,PWR,XC,RN,X,Y,Z,HXT15,HYT15,HZT15)
       CALL TAIL15_SHLD (X,Y,Z,BZIMF,PWR,XC,RN,FXT15,FYT15,FZT15)

       BX=(HXT15+FXT15)*T15FACT
       BY=(HYT15+FYT15)*T15FACT
       BZ=(HZT15+FZT15)*T15FACT

       RETURN
       END
C
C================================================================================================
c
      SUBROUTINE SRC_SHIELDED (X,Y,Z,PARAMETERS,BX,BY,BZ)   
C
C  INPUT:  X, Y, Z  - CARTESIAN GSM POSITION (IN RE)
C  OUTPUT: BX,BY,BZ - CARTESIAN GSM B-FIELD COMPONENTS
C
C     AUTHOR:  N. A. TSYGANENKO, 2015
c
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION PARAMETERS(-10:30)
C
       BZIMF  =PARAMETERS(-3)
       SRCSCL = PARAMETERS(13)      !   SRCSCL: SRC SCALE FACTOR
       SRCEPS = PARAMETERS(14)      !   SRCEPS: SRC EPS PARAMETER (CONTROLS NOON-MIDNIGHT ASYMMETRY OF SRC)
C
       CALL SRC_UNSH (SRCEPS,SRCSCL,X,Y,Z,HXSRC,HYSRC,HZSRC)
       CALL SRC_SHLD (X,Y,Z,BZIMF,SRCEPS,SRCSCL,FXSRC,FYSRC,FZSRC)

       BX=HXSRC+FXSRC          
       BY=HYSRC+FYSRC          
       BZ=HZSRC+FZSRC          

       RETURN
       END
C
C================================================================================================
c
      SUBROUTINE PRC_SHIELDED (X,Y,Z,PARAMETERS,BX,BY,BZ)   
C
C  INPUT:  X, Y, Z  - CARTESIAN GSM POSITION (IN RE)
C  OUTPUT: BX,BY,BZ - CARTESIAN GSM B-FIELD COMPONENTS
C
C     AUTHOR:  N. A. TSYGANENKO, 2015

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION PARAMETERS(-10:30)
C
       BZIMF  = PARAMETERS(-3)
       PRCSCL = PARAMETERS(15)      !   PRCSCL: PRC SCALE FACTOR 
       PRCEPS = PARAMETERS(16)      !   PRCEPS: PRC EPS PARAMETER (CONTROLS NOON-MIDNIGHT ASYMMETRY OF PRC) 
       PRC_PHI= PARAMETERS(17)      !   PRC LONGITUDE ANGLE (RADIANS)
C
       CPRC=DCOS(PRC_PHI)
       SPRC=DSIN(PRC_PHI)
C
       CALL PRC_UNSH_NM (PRCEPS,PRCSCL,X,Y,Z,HXPRC1,HYPRC1,HZPRC1)
       CALL PRC_SHLD_NM (X,Y,Z,BZIMF,PRCEPS,PRCSCL,FXPRC1,FYPRC1,FZPRC1)
       BNMX=HXPRC1+FXPRC1
       BNMY=HYPRC1+FYPRC1
       BNMZ=HZPRC1+FZPRC1
C
       CALL PRC_UNSH_DD (PRCEPS,PRCSCL,X,Y,Z,HXPRC2,HYPRC2,HZPRC2)
       CALL PRC_SHLD_DD(X,Y,Z,BZIMF,PRCEPS,PRCSCL,FXPRC2,FYPRC2,FZPRC2)
       BDDX=HXPRC2+FXPRC2
       BDDY=HYPRC2+FYPRC2
       BDDZ=HZPRC2+FZPRC2
C
       CALL PRCS_UNSH (PRCEPS,PRCSCL,X,Y,Z,HXPRCS,HYPRCS,HZPRCS)
       CALL PRCS_SHLD (X,Y,Z,BZIMF,PRCEPS,PRCSCL,FXPRCS,FYPRCS,FZPRCS)
       BSYMX=HXPRCS+FXPRCS
       BSYMY=HYPRCS+FYPRCS
       BSYMZ=HZPRCS+FZPRCS
C
       BX=-BNMX*CPRC+BDDX*SPRC+BSYMX
       BY=-BNMY*CPRC+BDDY*SPRC+BSYMY
       BZ=-BNMZ*CPRC+BDDZ*SPRC+BSYMZ

       RETURN
       END
C
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE DEFORM_XZ_YZ (PS,PARAMETERS,X,Y,Z,BX,BY,BZ,
     *   B_EXT_UNTILTED)
C
C   CALCULATES GSM COMPONENTS OF THE WARPED (IN YZ) & BENT (IN XZ) FIELD, DERIVED FROM A SHIELDED
C    UNTILTED  MODEL TAIL FIELD.
C
C   THE TILT-RELATED DEFORMATION IS IMPOSED ON THE UNWARPED FIELD, COMPUTED BY THE S/R "B_EXT_UNTILTED".
C
C     AUTHOR:  N. A. TSYGANENKO, 2015

      IMPLICIT REAL*8 (A-H,O-Z)
C
      DIMENSION A(27)
      DIMENSION PARAMETERS(-10:30)
C
      DATA A/.6563406936D0,.4596362842D0,-.1206495537D0,.3175118510D0,
     *0.3546433781D0,-0.1987301198D0,-0.6233108192D-01,0.2040942759D0,
     *0.2718066023D-01,-.2798547019D-01,-.1610526D-01,-0.1069132928D-02,
     *.2242677676D-01,-.5679826540D-01,-.2772268386D-02,-.258946203D-02,
     *0.3512794890D-02,0.4717943878D-02,0.5878180276D0,0.3796190668D-01,
     *-0.1438414960D0,0.6721710243D0,0.3461738529D-01,-0.7534481962D0,
     *-0.1278768535D0,0.4255447972D0,0.7119080497D-01/
C
      DATA RH0/8.0D0/
C
c  BEGIN WITH THE 'UNBENDING' DEFORMATION, WHICH TRANSFORMS THE BENT (IN X-Z COORDINATES) CONFIGURATION
C  TO THE UNBENT ONE, BY REPLACING PHI=ATAN(Z,X) WITH PHIS=ATAN(ZAS,XAS) AND KEEPING Y-COORDINATE INTACT.
C
C  TO THAT END, CALCULATE (1) THE "UNBENT" POLAR COORDINATES RHOS AND PHIS AND
C                         (2) THEIR DERIVATIVES W.R.T. ORIGINAL COORDS RHO AND PHI, ENTERING
C                             IN THE DEFORMATION TENSOR (SEE T-1998 PAPER FOR DETAILS):
      RH=PARAMETERS(12)
      BZIMF=PARAMETERS(-3)
C
      FRH=(RH-RH0)/RH0
      FBZ=BZIMF/5.D0
C
      B1=A(1)+FBZ*A(10)+FRH*A(19)
      B2=A(2)+FBZ*A(11)+FRH*A(20)
      B3=A(3)+FBZ*A(12)+FRH*A(21)
      B4=A(4)+FBZ*A(13)+FRH*A(22)
      B5=A(5)+FBZ*A(14)+FRH*A(23)
      B6=A(6)+FBZ*A(15)+FRH*A(24)
      B7=A(7)+FBZ*A(16)+FRH*A(25)
      B8=A(8)+FBZ*A(17)+FRH*A(26)
      B9=A(9)+FBZ*A(18)+FRH*A(27)

      RH2=RH**2
      SPS=DSIN(PS)
      CPS=DCOS(PS)
      RHO2XZ=X**2+Z**2
      RHOXZ=DSQRT(RHO2XZ)
      RHO4RH4=DSQRT(DSQRT(RHO2XZ**2+RH2**2))
C
      IF (X.EQ.0.D0.AND.Z.EQ.0.D0) THEN
       PHI1=0.D0
       CPHI1=1.D0
       SPHI1=0.D0
      ELSE
       PHI1=DATAN2(Z,X)
       CPHI1=X/RHOXZ
       SPHI1=Z/RHOXZ
      ENDIF

      S=PS*(RHO4RH4-RH)
      DSDRHO=PS*RHO2XZ*RHOXZ/RHO4RH4**3

      SQT=S+RHOXZ*DSIN(PS+PHI1)
      DTDRHO=2.D0*SQT*(DSDRHO+DSIN(PS+PHI1))
      DTDPHI=2.D0*SQT*RHOXZ*DCOS(PS+PHI1)
      T=SQT**2
C
      F=1.D0-(B1+B2*CPHI1+B3*(2.D0*CPHI1**2-1.D0))*T/(RH2+T)
      DFDRHO=-RH2*DTDRHO/(RH2+T)**2*(B1+B2*CPHI1
     *  +B3*(2.D0*CPHI1**2-1.D0))
      DFDPHI=-RH2/(RH2+T)**2*DTDPHI*(B1+B2*CPHI1
     *  +B3*(2.D0*CPHI1**2-1.D0))
     *  +(B2*SPHI1+4.D0*B3*SPHI1*CPHI1)/(RH2+T)*T
      BR1=RHO4RH4+RH
      BR2=RHO4RH4**2+RH**2
      PHIS1=PHI1+F*PS*(1.D0+RHO2XZ*RHOXZ/BR1*CPHI1/BR2)

      RHOS_OVER_RHOXZ=1.D0+RHOXZ/RH**2*(PS*SPHI1*(B4+B5*SPHI1**2)
     *  +B6*(PS*SPHI1)**2)
     * +RHOXZ**2/RH**3*(PS*SPHI1*(B7+B8*SPHI1**2)+B9*(PS*SPHI1)**2)

      RHOS=RHOS_OVER_RHOXZ*RHOXZ

      DRHOSDRHO=1.D0+
     *2.D0*(RHOXZ/RH**2)*(PS*SPHI1*(B4+B5*SPHI1**2)+B6*(PS*SPHI1)**2)+
     *3.D0*(RHOXZ/RH)**2/RH*(PS*SPHI1*(B7+B8*SPHI1**2)+B9*(PS*SPHI1)**2)

      DRHOSDPHI_OVER_RHOXZ=RHOXZ/RH**2*PS*(B4+3.D0*B5*SPHI1**2
     * +2.D0*B6*PS*SPHI1)*CPHI1+RHOXZ**2/RH**3*PS*(B7+3.D0*B8*SPHI1**2
     * +2.D0*B9*PS*SPHI1)*CPHI1

      DRHOSDPHI=DRHOSDPHI_OVER_RHOXZ*RHOXZ

      DPHIS1DRHO=PS*(DFDRHO*(1.D0+RHO2XZ*RHOXZ*CPHI1/(BR1*BR2))
     *  +F*CPHI1*RHO2XZ*(1.D0/RHO4RH4**3-1.D0/(BR1*BR2)))

      DPHIS1DPHI=1.D0+PS*(DFDPHI*(1.D0+RHO2XZ*RHOXZ*CPHI1/(BR1*BR2))
     *  -F*RHO2XZ*RHOXZ*SPHI1/(BR1*BR2))
C
C  CONVERT THE OBTAINED "UNBENT" POLAR COORDS TO CARTESIAN ONES XAS & ZAS:
C
      CPHIS1=DCOS(PHIS1)
      SPHIS1=DSIN(PHIS1)
      XAS1=RHOS*CPHIS1
      ZAS1=RHOS*SPHIS1
C
C  NOW APPLY ANOTHER DEFORMATION TO THE "UNBENT" COORDINATES: TRANSFORM THEM TO "UNWARPED" ONES
C    (THEREBY CONVERTING THE "TROUGH-LIKE" CURRENT SHEET INTO A PLANAR ONE):
C
      RHO2YZ=Y**2+ZAS1**2
      RHOYZ=DSQRT(RHO2YZ)
      RHO4RH4=DSQRT(DSQRT(RHO2YZ**2+RH2**2))
C
      IF (Y.EQ.0.D0.AND.ZAS1.EQ.0.D0) THEN   !  ATTENTION:  HERE PHI AND RELATED QUANTITIES
       PHI2=0.D0                             !   ARE DIFFERENT FROM THOSE IN THE 1ST DEFORMATION !
       CPHI2=1.D0                            !   (HENCE PHI2 ETC., IN PLACE OF PHI1 ...)
       SPHI2=0.D0
      ELSE
       PHI2=DATAN2(ZAS1,Y)
       CPHI2=Y/RHOYZ
       SPHI2=ZAS1/RHOYZ
      ENDIF
C
      G=DEXP(XAS1/(2.D0*RH))   !  WE ASSUME THE SCALE EQUAL TO 2*RH (ON THE ORDER OF DOUBLED HINGING DISTANCE)
      DGDX=G/(2.D0*RH)
      F=RHO2YZ/((RH+RHO4RH4)*(RHO4RH4**2+RH2))
      PHIS2=PHI2+F*CPHI2*PS*G*RHOYZ
      CPHIS2=DCOS(PHIS2)
      SPHIS2=DSIN(PHIS2)
C
      DPHIS2DRHO=PS*CPHI2*G*RH*F/RHO4RH4**3*(RHO4RH4**2+RH2+RH*RHO4RH4)
      DPHIS2DPHI=1.D0-SPHI2*PS*G*F*RHOYZ
      DPHIS2DX=(PHIS2-PHI2)*DGDX/G

      YAS2=RHOYZ*CPHIS2
      ZAS2=RHOYZ*SPHIS2
c
c  CALCULATE THE UNDEFORMED (NO-TILT) FIELD AT (XAS1,YAS2,ZAS2):
c
      CALL B_EXT_UNTILTED (XAS1,YAS2,ZAS2,PARAMETERS,BX_AS2,BY_AS2,
     *  BZ_AS2)
C
C   CONVERT THE OBTAINED CARTESIAN BY AND BZ COMPONENTS OF THE "ASTERISKED" B
C   TO CYLINDRICAL ONES, USING THE "UNWARPED" ANGULAR COORDINATE PHIS2 AND ITS
C    SINE/COSINE (SEE TSY 2002-I PAPER, EQS. (7)-(10)):
C
      BRHO_AS2 =  BY_AS2*CPHIS2+BZ_AS2*SPHIS2
      BPHI_AS2 = -BY_AS2*SPHIS2+BZ_AS2*CPHIS2
C
C  FIND B_PRIME-2 COMPONENTS FROM ASTERISKED ONES:
C
      BRHO_S2 =  BRHO_AS2*DPHIS2DPHI
      BPHI_S2 =  BPHI_AS2-RHOYZ*(BX_AS2*DPHIS2DX+BRHO_AS2*DPHIS2DRHO)
      BX_S2   =  BX_AS2*DPHIS2DPHI
      BY_S2 = BRHO_S2*CPHI2-BPHI_S2*SPHI2
      BZ_S2 = BRHO_S2*SPHI2+BPHI_S2*CPHI2
C
C  NOW FIND B_PRIME-1 COMPONENTS (I.E., CORRESPONDING TO BOTH "WARPED" AND "BENT" FIELD).
C   FIRST, CALCULATE RHO AND PHI-COMPONENTS OF THE "ASTERISKED" B IN THE 1ST SYSTEM FROM
C    CARTESIAN ONES, USING THE "UNBENT" ANGULAR COORDINATE PHIS1 AND ITS SINE/COSINE:
C
      BRHO_AS1 = BX_S2*CPHIS1+BZ_S2*SPHIS1
      BPHI_AS1 =-BX_S2*SPHIS1+BZ_S2*CPHIS1
C
C  CALCULATE THE DOUBLE-DEFORMED FIELD COMPONENTS (CYLINDRICAL COORDS), USING
C    THE ABOVE CALCULATED ELEMENTS OF THE DEFORMATION TENSOR:
C
      BRHO_S1 = BRHO_AS1*RHOS_OVER_RHOXZ*DPHIS1DPHI
     * -BPHI_AS1*DRHOSDPHI_OVER_RHOXZ
      BPHI_S1 = -BRHO_AS1*RHOS*DPHIS1DRHO+BPHI_AS1*DRHOSDRHO
      BY      =  BY_S2*RHOS_OVER_RHOXZ*(DRHOSDRHO*DPHIS1DPHI
     * -DRHOSDPHI*DPHIS1DRHO)
C
C  FINALLY, TRANSFORM BRHO & BPHI INTO CARTESIAN COMPONENTS (OUTPUT OF THIS S/R):
C
      BX    = BRHO_S1*CPHI1-BPHI_S1*SPHI1
      BZ    = BRHO_S1*SPHI1+BPHI_S1*CPHI1
C
      RETURN
      END
C
C=============================================================================================
C
      SUBROUTINE DIPOLE_SHIELD (X,Y,Z,PSI,PDPM,BZIMF,BX,BY,BZ)
C
C   CALCULATES COMPONENTS {FX,FY,FZ} OF THE DIPOLE SHIELDING FIELD VECTOR AT ANY GIVEN POSITION X,Y,Z
C
C      INPUT:  X,Y,Z       - POSITION VECTOR (GSM)
C              PSI         - DIPOLE TILT ANGLE (RADIANS)
C              PDPM        - SOLAR WIND RAM + MAGNETIC PRESSURE, nPa  (PM=0.000398*B_IMF^2)
C              BZIMF       - BZ (GSM) COMPONENT OF THE IMF
C
C      OUTPUT: BX,BY,BZ    - DIPOLE SHIELDING FIELD COMPONENTS
c
C      N. A. Tsyganenko, March 2015;   
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C     NLIN=192 linear parameters enter in the amplitudes of the "cartesian" harmonics
c     NNP=8 nonlinear parameters are the scales Pi & Qi, entering in the arguments
C     of exponents, sines, and cosines in the Cartesian "box" harmonics
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
       IMPLICIT  REAL * 8  (A - H, O - Z)
C
       DIMENSION P(4),Q(4)
       DIMENSION A(216)
C
       DATA A
     */ 63.86081318D0,-0.3885695056D0, -2.849842473D0, -60.28997524D0,
     *  3.233295175D0,  2.871808009D0, -73.62746378D0, -8.245643028D0,
     *  6.595835800D0, -6.951805745D0,  2.800347308D0, -1.356635839D0,
     * -5.143464615D0,  6.408399581D0, -6.067725736D0,  30.67977107D0,
     *  5.000396216D0, -1.055729325D0,  3.419893572D0, -3.075383297D0,
     *  2.659892266D0, -19.75394468D0, -4.566011794D0,  2.951809155D0,
     * -109.7271663D0, -34.43568959D0,  27.41946353D0, -95.07105895D0,
     *  9.843457076D0,  1.719957805D0, -22.79541332D0,  12.99857748D0,
     * -11.40354272D0,  44.02049830D0, 0.6871863029D0,  2.199806700D0,
     *  6.757295444D0, -1.408254083D0, 0.5197439507D0, -27.60415416D0,
     * -10.25059155D0,  4.134026820D0, 0.7404641498D-01,2.238492418D0,
     * -1.864478684D0,  16.84429987D0,  6.769569345D0, -3.726813564D0,
     *  162.5809875D0,  67.22623843D0, -58.30248283D0, -27.72898928D0,
     * -169.2137151D0,  42.14614616D0, -37.34007191D0, -16.98302872D0,
     *  16.18609972D0,  19.73195259D0,  44.50393218D0, -18.41892067D0,
     *  11.97092245D0,  8.064302756D0, -9.609617953D0, -13.33814648D0,
     * -12.97520543D0,  13.68442596D0, -3.468923614D0, -4.018150510D0,
     *  4.689195165D0,  2.036783832D0,  4.080251737D0, -5.416868377D0,
     * -80.84117540D0, -45.53695917D0,  21.29724170D0, -196.1425005D0,
     *  139.1928444D0,  10.81721683D0,  21.07960678D0,  12.10130166D0,
     * -7.425059208D0,  37.59242095D0, -37.11111910D0,  3.418232069D0,
     * -5.696353206D0, -4.803016086D0,  4.970713066D0,-0.9094040713D0,
     *  10.96145505D0, -7.336800977D0,  1.899094764D0,  2.032232484D0,
     * -2.243069677D0, 0.1098379146D-01,-3.720170927D0, 3.270198923D0,
     *  6.453873059D0,-0.3456294745D0, 0.6299853220D-01,-6.121609605D0,
     * 0.3728488180D0, 0.1615893793D0, -48.10449335D0, -20.31468938D0,
     *  21.11181314D0,  2.555708375D0,  25.01753057D0, -36.79671388D0,
     * -105.1198326D0, -9.203192329D0,  12.26438789D0,  27.13684161D0,
     *  14.83319028D0, -22.52796872D0,  57.12677447D0,  42.57944976D0,
     * -57.08889030D0,  54.52422848D0, -23.45756314D0,  21.76700933D0,
     * -52.40139083D0, -39.73980409D0,  53.82491643D0, -50.71497018D0,
     *  23.25099793D0, -22.20835644D0, -57.87812254D0, -5.289278367D0,
     *  6.664141243D0,  6.258399887D0,  11.51824518D0, -15.16654009D0,
     * -1.941465351D0,  3.421124393D0, -2.587475652D0,  16.70299906D0,
     * 0.9129739383D0, -8.260190639D0, -38.71315542D0, -15.95894571D0,
     *  30.52336081D0, -98.21679787D0,  48.74035160D0, -23.63976948D0,
     *  39.87167804D0,  14.57342294D0, -28.90843517D0,  92.25160373D0,
     * -44.66671904D0,  20.43199614D0,  1.153643473D0, -6.738510233D0,
     *  7.189828136D0, -22.19675170D0,  65.32693194D0, -54.92558715D0,
     * -8.414249373D0,  29.34844552D0, -27.20567962D0,  32.30112038D0,
     * -79.30209098D0,  60.88466980D0,  449.2928512D0,  110.7275433D0,
     * -234.6865986D0,  73.64104049D0, -2.021078006D0, -14.24961044D0,
     * -423.6136930D0, -107.7302335D0,  225.4358329D0, -84.25699978D0,
     *  5.750942072D0,  19.30387318D0,  2.179232843D0,  7.744352121D0,
     * -9.612057958D0,  13.63358628D0, -68.31902318D0,  62.40732796D0,
     *  5.641664969D0, -30.12199545D0,  30.13377902D0, -19.39562341D0,
     *  87.13214102D0, -76.43904826D0, -407.1903205D0, -98.05581795D0,
     *  208.9313367D0, -89.98673432D0, -41.31483346D0,  70.22247312D0,
     *  384.2137860D0,  95.43979917D0, -200.8837009D0,  97.66803457D0,
     *  34.50569490D0, -70.35473096D0,-0.3043796699D-01,0.7762568873D0,
     *-0.8998460979D0,-0.7098931980D0, -1.289923999D0,  1.986337283D0,
     * 0.1447178814D-01,0.8413473832D-01,0.1991022951D0,0.2550624546D0,
     * 0.4257168124D-01,0.1103920722D0, 0.2573823975D0, 0.2625976694D0,
     * 0.7183038459D-02,0.7537449434D-02,1.425884491D0, 1.124702200D0/
C
      DATA P,Q /0.1447178814D-01,0.8413473832D-01,0.1991022951D0, 
     * 0.2550624546D0,0.4257168124D-01,0.1103920722D0,0.2573823975D0,
     * 0.2625976694D0/
      DATA FACT1,FACT2,XD1,XD2/0.7183038459D-02,0.7537449434D-02,
     * 28.51768982D0,22.494044D0/
C
         FX=0.D0
         FY=0.D0
         FZ=0.D0
c
         PS2=PSI**2
C
         FPDPM=(PDPM/2.01D0)**0.194D0   !  THE POWER INDEX SET ACCORDING TO LIN ET AL. [2010]
         FPDPM3=FPDPM**3
         XX=X*FPDPM
         YY=Y*FPDPM
         ZZ=Z*FPDPM
C
         FBZ=BZIMF/5.D0
         FBZ21=DABS(FBZ)/DSQRT(1.D0+FACT1*FBZ**2)
         FBZ22=DABS(FBZ)/DSQRT(1.D0+FACT2*FBZ**2)
C
         L=1
C
        DO 2 I=1,4
             PI=P(I)
             CYPI=DCOS(YY*PI)
             SYPI=DSIN(YY*PI)
C
           DO 2 K=1,4
             PK=P(K)
             SZPK=DSIN(ZZ*PK)
             CZPK=DCOS(ZZ*PK)
             SQPP=DSQRT(PI**2+PK**2)
             EPP=DEXP(XX*SQPP)
C
             HX0=-SQPP*EPP*CYPI*SZPK
             HY0= EPP*SYPI*SZPK*PI
             HZ0=-EPP*CYPI*CZPK*PK
C
             HX1=HX0*FBZ
             HY1=HY0*FBZ
             HZ1=HZ0*FBZ
C
             HX2=HX0*FBZ21
             HY2=HY0*FBZ21
             HZ2=HZ0*FBZ21
C
             AL=A(L)
             AL1=A(L+1)
             AL2=A(L+2)
             AL3=A(L+3)
             AL4=A(L+4)
             AL5=A(L+5)
C
             FX=FX+AL*HX0+AL1*HX1+AL2*HX2+(AL3*HX0+AL4*HX1+AL5*HX2)*PS2
             FY=FY+AL*HY0+AL1*HY1+AL2*HY2+(AL3*HY0+AL4*HY1+AL5*HY2)*PS2
             FZ=FZ+AL*HZ0+AL1*HZ1+AL2*HZ2+(AL3*HZ0+AL4*HZ1+AL5*HZ2)*PS2
C
             L=L+6
C
  2        CONTINUE
C
             CALL DIPOLE(0.D0,XX-XD1,YY,ZZ,DPERPX,DPERPY,DPERPZ)
C
             FX=FX+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPERPX
             FY=FY+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPERPY
             FZ=FZ+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPERPZ

             L=L+6

        DO 3 I=1,4
             QI=Q(I)
             CYQI=DCOS(YY*QI)
             SYQI=DSIN(YY*QI)
C
           DO 3 K=1,4
             QK=Q(K)
             SZQK=DSIN(ZZ*QK)
             CZQK=DCOS(ZZ*QK)
             SQQQ=DSQRT(QI**2+QK**2)
             EQQ=DEXP(XX*SQQQ)
C
             HX0=(-SQQQ*EQQ*CYQI*CZQK)*PSI
             HY0=( EQQ*SYQI*CZQK*QI)*PSI
             HZ0=( EQQ*CYQI*SZQK*QK)*PSI
C
             HX1=HX0*FBZ
             HY1=HY0*FBZ
             HZ1=HZ0*FBZ
C
             HX2=HX0*FBZ22
             HY2=HY0*FBZ22
             HZ2=HZ0*FBZ22
C
             AL=A(L)
             AL1=A(L+1)
             AL2=A(L+2)
             AL3=A(L+3)
             AL4=A(L+4)
             AL5=A(L+5)
C
             FX=FX+AL*HX0+AL1*HX1+AL2*HX2+(AL3*HX0+AL4*HX1+AL5*HX2)*PS2
             FY=FY+AL*HY0+AL1*HY1+AL2*HY2+(AL3*HY0+AL4*HY1+AL5*HY2)*PS2
             FZ=FZ+AL*HZ0+AL1*HZ1+AL2*HZ2+(AL3*HZ0+AL4*HZ1+AL5*HZ2)*PS2
c
             L=L+6
C
  3          CONTINUE
C
            CALL DIPOLE(1.570796327D0,XX-XD2,YY,ZZ,DPARAX,DPARAY,DPARAZ)

             FX=FX+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPARAX *PSI
             FY=FY+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPARAY *PSI
             FZ=FZ+(A(L)+A(L+1)*FBZ+A(L+2)*FBZ21+A(L+3)*PS2
     *          +A(L+4)*PS2*FBZ+A(L+5)*PS2*FBZ21)*DPARAZ *PSI
C
      BX=FX*FPDPM3
      BY=FY*FPDPM3
      BZ=FZ*FPDPM3
C
      RETURN
      END
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      SUBROUTINE TAIL15_UNSHIELDED (DMIDN,PW,XC,RN,X,Y,Z,BX,BY,BZ)
      IMPLICIT  REAL * 8  (A - H, O - Z)
C
C  Input:  PW - exponent defining the rate of radial decrease of the current in the sheet
C          XC - GSM coordinate of the curvature center of the current sheet flow lines 
C          RN - geocentric radial distance to the inner edge of the current sheet at midnight 
C
C     AUTHOR:  N. A. TSYGANENKO, 2015

      DATA A0,A1,A2,A3,A4,B0,B1,B2,B3,B4
     */-0.3068451808D0,-0.6047641772D-02,0.5689361754D-01,0.184348423D0,
     *  0.7165072182D-01,0.2499990832D0,0.3119753229D0,0.2950446164D0,
     *  0.1672154523D0,0.2133127189D-01/
c
      DATA PI/3.141592653589793D0/
      DATA FACTOR/400.D0/  ! normalization factor, adjusted so that the unshielded 
c                          current disk yields B~30 nT near the peak of J and at origin
c
      RI=RN+XC    !  starting radius in the summation

      BX=0.D0
      BY=0.D0
      BZ=0.D0

      DD= 7.D0     !  flankward flaring parameter, corresponds to D~4.5 at noon and ~3.5 at terminator
      DRH=5.D0     !  regularization parameter in the equation for D=D(rho,phi) with removed singularity at X=XC 

      RHO=DSQRT((XC-X)**2+Y**2)
      D=DMIDN+DD*(RHO-XC+X)/2.D0*RHO/(RHO**2+DRH**2)
      DDDRHO=DD*DRH**2*(RHO-XC+X)/(RHO**2+DRH**2)**2

      STEP=0.5D0

      DO 1 I=1,300

       R=RI+(I-1)*STEP
       AMP=-(RN/(R-XC))**PW         

       IF (RHO.LE.1.D-8) THEN  ! VERY CLOSE TO Z-AXIS: HERE, BY SYMMETRY, SET DBX=DBY=0. BZ IS CALCULATED
         DBX=0.D0                 ! RETAINING ONLY k^2 AND k^4 TERMS IN THE EXPANSIONS FOR K AND E FOR SMALL k^2
         DBY=0.D0
         DBZ=PI/4.D0*R/DSQRT(R**2+Z**2+D**2)**3 *FACTOR
       ELSE
         IF (RHO.LE.1.D-3) THEN
          SCALE=1.D-3/RHO
         ELSE
          SCALE=1.D0
         ENDIF
 
         X1=(X-XC)*SCALE
         Y1=Y*SCALE
         RH1=RHO*SCALE

         SQB=(R+RH1)**2+Z**2+D**2
         P=1.D0-4.D0*R*RH1/SQB
         F=A0+P*(A1+P*(A2+P*(A3+P*A4)))
     *     -DLOG(P)*(B0+P*(B1+P*(B2+P*(B3+P*B4))))
         FS=A1+P*(2.D0*A2+P*(3.D0*A3+P*4.D0*A4))
     *     -(B0+P*(B1+P*(B2+P*(B3+P*B4))))/P
     *     -DLOG(P)*(B1+P*(2.D0*B2+P*(3.D0*B3+P*4.D0*B4)))

         BRHO=Z/DSQRT(SQB)**3*(F-FS/SQB*8.D0*R*RH1) *FACTOR
         DBZ=(F/RH1-(F*(R+RH1+D*DDDRHO)
     *   +4.D0*R/SQB*FS*(R**2-RH1**2+Z**2+D**2-2.D0*D*RH1*DDDRHO))/SQB)
     *   /DSQRT(SQB) *FACTOR

         DBX=BRHO/RH1*X1/SCALE
         DBY=BRHO/RH1*Y1/SCALE

       ENDIF

       BX=BX+DBX*AMP
       BY=BY+DBY*AMP
  1    BZ=BZ+DBZ*AMP
C
      RETURN
      END
C
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE TAIL15_SHLD (X,Y,Z,BZIMF,PW,XC,RN,FX,FY,FZ)
C
C   CALCULATES COMPONENTS {FX,FY,FZ} OF THE TAIL15 SHIELDING FIELD VECTOR AT A GIVEN LOCATION {X,Y,Z}
C
C     AUTHOR:  N. A. TSYGANENKO, 2015

C   INPUT:  X,Y,Z           - POSITION VECTOR (GSM)
C           BZIMF           - IMF BZ 
C           PW              - EXPONENT IN THE EQUATION FOR THE CURRENT AS FUNCTION OF RADIAL DISTANCE
C           XC              - XGSM POSITION OF THE CURVATURE CENTER OF CURRENT FLOW LINES (XC>0)
C           RN              - GEOCENTRIC RADIAL DISTANCE TO THE INNER EDGE OF THE CURRENT SHEET AT MIDNIGHT (RN>0)
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=510,NTOT=529)
C
        DIMENSION A(NTOT)
        DIMENSION D(NLIN)
        DATA RMAX0/11.D0/
        DATA A/
     * -77.299735D0  ,  144.91631D0  , -91.057277D0  , -2971.3349D0  ,
     *  113.54416D0  ,  35.663838D0  ,  1861.5598D0  , -12.396176D0  ,
     *  304.55727D0  , -211.56189D0  ,  103.51096D0  , -71.726085D0  ,
     * -1489.4013D0  , -65.883343D0  ,  66.252460D0  ,  107.40580D0  ,
     * -377.64247D0  ,  231.18301D0  ,  107.20919D0  , -549.38825D0  ,
     * -355.03208D0  ,  1557.3685D0  ,  86.964365D0  , -638.09932D0  ,
     * -293.30601D0  ,  669.46881D0  ,  563.20128D0  ,  289.55635D0  ,
     * -38.270496D0  , -137.76836D0  ,  40.322425D0  ,  15.783851D0  ,
     * -307.98049D0  ,  268.63108D0  ,  61.467583D0  , -219.97292D0  ,
     * -51.607951D0  ,  61.989688D0  , -247.68114D0  ,  53.642657D0  ,
     *  197.28704D0  , -361.04334D0  ,  1960.1476D0  ,  23.278720D0  ,
     *  202.74748D0  , -1188.9041D0  ,  839.31271D0  ,  533.15725D0  ,
     *  488.15395D0  , -2.9048013D0  ,  108.98846D0  ,  48.231822D0  ,
     * -27.318735D0  ,  153.38323D0  , -645.00110D0  ,  4.6456912D0  ,
     *  138.58784D0  , -17.964537D0  , -454.25003D0  ,  47.790107D0  ,
     * -11.822084D0  , -607.91138D0  , -206.24438D0  ,  1676.2319D0  ,
     *  86.375501D0  , -254.54485D0  , -568.17535D0  ,  943.27563D0  ,
     *  669.36923D0  ,  851.69758D0  ,  27.384575D0  , -66.444513D0  ,
     * -79.140068D0  ,  67.127566D0  , -204.55387D0  , -596.23477D0  ,
     * -76.118178D0  , -212.47855D0  ,  53.662951D0  , -240.62353D0  ,
     * -46.489539D0  ,  39.520590D0  , -370.47347D0  , -149.39785D0  ,
     *  1897.7764D0  ,  212.09286D0  , -262.34758D0  , -762.37973D0  ,
     *  1061.1355D0  ,  724.64006D0  ,  465.81606D0  , -15.267786D0  ,
     *  26.024148D0  , -19.186665D0  ,  183.45034D0  , -143.24679D0  ,
     * -608.01903D0  , -35.883648D0  , -169.40933D0  ,  35.113410D0  ,
     * -426.53431D0  , -149.10499D0  ,  62.658793D0  , -621.87964D0  ,
     * -6.3928884D0  ,  2373.7081D0  ,  92.982535D0  , -82.156689D0  ,
     * -964.36573D0  ,  1121.9716D0  ,  740.33337D0  ,  920.55078D0  ,
     *-0.39154359D0  , -8.8822768D0  , -45.841782D0  ,  80.189853D0  ,
     * -97.279282D0  , -430.38857D0  ,  16.752319D0  , -21.800884D0  ,
     *  78.835637D0  , -222.41397D0  , -31.910808D0  ,  55.998459D0  ,
     * -316.86458D0  , -97.596877D0  , -988.66329D0  , -88.531687D0  ,
     *  12.813123D0  , -159.16152D0  , -347.29918D0  , -247.20316D0  ,
     *  537.19663D0  ,  12.964520D0  ,  53.720638D0  , -128.39067D0  ,
     * -41.852226D0  ,  39.415405D0  , -50.614435D0  , -37.201993D0  ,
     *  42.565835D0  ,  19.422873D0  ,  208.19497D0  ,  102.13846D0  ,
     *  7.4315672D0  ,  314.41313D0  , -151.93639D0  , -449.31859D0  ,
     * -8.6423762D0  , -206.75967D0  ,  915.69882D0  , -345.00648D0  ,
     * -161.36385D0  , -625.89136D0  , -27.208018D0  , -84.782112D0  ,
     *  56.368211D0  , -19.522568D0  , -44.502500D0  ,  780.89478D0  ,
     *  61.210380D0  , -47.889991D0  , -63.278656D0  ,  311.31771D0  ,
     * -127.31346D0  , -4.6156762D0  ,  519.29283D0  , -75.570647D0  ,
     * -611.95787D0  , -45.383652D0  ,  385.05162D0  , -1.1786334D0  ,
     * -481.21512D0  , -374.35092D0  , -394.15952D0  ,  3.2438765D0  ,
     *  128.94143D0  ,  42.123911D0  , -58.164040D0  ,  254.96543D0  ,
     *  31.665308D0  ,  14.336831D0  ,  213.24856D0  , -24.776273D0  ,
     * -41.187272D0  ,  126.78179D0  , -45.849804D0  , -42.600682D0  ,
     *  48.287467D0  , -1069.9539D0  , -56.006882D0  , -75.600748D0  ,
     *  657.24072D0  , -503.53005D0  , -319.30797D0  , -106.94351D0  ,
     *  28.266515D0  , -73.568742D0  , -50.742379D0  , -35.911130D0  ,
     * -90.133229D0  ,  234.11186D0  , -45.154205D0  , -104.50833D0  ,
     *  14.506305D0  ,  255.33476D0  ,  32.846310D0  , -12.512339D0  ,
     *  332.50122D0  ,  32.877102D0  , -975.84190D0  , -5.5238290D0  ,
     *  47.943987D0  ,  390.88428D0  , -460.93763D0  , -323.81457D0  ,
     * -590.62076D0  , -11.699011D0  ,  34.413499D0  ,  45.268957D0  ,
     *  14.724945D0  ,  66.822507D0  ,  189.87360D0  ,  1.8328859D0  ,
     *  17.654656D0  , -38.527452D0  ,  28.557558D0  , -31.766343D0  ,
     * -18.139067D0  ,  34.453056D0  ,  141.25647D0  , -781.49228D0  ,
     * -114.21063D0  ,  152.99248D0  ,  306.16453D0  , -477.96714D0  ,
     * -349.12922D0  , -237.30717D0  ,  7.4733664D0  , -8.6682248D0  ,
     *  20.337126D0  , -78.059104D0  ,  65.729272D0  ,  293.58294D0  ,
     *  24.285912D0  ,  87.852207D0  , -9.9784521D0  ,  197.62640D0  ,
     *  59.232202D0  , -31.619550D0  ,  270.67970D0  ,  120.03945D0  ,
     * -1239.8713D0  , -211.53366D0  ,  106.04331D0  ,  1205.9156D0  ,
     * -425.59465D0  , -304.72494D0  , -355.80970D0  ,  28.206260D0  ,
     * -137.88278D0  , -19.990168D0  , -99.741167D0  ,  108.04172D0  ,
     *  509.83826D0  , -23.154102D0  ,  16.159284D0  , -36.090955D0  ,
     *  334.71036D0  ,  13.480783D0  , -55.417668D0  ,  474.51257D0  ,
     *  33.748552D0  , -1512.4720D0  , -44.358183D0  ,  130.23970D0  ,
     *  1306.5833D0  , -816.83076D0  , -537.22929D0  , -1116.6542D0  ,
     * -15.966669D0  , -38.815477D0  ,  73.628171D0  , -4.7370903D0  ,
     *  133.29417D0  ,  765.20960D0  ,  43.546208D0  ,  65.357403D0  ,
     * -51.747880D0  ,  277.27141D0  , -35.066069D0  , -37.933569D0  ,
     *  413.33117D0  ,  223.83057D0  , -2069.7591D0  , -59.779377D0  ,
     *  428.42883D0  ,  578.64921D0  , -1190.3341D0  , -866.73224D0  ,
     * -786.80860D0  ,  7.2461799D0  ,  125.14143D0  ,  19.497124D0  ,
     * -47.604326D0  ,  250.38011D0  ,  352.66425D0  ,  15.613848D0  ,
     *  216.93456D0  , -7.1068543D0  ,  190.15406D0  ,  160.63733D0  ,
     * -50.982616D0  ,  254.95280D0  ,  279.63302D0  , -2566.1350D0  ,
     * -152.33939D0  , -3.2813966D0  ,  1326.5931D0  , -1299.5151D0  ,
     * -860.07391D0  , -567.96050D0  ,  23.705746D0  , -101.24105D0  ,
     * -35.841136D0  , -110.16262D0  , -48.331271D0  ,  783.32137D0  ,
     * -10.824806D0  , -34.363522D0  , -3.9167162D0  ,  596.91441D0  ,
     *  74.994836D0  , -36.970159D0  ,  823.20477D0  ,  126.67412D0  ,
     * -2550.4585D0  , -170.21107D0  ,  309.16020D0  ,  963.63527D0  ,
     * -1317.5256D0  , -915.62501D0  , -1171.7935D0  , -11.985488D0  ,
     *  43.724305D0  ,  96.827542D0  , -123.31275D0  ,  251.41280D0  ,
     *  674.34043D0  ,  43.330829D0  ,  199.00045D0  , -93.467195D0  ,
     *  292.86929D0  ,  65.823833D0  , -73.398503D0  ,  436.88285D0  ,
     *  157.29372D0  , -2735.4453D0  , -203.65228D0  ,  233.03764D0  ,
     *  1106.0569D0  , -1421.3273D0  , -974.75338D0  , -955.46520D0  ,
     *  11.490972D0  , -9.1189979D0  ,  53.107594D0  , -153.59757D0  ,
     *  138.94824D0  ,  659.75450D0  ,  11.020499D0  ,  107.62402D0  ,
     * -69.670796D0  ,  406.59534D0  ,  97.672706D0  , -80.266083D0  ,
     *  563.37599D0  ,  228.72899D0  ,  656.64010D0  , -141.54978D0  ,
     * -15.915640D0  ,  310.96192D0  ,  650.62919D0  ,  401.34201D0  ,
     * -39.770055D0  ,  28.692408D0  , -176.36590D0  ,  62.487646D0  ,
     * -23.566163D0  , -6.1348345D0  , -222.59809D0  , -40.661225D0  ,
     * -103.59647D0  , -27.692213D0  , -183.13756D0  , -92.054085D0  ,
     * -39.716945D0  , -327.60975D0  ,  69.041307D0  ,  1219.1920D0  ,
     *  104.18408D0  , -5.4084372D0  , -514.47276D0  ,  747.75236D0  ,
     *  472.83283D0  , -8.3467961D0  , -12.395795D0  ,  11.560202D0  ,
     *  64.281785D0  ,  134.42826D0  , -25.137459D0  , -533.03678D0  ,
     * -5.8188090D0  , -57.088459D0  ,  18.547149D0  , -441.77460D0  ,
     * -96.825450D0  ,  17.240020D0  , -691.76842D0  ,  264.74377D0  ,
     *  1302.0372D0  ,  133.99073D0  , -5.1074071D0  , -1107.9965D0  ,
     *  656.84115D0  ,  392.66839D0  ,  706.84211D0  , 0.57802506D0  ,
     *  92.844715D0  , -50.025827D0  ,  104.68769D0  , -81.383713D0  ,
     * -652.03344D0  , -9.2735507D0  , -10.714266D0  ,  83.517954D0  ,
     * -314.51945D0  ,  20.674744D0  ,  45.629849D0  , -498.13106D0  ,
     *  160.94771D0  ,  1396.0982D0  ,  18.228963D0  , -371.93074D0  ,
     * -364.46842D0  ,  754.46872D0  ,  557.22932D0  ,  1019.4198D0  ,
     *  5.1169307D0  , -145.74957D0  , -90.174362D0  ,-0.41299552D0  ,
     * -296.03289D0  , -54.513013D0  ,  2.8270670D0  , -167.04331D0  ,
     *  72.859665D0  ,  140.35115D0  , -50.291869D0  ,  59.506179D0  ,
     *  196.91365D0  , -221.37274D0  ,  1681.8806D0  ,  20.608970D0  ,
     *  131.63976D0  , -900.12911D0  ,  843.00952D0  ,  568.65257D0  ,
     *  377.48664D0  , -25.890133D0  ,  62.748246D0  ,  48.134962D0  ,
     * -10.895728D0  ,  160.88251D0  , -322.97381D0  ,  57.473476D0  ,
     *  175.01825D0  , -25.888580D0  , -316.42995D0  , -2.4727534D0  ,
     *  15.867690D0  , -370.96957D0  , -336.06817D0  ,  1227.5772D0  ,
     *  210.35213D0  , -316.11337D0  , -423.05764D0  ,  747.95374D0  ,
     *  561.55358D0  ,  496.53624D0  , -3.1281332D0  , -1.3305618D0  ,
     * -65.152448D0  ,  137.61672D0  , -177.78877D0  , -520.99581D0  ,
     * -61.182452D0  , -210.49264D0  ,  32.996368D0  , -289.57438D0  ,
     * -87.035130D0  ,  55.447545D0  , -410.27431D0  , -179.23546D0  ,
     *  57.271902D0  , -3.1614126D0  , -4.2402151D0  , -102.00960D0  ,
     * -6.4069034D0  ,  4.1626399D0  , 0.11085764D-01, 0.24442371D-01,
     * -1.4296991D0  , -1.5673126D0  , 0.25464023D-02,-0.10305815D-03,
     * 0.68969354D-06,-0.33691538D-03, 0.91769431D-03,  6.7307586D0  ,
     * -2.2051366D0  , 0.38801141D-05, 0.50020353D-05, 0.26978491D-04,
     *-0.87027859D-04,-0.63041822D-02,-0.27773576D-02,-0.13934338D-02,
     *-0.16410856D0/
C
      DATA BZIMF_LOW,BZIMF_HIGH /-10.D0,10.D0/
      DATA PW_LOW,PW_HIGH /-0.6D0,0.6D0/
      DATA XC_LOW,XC_HIGH /-2.0D0,5.0D0/
      DATA RN_LOW,RN_HIGH /4.0D0,10.0D0/
c
        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   !  NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FBZ21=DABS(FBZ) ! /DSQRT(1.D0+A(NLIN+3)*FBZ**2)
        FPW  =(PW*2.D0-PW_HIGH-PW_LOW)/(PW_HIGH-PW_LOW)
        FXC =(XC*2.D0-XC_HIGH-XC_LOW)/(XC_HIGH-XC_LOW)
        FRN =(RN*2.D0-RN_HIGH-RN_LOW)/(RN_HIGH-RN_LOW)
c
        A0=A(NLIN+1)+A(NLIN+6)*FBZ+A(NLIN+7)*FBZ21+A(NLIN+13)*FRN
     *   +A(NLIN+14)*FXC+A(NLIN+15)*FPW
        B0=A(NLIN+2)+A(NLIN+8)*FBZ+A(NLIN+9)*FBZ21+A(NLIN+16)*FRN
     *   +A(NLIN+17)*FXC+A(NLIN+18)*FPW
C
        RMAX_THIN=RMAX0*(A(NLIN+10)+A(NLIN+4)*FBZ+A(NLIN+5)*FBZ21
     *   +A(NLIN+19)*FRN)
        SHIFT1=A(NLIN+11)*20.D0
        SHIFT2=A(NLIN+12)*20.D0
C
        FX=0.D0
        FY=0.D0
        FZ=0.D0
C
        IND=-20
C
        DO 1 I=1,4 
        DO 1 K=1,6 

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         IND=IND+21

         AI=A(IND)
     *     +A(IND+1) *FBZ                !   1
     *     +A(IND+2) *FXC                !   2
     *     +A(IND+3) *FPW                !   3
     *     +A(IND+4) *FBZ21              !   1^2
     *     +A(IND+5) *FXC**2             !   2^2
     *     +A(IND+6) *FPW**2             !   3^2
     *     +A(IND+7) *FBZ*FXC            !   1*2
     *     +A(IND+8) *FXC *FPW           !   2*3
     *     +A(IND+9) *FBZ*FPW            !   1*3
     *     +A(IND+10)*FBZ*FBZ21          !   1^3
     *     +A(IND+11)*FXC **3            !   2^3
     *     +A(IND+12)*FPW**3             !   3^3
     *     +A(IND+13)*FBZ*FXC*FPW        !   1*2*3
     *     +A(IND+14)*FBZ21*FXC          !   1^2*2
     *     +A(IND+15)*FBZ*FXC**2         !   1*2^2
     *     +A(IND+16)*FXC**2*FPW         !   2^2*3
     *     +A(IND+17)*FXC*FPW**2         !   2*3^2
     *     +A(IND+18)*FBZ*FPW**2         !   1*3^2
     *     +A(IND+19)*FBZ21*FPW          !   1^2*3
     *     +A(IND+20)*FRN                !   4    

         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ

  1      CONTINUE
C
         IND=IND+21
         CALL T89_2_DISK_THIN (X-SHIFT1,Y,Z-50.D0,RMAX_THIN,BXN,BYN,BZN)
         CALL T89_2_DISK_THIN (X-SHIFT1,Y,Z+50.D0,RMAX_THIN,BXS,BYS,BZS)

         AI=A(IND)+A(IND+1)*FBZ+A(IND+2)*FBZ21
         FX=FX+AI*(BXN+BXS)
         FY=FY+AI*(BYN+BYS)
         FZ=FZ+AI*(BZN+BZS)

         IND=IND+3
         CALL T89_2_DISK_THIN (X-SHIFT2,Y,Z-70.D0,RMAX_THIN,BXN,BYN,BZN)
         CALL T89_2_DISK_THIN (X-SHIFT2,Y,Z+70.D0,RMAX_THIN,BXS,BYS,BZS)

         AI=A(IND)+A(IND+1)*FBZ+A(IND+2)*FBZ21
         FX=FX+AI*(BXN+BXS)
         FY=FY+AI*(BYN+BYS)
         FZ=FZ+AI*(BZN+BZS)
C
         RETURN  
         END
C
C=======================================================================================
c
      SUBROUTINE T89_2_DISK_THIN (X,Y,Z,RMAX,BX,BY,BZ)
C
C  A REDUCED VERSION OF THE S/R T89_2_DISK, IN WHICH D=0. IT IS USED IN SHIELD FOR AUXILIARY DISKS.
C
C    RMAX:  RADIAL DISTANCE TO THE CURRENT PEAK
C
C  MAGNITUDE OF THE CURRENT IS NORMALIZED SO THAT BZ=1nT AT THE DISK CENTER
C
C     AUTHOR:  N. A. TSYGANENKO, 2015

      IMPLICIT REAL*8 (A-H,O-Z)
C
      R0=RMAX*1.41421356D0
      R02=-R0**2      ! NORMALIZATION FACTOR
      RHO2=X**2+Y**2
      ZR=DABS(Z)+R0
      S=DSQRT(RHO2+ZR**2)
      SZR=S+ZR
      SIGZ=DSIGN(1.D0,Z)
      FXY=SIGZ/S**3*R02
      BX=FXY*X
      BY=FXY*Y
      BZ=-DABS(FXY)/SZR*ZR*(2.D0*S-RHO2/SZR)
C
C  NOTE: THE SIGN AND MAGNITUDE OF THE NORMALIZATION FACTOR IS SUCH THAT BZ=-1nT AT ORIGIN,
C
      RETURN
      END
c
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
c
      SUBROUTINE SRC_UNSH (EPS,SCALE,X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
c
C     AUTHOR:  N. A. TSYGANENKO, 2015

      DATA DT0 /0.07D0/  !  controls the noon-midnight angular stretch (radians)
c              Typical values: between 0.04 and 0.08; here fixed at 0.07
C
      DATA DR0 /0.17D0/       !  controls the magnitude of radial stretch
C                                Tentative trial values: 0.1 and larger  
c    Input:
C
C    EPS    -  controls the noon-midnight asymmetry of the FAC surface, in the distant magnetosphere 
c              Typical values: between 0.1 and 1.0
c
c    SCALE  -  controls the overall size of the system. Normal values around 1.0-1.5. Larger values yield smaller SRC.
c
      Xsc=X*SCALE
      Ysc=Y*SCALE
      Zsc=Z*SCALE
c
c  -----   convert Cartesian into spherical:
c
      RHOsc=DSQRT(Ysc**2+Zsc**2)
      Rsc=DSQRT(Xsc**2+Ysc**2+Zsc**2)
      T=DATAN2(RHOsc,Xsc)
      Ts=T-DT0*Rsc**EPS*RHOsc/Rsc   ! deformed Theta_s=Theta-DT0*R^EPS*sin(Theta), where sin(Theta)=RHOsc/Rsc
      DTsDR=-DT0*EPS*Rsc**(EPS-1.D0)*RHOsc/Rsc  !  derivative dTheta_s/dR
      DTsDT=1.D0-DT0*Rsc**EPS*Xsc/Rsc          !  derivative dTheta_s/dTheta
c
      R_prime=Rsc*(1.d0-DR0*dsin(T/2.d0)**2)
      dR_prime_dR=1.d0-DR0*dsin(T/2.d0)**2
      dR_prime_dT= -DR0*Rsc*0.5d0*DSIN(T)

c
      IF (RHOsc.GT.1.D-5) THEN
         CP=Ysc/RHOsc         !  CP & SP will be needed below to convert from spherical to Cartesian
         SP=Zsc/RHOsc
         STsST=DSIN(Ts)/DSIN(T)
      ELSE
         CP=1.D0
         SP=0.D0
         STsST=DTsDT
      ENDIF
c
      X_s=R_prime*DCOS(Ts)
      Y_s=R_prime*DSIN(Ts)*CP
      Z_s=R_prime*DSIN(Ts)*SP
c
      CALL SRC_AXISYMMETRIC (X_s,Y_s,Z_s,BXas,BYas,BZas)    !  calculate Cartesian B components at primed location
C
      RHO2_s=Y_s**2+Z_s**2
      R_s=DSQRT(RHO2_s+X_s**2)
      RHO_s=DSQRT(RHO2_s)
c
      CT_s=X_s/R_s
      ST_s=RHO_s/R_s
c
      BRas=(X_s*BXas+Y_s*BYas+Z_s*BZas)/R_s               ! convert from Cartesian to spherical
      BTHETAas=(BYas*CP+BZas*SP)*CT_s-BXas*ST_s
      BPHIas=BZas*CP-BYas*SP
c
      BR_s  =STsST*DTsDT*(R_prime/Rsc)**2*BRas 
     * -R_prime/Rsc**2*STsST*dR_prime_dT*BTHETAas                           ! apply deformation tensor

      BTHETA_s=-R_prime**2/Rsc*STsST*DTsDR*BRas
     * +STsST*(R_prime/Rsc)*dR_prime_dR*BTHETAas

      BPHI_s =(R_prime/Rsc)*(dR_prime_dR*dTsDT-dR_prime_dT*dTsDR)*BPHIas
c
      BXsc=BR_s*DCOS(T)-BTHETA_s*DSIN(T)                  ! convert from spherical to Cartesian
      Bmerid =BR_s*DSIN(T)+BTHETA_s*DCOS(T)
      BYsc=Bmerid*CP-BPHI_s*SP
      BZsc=Bmerid*SP+BPHI_s*CP
c
      BX=BXsc    !  here we do not scale the field, so that different values of SCALE yield the same Bz at origin
      BY=BYsc    !   (that is, shrinking/expansion of the SRC keeps Dst-index constant)
      BZ=BZsc
c
      RETURN
      END
c
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE SRC_AXISYMMETRIC (X,Y,Z,BX,BY,BZ)
      IMPLICIT  REAL * 8  (A - H, O - Z)
c
C     AUTHOR:  N. A. TSYGANENKO, 2015
c
      DATA F1,F2,DR,R1,D1,R2,D2,AL,A1,A2,A3,A4,A5,A6,A7,B1,B2,B3,B4,B5,
     * B6,B7 /-42.6031D0,2.16189D0,3.76802D0,5.70915D0,1.98537D0,
     * 2.48923D0,0.607298D0,0.931890D0,0.520056D-1,-0.510551D0,
     * -0.570999D0,0.236973D0,0.166199D0,0.133038D0,0.420191D-1,
     * 1.21158D0,-0.241321D0,-1.51826D0,-0.539575D0,0.144579D0,    
     * 0.257290D0,0.134827D0/    
c
C  F1: first (outer) loop magnitude,F2: second (inner) loop magnitude
C  DR: damping scale radius, R1: outer loop radius, D1: outer loop spread 
C  R2: inner loop radius, D2: inner loop spread, AL: a factor in 
C  cos(fr), sin(fr), cos(fz), sin(fz), cos(2*fr), sin(2*fr), cos(2*fz), sin(2*fz) terms
C  A1 - A7, B1 - B7: Fourier coefficients in the deformation equations for Rho and Z
C
      RHO=DSQRT(X**2+Y**2)
      IF (RHO.LT.1.D-9) RHO=1.D-9    !   TO PREVENT DIVISION BY 0 AT Z-AXIS
C
      RR2=((RHO-R1)**2+Z**2)/DR**2
      DEX=DEXP(-RR2)
      ARGR=AL*(RHO-R1)/DR
      ARGZ=AL*Z/DR
C
      SR=DSIN(ARGR)
      CR=DCOS(ARGR)
      S2R=2.D0*SR*CR
      C2R=CR**2-SR**2
      SZ=DSIN(ARGZ)
      CZ=DCOS(ARGZ)
      S2Z=2.D0*SZ*CZ
      C2Z=CZ**2-SZ**2
C
      SUMRHO=A1+A2*SR+A3*CR+A4*CZ+A5*S2R+A6*C2R+A7*C2Z
      BRACKRHO=(1.D0+DEX*SUMRHO)
      RHO_AST=RHO*BRACKRHO
C
      SUMZ=B1+B2*SR+B3*CR+B4*CZ
     *+B5*S2R+B6*C2R+B7*C2Z
      BRACKZ=(1.D0+DEX*SUMZ)
      Z_AST  =Z  *BRACKZ
C    
      DRHOAST_DRHO=BRACKRHO-RHO*DEX*SUMRHO*2.D0/DR**2*(RHO-R1)
     * +RHO*DEX*(A2*CR-A3*SR+2.D0*A5*C2R-2.D0*A6*S2R)*AL/DR
      DRHOAST_DZ=-RHO*DEX*2.D0*Z/DR**2*SUMRHO-RHO*DEX*(A4*DSIN(ARGZ)
     * +2.D0*A7*S2Z)*AL/DR
      DZAST_DRHO=-Z*2.D0*(RHO-R1)/DR**2*DEX*SUMZ+Z*DEX*(B2*CR-B3*SR
     * +2.D0*B5*C2R-2.D0*B6*S2R)*AL/DR
      DZAST_DZ=BRACKZ-Z*DEX*2.D0*Z/DR**2*SUMZ-Z*DEX*(B4*DSIN(ARGZ)
     * +2.D0*B7*S2Z)*AL/DR
C
      X_AST  =RHO_AST*X/RHO  
      Y_AST  =RHO_AST*Y/RHO
C
C     THE FIRST (OUTER) LOOP:
C     
      CALL SPREAD_LOOP_B (R1,D1,X_AST,Y_AST,Z_AST,BX_AST,BY_AST,BZ_AST)
C
      BRHO_AST=(BX_AST*X_AST+BY_AST*Y_AST)/RHO_AST
C
      BRHO_S=RHO_AST/RHO*(DZAST_DZ*BRHO_AST-DRHOAST_DZ*BZ_AST)
      BPHI_S=0.D0
C
      BX1= BRHO_S*X/RHO
      BY1= BRHO_S*Y/RHO
      BZ1=RHO_AST/RHO*(-DZAST_DRHO*BRHO_AST+DRHOAST_DRHO*BZ_AST)
C
C     THE SECOND (INNER) LOOP:
C     
      CALL SPREAD_LOOP_B (R2,D2,X,Y,Z,BX2,BY2,BZ2)
C      
      BX=F1*BX1+F2*BX2
      BY=F1*BY1+F2*BY2
      BZ=F1*BZ1+F2*BZ2
C
      RETURN 
      END
C
C==========================================================================
C
      SUBROUTINE SPREAD_LOOP_B (R,D,X,Y,Z,BX,BY,BZ)
C
C     AUTHOR:  N. A. TSYGANENKO, 2015
c
C   RETURNS CARTESIAN COMPONENTS OF THE FIELD FROM A SPREAD-OUT CURRENT LOOP
C      
      IMPLICIT REAL*8 (A-H,O-Z)
      DATA A0,A1,A2,A3,A4,B0,B1,B2,B3,B4 
     */-0.3068451808D0,-0.6047641772D-02,0.5689361754D-01,0.184348423D0,
     *  0.7165072182D-01,0.2499990832D0,0.3119753229D0,0.2950446164D0,
     *  0.1672154523D0,0.2133127189D-01/
c
      DATA PI/3.141592653589793D0/
      DATA FACTOR/34.D0/   !  adjusted in order that the model SRC yields Bz = +1 nT at origin
C
      RHO=DSQRT(X**2+Y**2)
    
      IF (RHO.LE.1.D-8) THEN  ! VERY CLOSE TO Z-AXIS: HERE, BY SYMMETRY, SET BX=BY=0. BZ IS CALCULATED  
      BX=0.D0                 ! RETAINING ONLY k^2 AND k^4 TERMS IN THE EXPANSIONS FOR K AND E FOR SMALL k^2 
      BY=0.D0 
      BZ=PI/4.D0*R/DSQRT(R**2+Z**2+D**2)**3 *FACTOR
      RETURN 
      ENDIF

      IF (RHO.LE.1.D-3) THEN
       SCALE=1.D-3/RHO
      ELSE
       SCALE=1.D0
      ENDIF

      X1=X*SCALE
      Y1=Y*SCALE
      RH1=RHO*SCALE

      SQB=(R+RH1)**2+Z**2+D**2
      P=1.D0-4.D0*R*RH1/SQB
      F=A0+P*(A1+P*(A2+P*(A3+P*A4)))
     *  -DLOG(P)*(B0+P*(B1+P*(B2+P*(B3+P*B4))))
      FS=A1+P*(2.D0*A2+P*(3.D0*A3+P*4.D0*A4))
     * -(B0+P*(B1+P*(B2+P*(B3+P*B4))))/P
     * -DLOG(P)*(B1+P*(2.D0*B2+P*(3.D0*B3+P*4.D0*B4)))

      BRHO=Z/DSQRT(SQB)**3*(F-FS/SQB*8.D0*R*RH1) *FACTOR
      BZ=(F/RH1-(F*(R+RH1)+4.D0*R/SQB*FS*(R**2-RH1**2+Z**2+D**2))/SQB)
     * /DSQRT(SQB) *FACTOR

      BX=BRHO/RH1*X1/SCALE
      BY=BRHO/RH1*Y1/SCALE

      RETURN
      END
C
C=======================================================================================
C
      SUBROUTINE SRC_SHLD (X,Y,Z,BZIMF,EPS,SCALE,FX,FY,FZ)
C
C   CALCULATES: (1) COMPONENTS OF THE TOTAL SHIELDING FIELD VECTOR AT ANY GIVEN POSITION X,Y,Z
C               (2) SCALAR PRODUCTS OF THE 'PARTIAL' FIELDS, CORRESPONDING TO EACH LINEAR TERM
C                     IN THE SHIELDING FIELD EXPANSION (WITHOUT THE COEFFICIENTS), AND A GIVEN
C                     UNIT NORMAL VECTOR {XNNX,XNNY,XNNZ}
C
C   THE OUTPUT VECTOR {FX,FY,FZ} IS USED FOR CHECKING THE MODEL (E.G., LINE TRACING),
C     WHILE THE NORMAL FIELDS ARE USED IN THIS FITTING CODE, WHICH MINIMIZES THE RMS <Bn^2> OVER
C       THE MODEL MAGNETOPAUSE
C
C      INPUT:  X,Y,Z           - POSITION VECTOR (KSM FOR SATURN OR GSM FOR EARTH)
C              BZIMF           - IMF BZ 
C              EPS             - EXPONENT OF R IN THE RADIAL STRETCH FUNCTION
C              SCALE           - OVERALL SCALE OF THE MODEL RING CURRENT       
C              (variation ranges of each of the above 4 parameters are specified in DATA statements below)
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C      AUTHOR:  N. A. TSYGANENKO, 2015
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=400,NTOT=410)
        DIMENSION A(NTOT)
        DATA A/
     *  4.0567599D0  , -3.2150442D0  , -11.606128D0  ,  4.0569091D0  ,
     * 0.88345389D0  ,  16.920233D0  , 0.43084505D0  ,  2.1011254D0  ,
     * -1.6231375D0  ,  1.0988260D0  , 0.85336119D-01,  2.9336396D0  ,
     *-0.92975675D-01, -1.0484057D0  ,  1.0059430D0  ,  3.0636431D0  ,
     * -2.7410025D0  ,-0.39395174D0  ,  1.4484220D0  , -1.8025719D0  ,
     *  10.972415D0  ,  5.8446122D0  ,  4.5560792D0  , -4.1708999D0  ,
     * -7.3121544D0  , -23.631832D0  , -1.5022734D0  , -7.4707736D0  ,
     *  8.4421859D0  ,-0.84427471D0  ,  1.2802786D0  , -17.772983D0  ,
     * -1.8669809D0  ,  2.9385014D0  ,  3.5599169D0  , -5.6309138D0  ,
     *  3.1069949D0  ,  1.4914055D0  , -1.6342478D0  ,-0.34031158D0  ,
     * -11.705016D0  , -3.5666562D0  ,  4.5062122D0  ,  2.7603249D0  ,
     *  10.828069D0  ,  21.902930D0  , 0.65213025D0  ,  6.1327050D0  ,
     * -11.520702D0  , 0.59635250D0  ,-0.73763942D0  ,  8.6550318D0  ,
     * 0.42447093D0  , -1.7995500D0  , -2.2679181D0  ,-0.78533404D0  ,
     *  3.7468857D0  ,  1.0407885D0  , 0.63357353D0  , -2.6649078D0  ,
     *  8.1375129D0  ,-0.68600378D0  , -5.3122802D0  , -3.7207562D0  ,
     * -7.0787620D0  , -6.5418816D0  , 0.61388770D-01,  5.8818968D0  ,
     *  7.3533205D0  , 0.19014250D-01,  3.0419410D0  , 0.48937089D0  ,
     *  2.9654333D0  ,-0.94256417D-01, -2.6949448D0  , 0.14858587D0  ,
     * -7.5179795D0  , -2.1637333D0  ,  1.0933315D0  ,  5.4998983D0  ,
     * -2.2107347D0  ,  1.4116049D0  , 0.86824461D0  ,  1.8276170D0  ,
     * 0.63904086D0  , -1.6169895D0  ,-0.29275043D0  , -6.0782656D0  ,
     * -1.8578759D0  ,-0.51020452D0  , -3.4419646D0  ,  1.4927971D0  ,
     * -2.1392062D0  , 0.80015595D0  ,  2.6172124D0  ,  2.5821779D0  ,
     *  3.3779296D0  , 0.46641773D0  , -1.0946874D0  , -2.5011705D0  ,
     * -14.756312D0  ,  11.934882D0  ,  4.7870385D0  , -11.681704D0  ,
     *  10.304370D0  ,  11.709727D0  , -1.9771252D0  ,  7.8470299D0  ,
     * -5.4399217D0  ,-0.66877010D0  , 0.23515478D0  ,  25.329813D0  ,
     *-0.44374727D0  ,-0.69327899D0  ,  7.9093064D0  ,  10.811746D0  ,
     *  1.2980183D0  ,  1.0929477D0  , -3.0743639D0  ,  2.8471951D0  ,
     *  3.2502413D0  , -15.654648D0  , -11.240328D0  ,  7.2597118D0  ,
     * -3.9779115D0  , -3.1840833D0  , 0.78406032D0  ,  2.3224440D0  ,
     * -2.4925237D0  ,  1.4939157D0  , -5.5929478D0  , -3.6208797D0  ,
     *  6.2955249D0  , -1.6672707D0  , -17.425847D0  , -6.1414547D0  ,
     * -5.4550079D0  , -3.7407740D0  ,  2.1844746D0  ,  5.8570740D0  ,
     *  9.0048780D0  ,  9.4118751D0  , -2.4111646D0  , -3.4503672D0  ,
     * -9.0898024D0  , -6.1527829D0  ,  2.2164474D0  , -9.5623747D0  ,
     *  13.718593D0  , -3.0919839D0  ,  2.3986870D0  ,  2.7803577D0  ,
     * -4.7014664D0  ,-0.40013649D0  ,  11.137541D0  ,  9.2950202D0  ,
     * -3.3880186D0  ,-0.51536182D0  , 0.26081261D0  ,-0.70735412D0  ,
     * -9.4535407D0  , 0.16020783D0  ,  8.2856224D0  ,  5.5799290D0  ,
     *  8.4563940D0  , -2.5641790D0  , -2.8087744D0  , -8.8316198D0  ,
     * -12.041722D0  ,  2.0510702D0  , -4.0342184D0  , -7.1767567D0  ,
     * -3.6114171D0  ,  1.8897827D0  ,  3.3653286D0  ,-0.96926041D0  ,
     *  10.494299D0  ,  2.9840327D0  , -2.9551398D0  , -8.4438220D0  ,
     *  3.0095613D0  , -2.6626326D0  , -1.1870524D0  , -3.2795824D0  ,
     *-0.14309899D0  ,  6.5783854D0  ,  1.3267524D0  ,  10.883429D0  ,
     *  3.4782674D0  , 0.31784490D0  ,  6.0313396D0  , -2.4857622D0  ,
     *  3.7758980D0  , -1.8269256D0  , -5.3219269D0  , -5.7333992D0  ,
     * -5.3321527D0  ,-0.45541208D0  ,  2.2989822D0  ,  4.7053565D0  ,
     *  27.206616D0  , -5.4373795D0  ,  14.985909D0  ,  10.568373D0  ,
     * -15.067481D0  ,  22.167147D0  ,  2.1077573D0  ,  1.2813168D0  ,
     *  17.445967D0  , -4.5298853D0  , -3.8749488D0  , -21.857153D0  ,
     *-0.33539384D0  ,  2.1862486D0  , -1.2261397D0  , -10.763960D0  ,
     *  4.9033306D0  , 0.33745450D0  , 0.89187807D0  ,-0.87146227D0  ,
     * -21.344961D0  ,  6.7876961D0  ,  4.8441510D0  , -4.1353252D0  ,
     *  13.148079D0  , -19.310842D0  , 0.23430889D0  , -10.670645D0  ,
     * -16.856733D0  ,  4.1926396D0  ,  10.903613D0  ,  6.4191176D0  ,
     * -6.0338409D0  , -1.6446804D0  ,  6.2580204D0  ,  7.0709549D0  ,
     * 0.48976456D0  ,  1.8302092D0  ,  1.7804860D0  , -11.791449D0  ,
     *  3.5548148D0  , -4.5864163D0  , -12.271088D0  , -2.1164685D0  ,
     *-0.21297715D0  ,  3.2345289D0  , -4.1238710D0  ,  18.888733D0  ,
     *  2.9010470D0  , 0.52596943D0  , -5.0522649D0  , -2.2022351D0  ,
     *  7.1542968D0  ,  3.0571825D0  , -4.7109594D0  , -5.5241830D0  ,
     * -1.2295989D0  ,-0.20839160D-01, -3.5810073D0  ,  10.262012D0  ,
     *  3.3730524D0  ,-0.35300787D0  ,  5.5544007D0  ,-0.49215171D0  ,
     * -3.0484051D0  ,  8.8406339D0  ,  4.0829543D0  , -3.6085812D0  ,
     *  4.1223855D0  , -2.3146242D0  ,  1.4168210D0  ,  4.8755271D0  ,
     *-0.35984407D0  , -2.4964330D0  , -3.8513665D0  , -1.5564510D0  ,
     * -1.9459387D0  , -1.3577985D0  ,  3.5000945D0  ,  1.0252986D0  ,
     * -1.7508598D0  ,  1.7307071D0  , -2.1822014D0  ,  1.4139510D0  ,
     *-0.26492313D0  , -6.9929332D0  , -1.4905007D0  , -4.5523653D0  ,
     * -1.8904264D0  , 0.37805394D0  , -3.0843898D0  ,  1.4855542D0  ,
     * -1.8286614D0  ,  1.3366694D0  ,  4.1064140D0  ,  4.5006266D0  ,
     *  1.7839179D0  , 0.65352770D-01, -1.7194005D0  , -2.3763683D0  ,
     * -5.3105012D0  ,  2.9516056D0  ,  7.6944122D0  ,-0.60640483D0  ,
     *-0.90663647D0  , -18.197919D0  ,-0.22268430D0  , -3.4400329D0  ,
     *  6.9501785D0  ,  7.7956870D0  , 0.37390682D-02,  10.998891D0  ,
     *  2.9412122D0  ,  2.0622571D0  , -15.363395D0  ,  4.1556472D0  ,
     *  7.0323148D0  ,  1.0251932D0  , 0.93672411D0  ,  2.1684778D0  ,
     * 0.85332394D0  , -4.2812941D0  , -25.915980D0  , -3.4342963D0  ,
     *  3.2791777D0  ,  11.406014D0  ,-0.65809084D0  ,  4.2865970D0  ,
     * -8.2890036D0  , -9.3558858D0  , -2.9382501D0  , -8.5033111D0  ,
     *-0.99220017D0  , -2.8478891D0  ,  17.505131D0  , -5.2939784D0  ,
     * -10.078519D0  , -1.4674728D0  , -2.6179193D0  ,  3.5409029D0  ,
     *  4.3257668D0  ,  3.5148688D0  ,  28.178136D0  ,  5.2449612D0  ,
     * -5.0636617D0  ,  2.0006635D0  ,  1.8238269D0  , -5.6780770D0  ,
     *  6.2723766D0  ,  4.9277323D0  ,  1.7120995D0  ,  4.4704590D0  ,
     * -1.2675444D0  ,  1.1493339D0  , -10.753774D0  ,  4.3761007D0  ,
     *  7.2677427D0  , 0.36360394D0  ,  2.8309246D0  , -5.3059011D0  ,
     * -3.5536583D0  , -1.0870696D0  , -13.639178D0  , -2.3551396D0  ,
     *  2.8576233D0  , -5.5331230D0  , -1.5627113D0  ,  2.3258584D0  ,
     * -3.3010578D0  ,-0.84873245D0  , 0.80297553D-02, -2.2476818D0  ,
     * 0.46460275D0  , 0.31989059D-01,  5.2853662D0  ,-0.61580318D0  ,
     * -2.8430810D0  , 0.26927965D0  , -1.6725977D0  ,  1.4794579D0  ,
     * 0.98257246D0  ,-0.15173515D0  ,  2.8783568D0  , 0.19381363D0  ,
     *-0.34831400D0  ,  2.4890052D0  , 0.51564540D0  , 0.47667340D0  ,
     * 0.84665044D0  ,-0.29806413D-02, 0.45702139D0  ,-0.16992506D0  ,
     * 0.27108456D0  ,-0.22027725D0  , -1.6750958D0  ,-0.92050932D0  ,
     * 0.36256877D0  ,-0.41920808D-02, 0.52454048D0  , 0.22281138D0  ,
     * 0.39193071D-01, 0.33185478D-01,-0.30239314D-02,-0.43899657D-02,
     *-0.15373169D-02, 0.90361619D-02, 0.15367701D-03,-0.20585023D-01,
     *-0.29039688D-02,-0.29995008D-02/
C
      DATA BZIMF_LOW,BZIMF_HIGH /-10.D0,10.D0/
      DATA SCALE_LOW,SCALE_HIGH /1.D0,2.D0/
      DATA EPS_LOW,EPS_HIGH / 0.5D0,0.8D0/

        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   !  NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FBZ21=DABS(FBZ) 
        FSCA  =(SCALE*2.D0-SCALE_HIGH-SCALE_LOW)/(SCALE_HIGH-SCALE_LOW)
        FEPS =(EPS*2.D0-EPS_HIGH-EPS_LOW)/(EPS_HIGH-EPS_LOW)
C
        A0=A(NLIN+1)+A(NLIN+3)*FBZ+A(NLIN+5)*FBZ21+A(NLIN+7)*FSCA
     *  +A(NLIN+9)*FEPS  
        B0=A(NLIN+2)+A(NLIN+4)*FBZ+A(NLIN+6)*FBZ21+A(NLIN+8)*FSCA
     *   +A(NLIN+10)*FEPS                                        
C
        FX=0.D0
        FY=0.D0
        FZ=0.D0
C
        IND=1
C
        DO 1 I=1,4 
        DO 1 K=1,5

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FSCA
     *     +A(IND+3) *FEPS  
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FSCA**2
     *     +A(IND+6) *FEPS**2
     *     +A(IND+7) *FBZ*FSCA
     *     +A(IND+8) *FSCA*FEPS
     *     +A(IND+9) *FBZ*FEPS
     *     +A(IND+10)*FBZ*FBZ21
     *     +A(IND+11)*FSCA**3
     *     +A(IND+12)*FEPS**3
     *     +A(IND+13)*FBZ*FSCA*FEPS
     *     +A(IND+14)*FBZ21*FSCA
     *     +A(IND+15)*FBZ*FSCA**2
     *     +A(IND+16)*FSCA**2*FEPS
     *     +A(IND+17)*FSCA*FEPS**2
     *     +A(IND+18)*FBZ*FEPS**2
     *     +A(IND+19)*FBZ21*FEPS
C
         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ

         IND=IND+20

  1      CONTINUE
C
        RETURN   
        END
C
C=====================================================================================
c
      SUBROUTINE PRC_UNSH_NM (EPS,SCALE,X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
c
C      AUTHOR:  N. A. TSYGANENKO, 2015
c
      DATA DT0 /0.07D0/  !  controls the noon-midnight angular stretch (radians)
c                           typical values: between 0.04 and 0.08; here fixed at 0.07
      DATA DR0 /0.17D0/  !  controls the magnitude of radial stretch 
c                           typical values: 0.1 and larger; here fixed at 0.17
c     Input:
C
C     EPS    -  controls the noon-midnight asymmetry of the FAC surface, in the distant magnetosphere 
c              Typical values: between 0.1 and 1.0
c
c     SCALE  -  controls the overall size of the system. Normal values around 1.0-1.5. Larger values yield smaller SRC.
c
      Xsc=X*SCALE
      Ysc=Y*SCALE
      Zsc=Z*SCALE
c
c -----  convert Cartesian into spherical:
c
      RHOsc=DSQRT(Ysc**2+Zsc**2)
      Rsc=DSQRT(Xsc**2+Ysc**2+Zsc**2)
      T=DATAN2(RHOsc,Xsc)
      Ts=T-DT0*Rsc**EPS*RHOsc/Rsc   ! deformed Theta_s=Theta-DT0*R^EPS*sin(Theta), where sin(Theta)=RHOsc/Rsc
      DTsDR=-DT0*EPS*Rsc**(EPS-1.D0)*RHOsc/Rsc  !  derivative dTheta_s/dR
      DTsDT=1.D0-DT0*Rsc**EPS*Xsc/Rsc          !  derivative dTheta_s/dTheta
c
      R_prime=Rsc*(1.d0-DR0*dsin(T/2.d0)**2)
      dR_prime_dR=1.d0-DR0*dsin(T/2.d0)**2
      dR_prime_dT= -DR0*Rsc*0.5d0*DSIN(T)

c
      IF (RHOsc.GT.1.D-5) THEN
         CP=Ysc/RHOsc         !  CP & SP will be needed below to convert from spherical to Cartesian
         SP=Zsc/RHOsc
         STsST=DSIN(Ts)/DSIN(T)
      ELSE
         CP=1.D0
         SP=0.D0
         STsST=DTsDT
      ENDIF
c
      X_s=R_prime*DCOS(Ts)
      Y_s=R_prime*DSIN(Ts)*CP
      Z_s=R_prime*DSIN(Ts)*SP
c
      CALL PRC_NM_UNDEFORMED (X_s,Y_s,Z_s,BXas,BYas,BZas)    !  calculate Cartesian B components at primed location
C
      RHO2_s=Y_s**2+Z_s**2
      R_s=DSQRT(RHO2_s+X_s**2)
      RHO_s=DSQRT(RHO2_s)
c
      CT_s=X_s/R_s
      ST_s=RHO_s/R_s
c
      BRas=(X_s*BXas+Y_s*BYas+Z_s*BZas)/R_s               ! convert from Cartesian to spherical
      BTHETAas=(BYas*CP+BZas*SP)*CT_s-BXas*ST_s
      BPHIas=BZas*CP-BYas*SP
c
      BR_s  =STsST*DTsDT*(R_prime/Rsc)**2*BRas 
     * -R_prime/Rsc**2*STsST*dR_prime_dT*BTHETAas                           ! apply deformation tensor

      BTHETA_s=-R_prime**2/Rsc*STsST*DTsDR*BRas
     * +STsST*(R_prime/Rsc)*dR_prime_dR*BTHETAas

      BPHI_s =(R_prime/Rsc)*(dR_prime_dR*dTsDT-dR_prime_dT*dTsDR)*BPHIas
c
      BXsc=BR_s*DCOS(T)-BTHETA_s*DSIN(T)                  ! convert from spherical to Cartesian
      Bmerid =BR_s*DSIN(T)+BTHETA_s*DCOS(T)
      BYsc=Bmerid*CP-BPHI_s*SP
      BZsc=Bmerid*SP+BPHI_s*CP
c
      BX=BXsc    !  here we do not scale the field, so that different values of SCALE yield the same Bz at origin
      BY=BYsc    !   (that is, shrinking/expansion of the SRC keeps Dst-index constant)
      BZ=BZsc
c
      RETURN
      END
c
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE PRC_NM_UNDEFORMED (X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
      CALL SPHCAR (R,T,P,X,Y,Z,-1)
      CALL BRBTBP_PRC (R,T,P,BR,BT,BP)
      CALL BSPCAR (T,P,BR,BT,BP,BX,BY,BZ)
      RETURN
      END
C
C#########################################################################
C
      SUBROUTINE BRBTBP_PRC (R,T,P,BR,BT,BP) 
      IMPLICIT REAL*8 (A-H,O-Z)
C
      DATA  ABR01,ABR02,ABR03,ABR04,ABR05,ABR06,ABR07,ABR08,ABR09,ABR10,
     *ABR11,ABR12,ABR13,ABR14,ABR15,ABR16,ABR17,ABR18,ABR19,ABR20,ABR21,
     *ABR22,ABR23,ABR24,ABR25,ABR26,ABR27,ABR28,ABR29,ABR30,ABR31,ABR32,
     *ABR33,ABR34,ABR35,ABR36,ABR37,ABR38,ABR39,ABR40,ABR41,ABR42,ABR43,
     *ABR44,ABR45,ABR46,ABR47,ABR48,ABR49,ABR50,ABR51,ABR52,ABR53,ABR54,
     *ABR55,ABR56/18.83023014D0,468934.6659D0,-468945.9056D0,
     *0.9310123492D-03,341.5640336D0,-341.5724899D0,0.2834275159D-01,
     *2815.457002D0,-2815.394238D0,4273.405633D0,142912593.7D0,    
     *-142912703.7D0,-.3144179212D-08,-.9652596615D-05,0.9661074043D-05,
     *0.2089306536D-02,-14080.28783D0,14080.43837D0,0.1113334263D0,    
     *-0.1062192861D0,0.3149232975D0,11.36515559D0,4.723154529D0,    
     *3.984559924D0,4.585739484D0,4.032706815D0,3.741667937D0,    
     *0.5798779413D-01,2.417509610D0,8.492134117D0,7.761662895D0,    
     *7.389416256D0,7.460193567D0,1.110098139D0,1.110074621D0,    
     *2.821936969D0,9.037995133D0,5.282888651D0,6.262172568D0,    
     *0.8249294900D0,1.195457593D0,2.469064281D0,5.071438197D0,    
     *4.822507104D0,0.1568808225D0,0.1079224544D0,0.1616535726D0,    
     *2.078602607D0,0.3735552854D0,4.279143730D0,1.863086547D0,    
     *1.080625720D0,1.104468013D0,6.456298650D0,1.970461884D0,    
     *1.974588436D0/    
      DATA  ABT01,ABT02,ABT03,ABT04,ABT05,ABT06,ABT07,ABT08,ABT09,ABT10,
     *ABT11,ABT12,ABT13,ABT14,ABT15,ABT16,ABT17,ABT18,ABT19,ABT20,ABT21,
     *ABT22,ABT23,ABT24,ABT25,ABT26,ABT27,ABT28,ABT29,ABT30,ABT31,ABT32,
     *ABT33,ABT34,ABT35,ABT36,ABT37,ABT38,ABT39,ABT40,ABT41,ABT42,ABT43,
     *ABT44,ABT45,ABT46,ABT47,ABT48,ABT49,ABT50,ABT51,ABT52,ABT53,ABT54,
     *ABT55,ABT56/3.356791597D0,-5.883419687D0,-2.058543379D0,   
     *.1100303924D-03,-.1943189958D-02,.2793918244D-02,0.2049870425D-03,
     *0.5220714000D-03,0.1019940452D-01,-75.87370859D0,-47.96110111D0,    
     *24.36085698D0,-.5362635518D-05,0.1925145540D-04,-0.3548180534D-05,
     *-.3037668516D-03,.7799787312D-03,-.1411180236D-01,.6526098636D-01,
     *-0.6165246724D-01,0.7321218781D0,11.63639868D0,4.570394366D0,    
     *5.991114966D0,7.610650545D0,4.318030252D0,4.599681445D0,    
     *0.3488975773D0,1.700478186D0,5.759626972D0,1.666470246D0,    
     *7.067086748D0,7.451917780D0,0.4817635507D0,2.681223071D0,    
     *4.434829894D0,5.173486046D0,0.4858312077D0,5.763669434D0,    
     *3.765430495D0,1.565144414D0,2.389620597D0, 6.040651474D0,    
     *5.200117858D0,0.1443594185D0,0.1078706697D0,0.1655199312D0,    
     *1.108410791D0,0.2563543278D0,1.647886458D0,2.002342423D0,    
     *1.537733423D0,1.010006109D0,2.360742095D0,1.907047647D0,    
     *3.557905953D0/    
C
      DATA KEEP/0/

      SAVE KEEP

      IF (KEEP.EQ.0) THEN
       KEEP=1
       ABR48=ABR48*ABR36
       ABR49=ABR49*ABR36
       ABR50=ABR50*ABR36
       ABT48=ABT48*ABT36
       ABT49=ABT49*ABT36
       ABT50=ABT50*ABT36
      ENDIF

      ST=DSIN(T)
      CT=DCOS(T)
      ST2=ST**2
      IF (ST2.LT.1.D-10) ST2=1.D-10  !  to avoid crashes at Z axis
      CT2=CT**2
      SP=DSIN(P)
      CP=DCOS(P)
c
      ALPHA=ST2/R
      IF (ALPHA.GE.1.D0) ALPHA=0.9999D0  !  to avoid crashes if inside Earth
      DAL_DR=-ALPHA/R
      DAL_DT=2.D0*ST*CT/R
C
C=======================================================================
C
C  CALCULATE BR:
C
      DALPHA=ABR19+ABR20*(R/10.D0)**ABR21
      DDAL_DR=ABR20/10.D0*ABR21*(R/10.D0)**(ABR21-1.D0)
C
      BR40=((R/ABR39)**2*CT2)
      D1=(R/ABR22)**ABR23+1.D0+BR40**ABR51

      BR41=((R/ABR40)**2*CT2)
      D2=(R/ABR24)**ABR25+1.D0+BR41**ABR52

      BR42=((R/ABR41)**2*CT2)
      D3=(R/ABR26)**ABR27+1.D0+BR42**ABR53

      BR43=((R/ABR42)**2*CT2)
      D4=(R/ABR28)**ABR29+1.D0+BR43**ABR54

      BR44=((R/ABR43)**2*CT2)
      D5=(R/ABR30)**ABR31+1.D0+BR44**ABR55

      BR45=((R/ABR44)**2*CT2)
      D6=(R/ABR32)**ABR33+1.D0+BR45**ABR56
C
      STA=(ST2**2)**ABR34
      STB=(ST2**2)**ABR35
C
C-----------------------------------------------------
      S1=ABR01+ABR02*STA+ABR03*STB
      DR1=1.D0/D1
      DR2=DR1*STA
      DR3=DR1*STB
C-----------------------------------------------------
      ALPHA01=ABR45
      SP46=DSQRT((ALPHA+ALPHA01)**2+DALPHA**2)
      SM46=DSQRT((ALPHA-ALPHA01)**2+DALPHA**2)
      F1_46=2.D0/(SP46+SM46)
      DF1_46_DAL=-F1_46/(SP46+SM46)*((ALPHA-ALPHA01)/SM46
     * +(ALPHA+ALPHA01)/SP46)
      DF1_46_DDA=-F1_46/(SP46+SM46)*DALPHA*(1.D0/SM46+1.D0/SP46)
C
      S2=ABR04+ABR05*STA+ABR06*STB
      DR4=F1_46**ABR48/D2       
      DR5=DR4*STA
      DR6=DR4*STB
C-----------------------------------------------------
      ALPHA02=ABR46
      SP47=DSQRT((ALPHA+ALPHA02)**2+DALPHA**2)
      SM47=DSQRT((ALPHA-ALPHA02)**2+DALPHA**2)
      F2_47=2.D0/(SP47+SM47)
      DF2_47_DAL=-F2_47/(SP47+SM47)*((ALPHA-ALPHA02)/SM47
     * +(ALPHA+ALPHA02)/SP47)
      DF2_47_DDA=-F2_47/(SP47+SM47)*DALPHA*(1.D0/SM47+1.D0/SP47)
C
      S3=ABR07+ABR08*STA+ABR09*STB
      DR7=F2_47**ABR37*ALPHA**ABR38/D3
      DR8=DR7*STA
      DR9=DR7*STB
C-----------------------------------------------------
      S4=ABR10+ABR11*STA+ABR12*STB
      DR10=F1_46**ABR49/D4  
      DR11=DR10*STA
      DR12=DR10*STB
C-----------------------------------------------------
      S5=ABR13+ABR14*STA+ABR15*STB
      DR13=F1_46**ABR50/D5  
      DR14=DR13*STA
      DR15=DR13*STB
C----------------------------------------------------
      ALPHA03=ABR47
      SP48=DSQRT((ALPHA+ALPHA03)**2+DALPHA**2)
      SM48=DSQRT((ALPHA-ALPHA03)**2+DALPHA**2)
      F3_48=2.D0/(SP48+SM48)
      DF3_48_DAL=-F3_48/(SP48+SM48)*((ALPHA-ALPHA03)/SM48
     * +(ALPHA+ALPHA03)/SP48)
      DF3_48_DDA=-F3_48/(SP48+SM48)*DALPHA*(1.D0/SM48+1.D0/SP48)    
C
      S6=ABR16+ABR17*STA+ABR18*STB
      DR16=F3_48**ABR36*(1.D0-ALPHA)**ABR38/D6
      DR17=DR16*STA
      DR18=DR16*STB
C----------------------------------------------------
      betar = ABR01*DR1+ABR02*DR2+ABR03*DR3+ABR04*DR4+ABR05*DR5+
     *ABR06*DR6+ABR07*DR7+ABR08*DR8+ABR09*DR9+ABR10*DR10+
     *ABR11*DR11+ABR12*DR12+ABR13*DR13+ABR14*DR14+ABR15*DR15+
     *ABR16*DR16+ABR17*DR17+ABR18*DR18             
C
      BR=betar*ST*CT*CP
C
C-------  NOW CALCULATE DERIVATIVE D_betar_DR: ------------------------- 
C
      D_betar_DR=-S1/D1**2*(ABR23/ABR22*(R/ABR22)**(ABR23-1.D0)+
     *ABR51/ABR39**2*BR40**(ABR51-1.D0)*CT2*2.D0*R)
     *           -S2/D2**2*(ABR25/ABR24*(R/ABR24)**(ABR25-1.D0)+
     *ABR52/ABR40**2*BR41**(ABR52-1.D0)*CT2*2.D0*R)*F1_46**ABR48
     *           -S3/D3**2*(ABR27/ABR26*(R/ABR26)**(ABR27-1.D0)+
     *ABR53/ABR41**2*BR42**(ABR53-1.D0)*CT2*2.D0*R)*F2_47**ABR37
     * *ALPHA**ABR38
     *           -S4/D4**2*(ABR29/ABR28*(R/ABR28)**(ABR29-1.D0)+
     *ABR54/ABR42**2*BR43**(ABR54-1.D0)*CT2*2.D0*R)*F1_46**ABR49
     *           -S5/D5**2*(ABR31/ABR30*(R/ABR30)**(ABR31-1.D0)+
     *ABR55/ABR43**2*BR44**(ABR55-1.D0)*CT2*2.D0*R)*F1_46**ABR50
     *           -S6/D6**2*(ABR33/ABR32*(R/ABR32)**(ABR33-1.D0)+
     *ABR56/ABR44**2*BR45**(ABR56-1.D0)*CT2*2.D0*R)*F3_48**ABR36
     * *(1.D0-ALPHA)**ABR38
     * +S2/D2*ABR48*F1_46**(ABR48-1.D0)*(DF1_46_DAL*DAL_DR
     * +DF1_46_DDA*DDAL_DR)+
     * +S3/D3*(ABR37*F2_47**(ABR37-1.D0)*(DF2_47_DAL*DAL_DR
     * +DF2_47_DDA*DDAL_DR)*ALPHA**ABR38
     * +F2_47**ABR37*ABR38*ALPHA**(ABR38-1.D0)*DAL_DR)
     * +S4/D4*ABR49*F1_46**(ABR49-1.D0)*(DF1_46_DAL*DAL_DR
     * +DF1_46_DDA*DDAL_DR)+
     * +S5/D5*ABR50*F1_46**(ABR50-1.D0)*(DF1_46_DAL*DAL_DR           
     * +DF1_46_DDA*DDAL_DR)+
     * +S6/D6*(ABR36*F3_48**(ABR36-1.D0)*(DF3_48_DAL*DAL_DR
     * +DF3_48_DDA*DDAL_DR)*(1.D0-ALPHA)**ABR38
     * -F3_48**ABR36*ABR38*(1.D0-ALPHA)**(ABR38-1.D0)*DAL_DR)

C=======================================================================
C
C  NOW CALCULATE BTHETA:
C
      DALPHA=ABT19+ABT20*(R/10.D0)**ABT21
      DDAL_DR=ABT20/10.D0*ABT21*(R/10.D0)**(ABT21-1.D0)
C
      D1=(R/ABT22)**ABT23+1.D0+((R/ABT39)**2*CT2)**ABT51
      D2=(R/ABT24)**ABT25+1.D0+((R/ABT40)**2*CT2)**ABT52
      D3=(R/ABT26)**ABT27+1.D0+((R/ABT41)**2*CT2)**ABT53
      D4=(R/ABT28)**ABT29+1.D0+((R/ABT42)**2*CT2)**ABT54
      D5=(R/ABT30)**ABT31+1.D0+((R/ABT43)**2*CT2)**ABT55
      D6=(R/ABT32)**ABT33+1.D0+((R/ABT44)**2*CT2)**ABT56
C
      STA=(ST2**2)**ABT34
      STB=(ST2**2)**ABT35
C
C-----------------------------------------------------
      S1=ABT01+ABT02*STA+ABT03*STB
      DS1DT=4.D0*ST*CT*(ABT02*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT03*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR1=1.D0/D1
      DR2=DR1*STA
      DR3=DR1*STB
C-----------------------------------------------------
      ALPHA01=ABT45
      SP46=DSQRT((ALPHA+ALPHA01)**2+DALPHA**2)
      SM46=DSQRT((ALPHA-ALPHA01)**2+DALPHA**2)
      F1_46=2.D0/(SP46+SM46)
      DF1_46_DAL=-F1_46/(SP46+SM46)*((ALPHA-ALPHA01)/SM46
     * +(ALPHA+ALPHA01)/SP46)
      DF1_46_DDA=-F1_46/(SP46+SM46)*DALPHA*(1.D0/SM46+1.D0/SP46)
c
      S2=ABT04+ABT05*STA+ABT06*STB
      DS2DT=4.D0*ST*CT*(ABT05*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT06*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR4=F1_46**ABT48/D2       
      DR5=DR4*STA
      DR6=DR4*STB
C-----------------------------------------------------
      ALPHA02=ABT46
      SP47=DSQRT((ALPHA+ALPHA02)**2+DALPHA**2)
      SM47=DSQRT((ALPHA-ALPHA02)**2+DALPHA**2)
      F2_47=2.D0/(SP47+SM47)
      DF2_47_DAL=-F2_47/(SP47+SM47)*((ALPHA-ALPHA02)/SM47
     * +(ALPHA+ALPHA02)/SP47)
      DF2_47_DDA=-F2_47/(SP47+SM47)*DALPHA*(1.D0/SM47+1.D0/SP47)
c
      S3=ABT07+ABT08*STA+ABT09*STB
      DS3DT=4.D0*ST*CT*(ABT08*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT09*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR7=F2_47**ABT37*ALPHA**ABT38/D3
      DR8=DR7*STA
      DR9=DR7*STB
C-----------------------------------------------------
      S4=ABT10+ABT11*STA+ABT12*STB
      DS4DT=4.D0*ST*CT*(ABT11*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT12*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR10=F1_46**ABT49/D4  
      DR11=DR10*STA
      DR12=DR10*STB
C-----------------------------------------------------
      S5=ABT13+ABT14*STA+ABT15*STB
      DS5DT=4.D0*ST*CT*(ABT14*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT15*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR13=F1_46**ABT50/D5  
      DR14=DR13*STA
      DR15=DR13*STB
C----------------------------------------------------
      ALPHA03=ABT47
      SP48=DSQRT((ALPHA+ALPHA03)**2+DALPHA**2)
      SM48=DSQRT((ALPHA-ALPHA03)**2+DALPHA**2)
      F3_48=2.D0/(SP48+SM48)
      DF3_48_DAL=-F3_48/(SP48+SM48)*((ALPHA-ALPHA03)/SM48
     * +(ALPHA+ALPHA03)/SP48)
      DF3_48_DDA=-F3_48/(SP48+SM48)*DALPHA*(1.D0/SM48+1.D0/SP48)    
c
      S6=ABT16+ABT17*STA+ABT18*STB
      DS6DT=4.D0*ST*CT*(ABT17*ABT34*ST2**(2.D0*ABT34-1.D0)
     *  +ABT18*ABT35*ST2**(2.D0*ABT35-1.D0))
      DR16=F3_48**ABT36*(1.D0-ALPHA)**ABT38/D6
      DR17=DR16*STA
      DR18=DR16*STB
C----------------------------------------------------
      bit = ABT01*DR1+ABT02*DR2+ABT03*DR3+ABT04*DR4+ABT05*DR5+
     *ABT06*DR6+ABT07*DR7+ABT08*DR8+ABT09*DR9+ABT10*DR10+
     *ABT11*DR11+ABT12*DR12+ABT13*DR13+ABT14*DR14+ABT15*DR15+
     *ABT16*DR16+ABT17*DR17+ABT18*DR18 
C
      BT=bit*CP
C
C=======================================================================  
C
C-------  NOW CALCULATE DERIVATIVE D_bit_Dt: ------------------------- 
C
      D_bit_Dt=DS1DT/D1+S1/D1**2*ABT51*((R/ABT39)**2*CT2)**(ABT51
     *  -1.D0)*(R/ABT39)**2*2.D0*ST*CT

     * +(DS2DT/D2+S2/D2**2*ABT52*((R/ABT40)**2*CT2)**(ABT52-1.D0)*
     *  (R/ABT40)**2*2.D0*ST*CT)*F1_46**ABT48
     *  +S2/D2*ABT48*F1_46**(ABT48-1.D0)*DF1_46_DAL*DAL_DT

     * +(DS3DT/D3+S3/D3**2*ABT53*((R/ABT41)**2*CT2)**(ABT53-1.D0)*
     *  (R/ABT41)**2*2.D0*ST*CT)*F2_47**ABT37*ALPHA**ABT38
     *  +S3/D3*ABT37*F2_47**(ABT37-1.D0)*DF2_47_DAL*DAL_DT*ALPHA**
     * ABT38+S3/D3*ABT38*F2_47**ABT37*ALPHA**(ABT38-1.D0)*DAL_DT

     * +(DS4DT/D4+S4/D4**2*ABT54*((R/ABT42)**2*CT2)**(ABT54-1.D0)*
     *  (R/ABT42)**2*2.D0*ST*CT)*F1_46**ABT49
     *  +S4/D4*ABT49*F1_46**(ABT49-1.D0)*DF1_46_DAL*DAL_DT

     * +(DS5DT/D5+S5/D5**2*ABT55*((R/ABT43)**2*CT2)**(ABT55-1.D0)*
     *  (R/ABT43)**2*2.D0*ST*CT)*F1_46**ABT50
     *  +S5/D5*ABT50*F1_46**(ABT50-1.D0)*DF1_46_DAL*DAL_DT

     * +(DS6DT/D6+S6/D6**2*ABT56*((R/ABT44)**2*CT2)**(ABT56-1.D0)*
     *  (R/ABT44)**2*2.D0*ST*CT)*F3_48**ABT36*(1.D0-ALPHA)**ABT38
     *  +S6/D6*ABT36*F3_48**(ABT36-1.D0)*DF3_48_DAL*DAL_DT*(1.D0
     *  -ALPHA)**ABT38-S6/D6*ABT38*F3_48**ABT36*(1.D0-ALPHA)**
     *  (ABT38-1.D0)*DAL_DT  
C
C  FINALLY, CALCULATE BPHI FROM BR AND BTHETA, FROM THE EQUATION DivB=0:
C
      BP=-SP*(ST*( ST*CT*(2.D0*betar+R*D_betar_DR)+D_bit_Dt)+bit*CT)          

      RETURN
      END
C
C================================================================================
c
      SUBROUTINE SPHCAR (R,THETA,PHI,X,Y,Z,J)  !   IDENTICAL TO SPHCAR_08 FROM GEOPACK-2008 PACKAGE
C
C   CONVERTS SPHERICAL COORDS INTO CARTESIAN ONES AND VICE VERSA
C    (THETA AND PHI IN RADIANS).
C
C                  J>0            J<0
C-----INPUT:   J,R,THETA,PHI     J,X,Y,Z
C----OUTPUT:      X,Y,Z        R,THETA,PHI
C
C  NOTE: AT THE POLES (X=0 AND Y=0) WE ASSUME PHI=0 WHEN CONVERTING
C        FROM CARTESIAN TO SPHERICAL COORDS (I.E., FOR J<0)
C
C   LAST MOFIFICATION:    MARCH 21, 2008 (DOUBLE-PRECISION VERSION)
C
C   AUTHOR:  N. A. TSYGANENKO
C
      IMPLICIT REAL*8 (A-H,O-Z)

      IF(J.GT.0) GOTO 3
      SQ=X**2+Y**2
      R=DSQRT(SQ+Z**2)
      IF (SQ.NE.0.D0) GOTO 2
      PHI=0.D0
      IF (Z.LT.0.D0) GOTO 1
      THETA=0.D0
      RETURN
  1   THETA=3.141592654D0
      RETURN
  2   SQ=DSQRT(SQ)
      PHI=DATAN2(Y,X)
      THETA=DATAN2(SQ,Z)
      IF (PHI.LT.0.D0) PHI=PHI+6.283185307D0
      RETURN
  3   SQ=R*DSIN(THETA)
      X=SQ*DCOS(PHI)
      Y=SQ*DSIN(PHI)
      Z=R*DCOS(THETA)
      RETURN
      END
C
C===========================================================================
c
      SUBROUTINE BSPCAR (THETA,PHI,BR,BTHETA,BPHI,BX,BY,BZ)  !  IDENTICAL TO BSPCAR_08 FROM GEOPACK-2008 PACKAGE
C
C   CALCULATES CARTESIAN FIELD COMPONENTS FROM LOCAL SPHERICAL ONES
C
C-----INPUT:   THETA,PHI - SPHERICAL ANGLES OF THE POINT IN RADIANS
C              BR,BTHETA,BPHI -  LOCAL SPHERICAL COMPONENTS OF THE FIELD
C-----OUTPUT:  BX,BY,BZ - CARTESIAN COMPONENTS OF THE FIELD
C
C   LAST MOFIFICATION:    MARCH 21, 2008 (DOUBLE-PRECISION VERSION)
C
C   AUTHOR: N. A. TSYGANENKO
C
      IMPLICIT REAL*8 (A-H,O-Z)

      S=DSIN(THETA)
      C=DCOS(THETA)
      SF=DSIN(PHI)
      CF=DCOS(PHI)
      BE=BR*S+BTHETA*C
      BX=BE*CF-BPHI*SF
      BY=BE*SF+BPHI*CF
      BZ=BR*C-BTHETA*S
      RETURN
      END
c
C=======================================================================================
C
      SUBROUTINE PRC_SHLD_NM (X,Y,Z,BZIMF,EPS,SCALE,FX,FY,FZ)
C
C   CALCULATES: (1) COMPONENTS OF THE TOTAL SHIELDING FIELD VECTOR AT ANY GIVEN POSITION X,Y,Z
C
C   THE OUTPUT VECTOR {FX,FY,FZ} IS USED FOR CHECKING THE MODEL (E.G., LINE TRACING),
C
C      INPUT:  X,Y,Z           - POSITION VECTOR (KSM FOR SATURN OR GSM FOR EARTH)
C              BZIMF           - IMF BZ 
C              EPS             - EXPONENT OF R IN THE RADIAL STRETCH FUNCTION
C              SCALE           - OVERALL SCALE OF THE MODEL RING CURRENT       
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=240,NTOT=245)
        DIMENSION A(NTOT)
        DATA A/ 
     *  34.181192D0  ,-0.29556168D0  ,  12.909051D0  ,  4.9801298D0  ,
     *  6.1500375D0  ,-0.74915782D0  , -21.611778D0  , -7.1847034D0  ,
     *  5.6796017D0  ,  2.2643727D0  ,-0.95787448D0  , -4.9424794D0  ,
     * -5.6198272D0  ,  1.2624966D0  ,  18.578143D0  ,-0.30092775D-01,
     *  1.6880753D0  , -9.2193277D0  , -1.5771469D0  , -5.3423316D0  ,
     * -20.261817D0  ,  3.2126805D0  , -2.4372304D0  ,  1.0275728D0  ,
     * -3.6717185D0  ,  12.767695D0  ,  8.1013226D0  ,  9.7460894D0  ,
     *  7.9191163D0  ,  1.8208879D0  ,  1.9377176D0  , 0.37277955D0  ,
     *  4.0809641D0  , -1.0439858D0  , -4.4019306D0  ,  12.338464D0  ,
     * -10.577554D0  ,  1.7692719D0  ,  2.0392403D0  ,  3.7824888D0  ,
     * -37.258280D0  ,  2.9374995D0  , -16.101684D0  , -5.7453565D0  ,
     * -10.174126D0  , -11.332124D0  ,  6.2470586D0  ,  11.112049D0  ,
     *-0.89338110D0  ,  3.8577045D0  ,  5.0320009D0  ,  2.1298388D0  ,
     *-0.56721888D0  , -1.6649850D0  ,  1.8637425D0  ,  2.4207808D0  ,
     * -1.2246224D0  , -1.0654469D0  , -7.5198908D0  , -1.7276819D0  ,
     *  3.0034762D0  , 0.91545679D0  ,  2.7752526D0  , 0.19492507D-01,
     * -7.7467291D0  , 0.28431333D0  ,  2.9739456D0  ,  2.2474879D0  ,
     * -1.1848038D0  , 0.31421829D0  , 0.99219624D0  , -1.5681495D0  ,
     * -1.6979963D0  , 0.13900377D0  ,  3.6281564D0  , -8.3170206D0  ,
     * 0.73709654D0  , -3.2741704D0  ,-0.45948651D0  ,-0.94160370D0  ,
     * -25.037655D0  , -1.0611731D0  , -5.5580580D0  , -4.5030523D0  ,
     * -15.515880D0  , -5.1363568D0  ,  18.529757D0  ,  10.939651D0  ,
     * -2.7949921D0  ,-0.54581881D-01,  11.195992D0  ,  3.3436551D0  ,
     *  5.0624281D0  , -1.3006263D0  , -5.1364744D0  ,  3.5690900D0  ,
     * -4.2923300D0  ,  2.1328852D0  , -5.2467221D0  ,-0.13620892D0  ,
     *  18.770186D0  , -2.3562018D0  ,  5.7961592D0  ,-0.63064669D0  ,
     * -5.5749313D0  , -10.435435D0  , -1.8886772D0  , -5.7032962D0  ,
     * -5.7101555D0  ,-0.76821310D0  , -1.2209654D0  , -1.5274722D0  ,
     * -5.3357059D0  ,  1.0722311D0  ,  8.6030986D0  , -18.215785D0  ,
     *  9.5865548D0  , -4.5046592D0  , -1.7123164D0  , -4.1614906D0  ,
     *  17.800578D0  , -2.1978165D0  ,  14.218007D0  ,  2.3727138D0  ,
     * -8.2979498D0  ,  9.2359859D0  ,  10.684782D0  , -2.2509492D0  ,
     *-0.60787776D0  , -2.2727511D0  , -1.8043182D0  , 0.73848940D0  ,
     * 0.76077508D0  ,  1.3688103D0  ,  2.5610340D0  , 0.47900536D0  ,
     * -4.5753533D0  ,  2.2541873D0  ,  6.4528160D0  ,  1.7519095D0  ,
     *  9.6375720D0  , -3.9428667D0  ,  1.4793671D0  ,-0.86386190D0  ,
     *  3.9761522D0  , -10.535133D0  , -4.9330324D0  , -8.1509487D0  ,
     * -1.4594917D0  , 0.20336362D0  , -2.8632923D0  ,  2.0394247D0  ,
     * -2.4186769D0  , 0.75471181D0  ,  1.5998619D0  ,-0.72653280D0  ,
     *  5.9104472D0  ,  2.1555852D0  , -1.6667320D0  , -2.4244646D0  ,
     *  20.502462D0  ,-0.27327685D0  ,  8.8452226D0  ,  4.7386919D0  ,
     *  10.593730D0  ,  13.271913D0  , -2.3608862D0  , -13.065089D0  ,
     *  3.5673961D0  , -2.1986219D0  , -28.413638D0  , 0.56823373D-01,
     * -10.554222D0  ,  2.2221769D0  ,  7.9440106D0  , -4.8503839D0  ,
     *-0.40088324D0  ,  5.6948041D0  ,  14.466581D0  ,  5.4476484D0  ,
     * -14.151370D0  , -1.8888276D0  , -6.6069793D0  ,-0.83710456D0  ,
     *  11.437575D0  ,  1.4843343D0  , -5.7288563D0  , -3.2866698D0  ,
     *  5.1769964D0  ,  1.4806221D0  ,-0.64790109D-01,  5.4533199D0  ,
     *  5.2463491D0  ,-0.70123068D0  , -14.265660D0  ,  29.018673D0  ,
     * -8.8034709D0  ,  10.430212D0  , -1.9187523D0  ,  2.8974242D0  ,
     * -5.5815235D0  , -3.4324352D0  , -12.599610D0  , -2.9010821D0  ,
     *  24.864737D0  ,-0.74230293D0  , -5.6457389D0  , -9.8136339D0  ,
     * -2.0992820D0  , -6.7259956D0  ,  8.8169633D0  ,  3.8670580D0  ,
     *  8.8654183D0  , -1.2980093D0  , -29.748927D0  , -4.6201843D0  ,
     *  5.5276324D0  ,  12.587564D0  , -10.083196D0  ,  5.8797310D0  ,
     *  4.8521062D0  ,  1.3373527D0  , 0.42719831D0  ,  1.1380295D0  ,
     *  3.1629646D0  ,  6.0495530D0  ,  1.8828022D0  , -1.8822390D0  ,
     * -2.0332153D0  , -6.2623047D0  , 0.84218678D0  , -3.9049652D0  ,
     *-0.59496182D0  ,-0.14949346D0  ,  4.7583767D0  , -14.009779D0  ,
     *  2.8995911D0  , -5.0087497D0  ,  2.7360707D0  ,  1.3343890D0  ,
     * 0.22048751D-01, 0.61515464D-01, 0.56954579D-03,-0.98457765D-03,
     *-0.33742397D-01/
C
      DATA BZIMF_LOW,BZIMF_HIGH /-10.D0,10.D0/
      DATA SCALE_LOW,SCALE_HIGH /0.7D0,1.4D0/
      DATA EPS_LOW,EPS_HIGH / 0.2D0,0.6D0/

        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   !  NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FBZ21=DABS(FBZ)/DSQRT(1.D0+A(NLIN+3)*FBZ**2)
        FSCA  =(SCALE*2.D0-SCALE_HIGH-SCALE_LOW)/(SCALE_HIGH-SCALE_LOW)
        FEPS =(EPS*2.D0-EPS_HIGH-EPS_LOW)/(EPS_HIGH-EPS_LOW)

        A0=A(NLIN+1)+A(NLIN+4)*FSCA   
        B0=A(NLIN+2)+A(NLIN+5)*FSCA   
C
        FX=0.D0
        FY=0.D0
        FZ=0.D0
C
        IND=1
C
        DO 1 I=1,6 
        DO 1 K=1,4

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         AI=A(IND)+A(IND+1)*FBZ+A(IND+2)*FBZ21+A(IND+3)*FEPS  
     *     +A(IND+4)*FSCA+A(IND+5)*FSCA**2+A(IND+6)*FSCA**3
     *     +A(IND+7)*FBZ*FSCA+A(IND+8)*FBZ21*FSCA+A(IND+9)*FBZ*FSCA**2
         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ

         IND=IND+10

  1      CONTINUE
C
         RETURN   
         END
C
C==========================================================================================
c
      SUBROUTINE PRC_UNSH_DD (EPS,SCALE,X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
c
      DATA DT0 /0.07D0/  !  controls the noon-midnight angular stretch (radians)
c                           typical values: between 0.04 and 0.08; here fixed at 0.07
      DATA DR0 /0.17D0/  !  controls the magnitude of radial stretch 
c                           typical values: 0.1 and larger; here fixed at 0.17
c     Input:
C
C     EPS    -  controls the noon-midnight asymmetry of the FAC surface, in the distant magnetosphere 
c              Typical values: between 0.1 and 1.0
c
c     DR0    -  controls the magnitude of radial stretch 
c              Tentative trial values: 0.1 and larger  
c
c     SCALE  -  controls the overall size of the system. Normal values around 1.0-1.5. Larger values yield smaller SRC.
c
      Xsc=X*SCALE
      Ysc=Y*SCALE
      Zsc=Z*SCALE
c
c -----  convert Cartesian into spherical:
c
      RHOsc=DSQRT(Ysc**2+Zsc**2)
      Rsc=DSQRT(Xsc**2+Ysc**2+Zsc**2)
      T=DATAN2(RHOsc,Xsc)
      Ts=T-DT0*Rsc**EPS*RHOsc/Rsc   ! deformed Theta_s=Theta-DT0*R^EPS*sin(Theta), where sin(Theta)=RHOsc/Rsc
      DTsDR=-DT0*EPS*Rsc**(EPS-1.D0)*RHOsc/Rsc  !  derivative dTheta_s/dR
      DTsDT=1.D0-DT0*Rsc**EPS*Xsc/Rsc          !  derivative dTheta_s/dTheta
c
      R_prime=Rsc*(1.d0-DR0*dsin(T/2.d0)**2)
      dR_prime_dR=1.d0-DR0*dsin(T/2.d0)**2
      dR_prime_dT= -DR0*Rsc*0.5d0*DSIN(T)

c
      IF (RHOsc.GT.1.D-5) THEN
         CP=Ysc/RHOsc         !  CP & SP will be needed below to convert from spherical to Cartesian
         SP=Zsc/RHOsc
         STsST=DSIN(Ts)/DSIN(T)
      ELSE
         CP=1.D0
         SP=0.D0
         STsST=DTsDT
      ENDIF
c
      X_s=R_prime*DCOS(Ts)
      Y_s=R_prime*DSIN(Ts)*CP
      Z_s=R_prime*DSIN(Ts)*SP
c
      CALL PRC_DD_UNDEFORMED (X_s,Y_s,Z_s,BXas,BYas,BZas)    !  calculate Cartesian B components at primed location
C
      RHO2_s=Y_s**2+Z_s**2
      R_s=DSQRT(RHO2_s+X_s**2)
      RHO_s=DSQRT(RHO2_s)
c
      CT_s=X_s/R_s
      ST_s=RHO_s/R_s
c
      BRas=(X_s*BXas+Y_s*BYas+Z_s*BZas)/R_s               ! convert from Cartesian to spherical
      BTHETAas=(BYas*CP+BZas*SP)*CT_s-BXas*ST_s
      BPHIas=BZas*CP-BYas*SP
c
      BR_s  =STsST*DTsDT*(R_prime/Rsc)**2*BRas 
     * -R_prime/Rsc**2*STsST*dR_prime_dT*BTHETAas                           ! apply deformation tensor

      BTHETA_s=-R_prime**2/Rsc*STsST*DTsDR*BRas
     * +STsST*(R_prime/Rsc)*dR_prime_dR*BTHETAas

      BPHI_s =(R_prime/Rsc)*(dR_prime_dR*dTsDT-dR_prime_dT*dTsDR)*BPHIas
c
      BXsc=BR_s*DCOS(T)-BTHETA_s*DSIN(T)                  ! convert from spherical to Cartesian
      Bmerid =BR_s*DSIN(T)+BTHETA_s*DCOS(T)
      BYsc=Bmerid*CP-BPHI_s*SP
      BZsc=Bmerid*SP+BPHI_s*CP
c
      BX=BXsc*SCALE                                       !  scale
      BY=BYsc*SCALE
      BZ=BZsc*SCALE

      BX=BXsc    !  here we do not scale the field, so that different values of SCALE yield the same Bz at origin
      BY=BYsc    !   (that is, shrinking/expansion of the SRC keeps Dst-index constant)
      BZ=BZsc
c
      RETURN
      END
c
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE PRC_DD_UNDEFORMED (X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
      X1=-Y
      Y1= X
      CALL SPHCAR (R,T,P,X1,Y1,Z,-1)
      CALL BRBTBP_PRC (R,T,P,BR,BT,BP)
      CALL BSPCAR (T,P,BR,BT,BP,BX1,BY1,BZ)
      BX= BY1
      BY=-BX1
      RETURN
      END
C
C=======================================================================================
C
      SUBROUTINE PRC_SHLD_DD (X,Y,Z,BZIMF,EPS,SCALE,FX,FY,FZ)
C
C   CALCULATES: (1) COMPONENTS OF THE TOTAL SHIELDING FIELD VECTOR AT ANY GIVEN POSITION X,Y,Z
C
C   THE OUTPUT VECTOR {FX,FY,FZ} IS USED FOR CHECKING THE MODEL (E.G., LINE TRACING),
C
C      INPUT:  X,Y,Z           - POSITION VECTOR (KSM FOR SATURN OR GSM FOR EARTH)
C              BZIMF           - IMF BZ 
C              EPS             - EXPONENT OF R IN THE RADIAL STRETCH FUNCTION
C              SCALE           - OVERALL SCALE OF THE MODEL RING CURRENT       
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=240,NTOT=245)
        DIMENSION A(NTOT)
        DATA A/  
     * -173.08748D0  , -12.051841D0  , -19.946824D0  ,  19.642193D0  ,
     *  70.310018D0  ,  75.177656D0  ,  34.520150D0  , -24.974531D0  ,
     *  70.732351D0  ,  10.096452D0  , -13.597701D0  ,  9.3944292D0  ,
     * -15.171517D0  , -27.923772D0  , -108.18465D0  , -35.158396D0  ,
     *  89.647468D0  ,  37.444494D0  , -61.947256D0  , -1.9842204D0  ,
     *  18.894318D0  , -27.206972D0  ,  13.294326D0  ,  22.073997D0  ,
     * -1.1570484D0  , -63.182196D0  ,  11.914302D0  , -12.687222D0  ,
     *  21.617200D0  ,-0.37557404D0  , -48.038747D0  ,  4.8491468D0  ,
     * -33.408076D0  ,  20.712963D0  ,  10.266758D0  ,  35.618844D0  ,
     * -49.948130D0  , -32.758746D0  , 0.21312420D0  , -4.6602706D0  ,
     *  193.50819D0  , -2.1276278D0  ,  59.650791D0  , -10.523259D0  ,
     *  18.628808D0  ,  18.457484D0  , -32.078623D0  ,  1.5128950D0  ,
     * -26.401573D0  , -16.213012D0  ,  9.3973986D0  ,  11.421672D0  ,
     * -15.995494D0  ,  2.5182425D0  , -59.907718D0  , -25.152481D0  ,
     *  27.371634D0  ,  21.511491D0  , -44.570481D0  ,  2.9882407D0  ,
     *  54.043177D0  , -17.106961D0  ,  35.876256D0  , -2.5569585D0  ,
     *  46.939193D0  , -42.179131D0  , -1.7869133D0  , -7.4885614D0  ,
     *  52.033555D0  ,  16.738295D0  , -32.316580D0  ,  19.629341D0  ,
     * -24.408864D0  , -6.3288935D0  ,  21.064037D0  ,  46.977419D0  ,
     * -26.677679D0  , -7.5744088D0  ,-0.86427191D0  ,  6.0345735D0  ,
     *  5.9322392D0  ,  7.8290484D0  , -29.787536D0  , -10.080507D0  ,
     *  22.539417D0  , -27.461210D0  , -12.472286D0  ,  4.3290604D0  ,
     * -16.318832D0  , -9.1716953D0  , -75.242932D0  ,  8.2176937D0  ,
     * -20.877768D0  ,  21.847929D0  ,  17.448625D0  ,  48.630093D0  ,
     * -41.600240D0  , -19.127657D0  ,  25.205628D0  ,  3.1041289D0  ,
     *  21.523811D0  ,  2.0616062D0  ,  20.424430D0  , -21.764999D0  ,
     *  29.124129D0  ,  13.141096D0  , -19.796480D0  , -2.6500204D0  ,
     *  25.988169D0  ,  7.7667605D0  ,  16.157029D0  ,  2.5209401D0  ,
     *  8.8512540D0  , -14.021652D0  , -13.846838D0  , -6.4784611D0  ,
     *  37.757105D0  ,  28.494851D0  , -20.904987D0  , -5.0475496D0  ,
     * -105.37290D0  ,-0.73661356D0  , -43.981724D0  ,  12.185297D0  ,
     * -18.062315D0  , -31.225747D0  , -3.4212167D0  , -3.6461840D0  ,
     *  8.1305868D0  ,  18.702539D0  , -30.208994D0  , -10.722295D0  ,
     *  19.462849D0  ,  5.2336031D0  ,  18.313843D0  ,  53.223035D0  ,
     * -34.990831D0  , -23.663671D0  ,  38.032338D0  , -6.2281490D0  ,
     * -20.807831D0  ,  13.285530D0  , -26.683761D0  ,  3.4193421D0  ,
     * -49.460525D0  ,  15.576843D0  , -16.895695D0  ,  6.2609664D0  ,
     * -57.219748D0  , -19.128612D0  ,  50.075329D0  , -26.916746D0  ,
     *  33.645466D0  ,  15.689089D0  , -36.789764D0  , -55.196457D0  ,
     *  47.574860D0  ,  17.132535D0  , -14.424625D0  , -15.822546D0  ,
     *  83.202588D0  , -5.0383153D0  ,  49.946407D0  , -2.4785598D0  ,
     * -47.015559D0  , -30.138088D0  ,  14.711106D0  ,  17.344510D0  ,
     * -4.0845934D0  ,  17.843695D0  ,  115.33077D0  , -21.099448D0  ,
     *  54.761848D0  , -28.303078D0  ,  62.293728D0  , -20.104082D0  ,
     *  58.378709D0  ,  7.1908603D0  ,  23.665277D0  , -11.362744D0  ,
     * -66.391112D0  ,  27.684715D0  , -73.050201D0  ,  19.355094D0  ,
     * -17.811321D0  , -5.3336640D0  ,  10.516419D0  ,  15.630203D0  ,
     * -62.763691D0  , -7.6699197D0  , -27.861515D0  ,  2.7636923D0  ,
     * 0.82739237D0  , -9.6442883D0  ,  40.062154D0  ,  29.212481D0  ,
     * -47.375480D0  , -36.804083D0  ,  64.721840D0  ,  32.029313D0  ,
     * -64.903513D0  ,  24.134575D0  , -35.928899D0  ,  1.1136802D0  ,
     * -34.727863D0  , -11.552687D0  , -28.495564D0  ,  18.288438D0  ,
     * -23.780405D0  , -15.319146D0  , -9.4164136D0  , -8.5964473D0  ,
     * -15.217163D0  ,  10.599390D0  ,  18.596816D0  ,  22.026133D0  ,
     * -1.5248560D0  , -27.218851D0  , -2.4900813D0  ,  6.3982362D0  ,
     *  24.291437D0  , -10.071961D0  ,  40.848718D0  , -9.2288516D0  ,
     * -4.0876113D0  , -7.8354217D0  ,-0.68464361D0  ,  6.3643265D0  ,
     *  40.134695D0  ,  12.120108D0  ,  2.9190973D0  ,  4.0186331D0  ,
     * -10.513156D0  ,  2.9362587D0  , -7.4585215D0  , -2.5039220D0  ,
     *  8.5429537D0  ,  9.3137662D0  , -29.772613D0  , -15.041690D0  ,
     * 0.28940081D-01, 0.48745244D-01, 0.27521861D0  , 0.19257827D-05,
     *-0.30864086D-01/
C
      DATA BZIMF_LOW,BZIMF_HIGH /-10.D0,10.D0/
      DATA SCALE_LOW,SCALE_HIGH /0.7D0,1.4D0/
      DATA EPS_LOW,EPS_HIGH / 0.2D0,0.6D0/

        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   !  NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FBZ21=DABS(FBZ)/DSQRT(1.D0+A(NLIN+3)*FBZ**2)
        FSCA  =(SCALE*2.D0-SCALE_HIGH-SCALE_LOW)/(SCALE_HIGH-SCALE_LOW)
        FEPS =(EPS*2.D0-EPS_HIGH-EPS_LOW)/(EPS_HIGH-EPS_LOW)

        A0=A(NLIN+1)+A(NLIN+4)*FSCA   
        B0=A(NLIN+2)+A(NLIN+5)*FSCA   
C
        FX=0.D0
        FY=0.D0
        FZ=0.D0
C
        IND=1
C
        DO 1 I=1,6 
        DO 1 K=1,4

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*SAY*SBZ
         BY= A_I*EX*CAY*SBZ
         BZ= B_K*EX*SAY*CBZ

         AI=A(IND)+A(IND+1)*FBZ+A(IND+2)*FBZ21+A(IND+3)*FEPS  
     *     +A(IND+4)*FSCA+A(IND+5)*FSCA**2+A(IND+6)*FSCA**3
     *     +A(IND+7)*FBZ*FSCA+A(IND+8)*FBZ21*FSCA+A(IND+9)*FBZ*FSCA**2
         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ

         IND=IND+10

  1      CONTINUE
C
         RETURN   
         END
C
C=======================================================================================
c
      SUBROUTINE PRCS_UNSH (EPS,SCALE,X,Y,Z,BX,BY,BZ)
      IMPLICIT REAL*8 (A-H,O-Z)
c
      DATA DT0 /0.07D0/  !  controls the noon-midnight angular stretch (radians)
c              Typical values: between 0.04 and 0.08; here fixed at 0.07
      DATA DR0 /0.17D0/     !  controls the magnitude of radial stretch
C                              Tentative trial values: 0.1 and larger  
c    Input:
C
C    EPS    -  controls the noon-midnight asymmetry of the FAC surface, in the distant magnetosphere 
c              Typical values: between 0.1 and 1.0
c
c    SCALE  -  controls the overall size of the system. Normal values around 1.0-1.5. Larger values yield smaller SRC.
c
      Xsc=X*SCALE
      Ysc=Y*SCALE
      Zsc=Z*SCALE
c
c  -----   convert Cartesian into spherical:
c
      RHOsc=DSQRT(Ysc**2+Zsc**2)
      Rsc=DSQRT(Xsc**2+Ysc**2+Zsc**2)
      T=DATAN2(RHOsc,Xsc)
      Ts=T-DT0*Rsc**EPS*RHOsc/Rsc   ! deformed Theta_s=Theta-DT0*R^EPS*sin(Theta), where sin(Theta)=RHOsc/Rsc
      DTsDR=-DT0*EPS*Rsc**(EPS-1.D0)*RHOsc/Rsc  !  derivative dTheta_s/dR
      DTsDT=1.D0-DT0*Rsc**EPS*Xsc/Rsc          !  derivative dTheta_s/dTheta
c
      R_prime=Rsc*(1.d0-DR0*dsin(T/2.d0)**2)
      dR_prime_dR=1.d0-DR0*dsin(T/2.d0)**2
      dR_prime_dT= -DR0*Rsc*0.5d0*DSIN(T)

c
      IF (RHOsc.GT.1.D-5) THEN
         CP=Ysc/RHOsc         !  CP & SP will be needed below to convert from spherical to Cartesian
         SP=Zsc/RHOsc
         STsST=DSIN(Ts)/DSIN(T)
      ELSE
         CP=1.D0
         SP=0.D0
         STsST=DTsDT
      ENDIF
c
      X_s=R_prime*DCOS(Ts)
      Y_s=R_prime*DSIN(Ts)*CP
      Z_s=R_prime*DSIN(Ts)*SP
c
      CALL PRCS_AXISYMMETRIC (X_s,Y_s,Z_s,BXas,BYas,BZas)    !  calculate Cartesian B components at primed location
C
      RHO2_s=Y_s**2+Z_s**2
      R_s=DSQRT(RHO2_s+X_s**2)
      RHO_s=DSQRT(RHO2_s)
c
      CT_s=X_s/R_s
      ST_s=RHO_s/R_s
c
      BRas=(X_s*BXas+Y_s*BYas+Z_s*BZas)/R_s               ! convert from Cartesian to spherical
      BTHETAas=(BYas*CP+BZas*SP)*CT_s-BXas*ST_s
      BPHIas=BZas*CP-BYas*SP
c
      BR_s  =STsST*DTsDT*(R_prime/Rsc)**2*BRas 
     * -R_prime/Rsc**2*STsST*dR_prime_dT*BTHETAas                           ! apply deformation tensor

      BTHETA_s=-R_prime**2/Rsc*STsST*DTsDR*BRas
     * +STsST*(R_prime/Rsc)*dR_prime_dR*BTHETAas

      BPHI_s =(R_prime/Rsc)*(dR_prime_dR*dTsDT-dR_prime_dT*dTsDR)*BPHIas
c
      BXsc=BR_s*DCOS(T)-BTHETA_s*DSIN(T)                  ! convert from spherical to Cartesian
      Bmerid =BR_s*DSIN(T)+BTHETA_s*DCOS(T)
      BYsc=Bmerid*CP-BPHI_s*SP
      BZsc=Bmerid*SP+BPHI_s*CP
c
      BX=BXsc    !  here we do not scale the field, so that different values of SCALE yield the same Bz at origin
      BY=BYsc    !   (that is, shrinking/expansion of the SRC keeps Dst-index constant)
      BZ=BZsc
c
      RETURN
      END
c
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE PRCS_AXISYMMETRIC (X,Y,Z,BX,BY,BZ)
      IMPLICIT  REAL * 8  (A - H, O - Z)
      DATA F1,F2,DR,R1,D1,R2,D2,AL,A1,A2,A3,A4,A5,A6,A7,B1,B2,B3,B4,B5,
     * B6,B7 /-77.8729D0,63.2601D0,2.60373D0,5.49124D0,2.80109D0,
     *4.27404D0,3.79450D0,0.537641D0,0.611891D0,-0.765870D0,4.75619D0,
     *-5.74791D0,0.344349D0,-1.10502D0,1.32319D0,6.24487D0,-2.47322D0,
     *3.80501D0,-12.3363D0,0.882129D0,-0.890554D0,3.31365D0/
c
C  F1: first (outer) loop magnitude,F2: second (inner) loop magnitude
C  DR: damping scale radius, R1: outer loop radius, D1: outer loop spread 
C  R2: inner loop radius, D2: inner loop spread, AL: a factor in 
C  cos(fr), sin(fr), cos(fz), sin(fz), cos(2*fr), sin(2*fr), cos(2*fz), sin(2*fz) terms
C  A1 - A7, B1 - B7: Fourier coefficients in the deformation equations for Rho and Z
C
      RHO=DSQRT(X**2+Y**2)
      IF (RHO.LT.1.D-9) RHO=1.D-9    !   TO PREVENT DIVISION BY 0 AT Z-AXIS
C
      RR2=((RHO-R1)**2+Z**2)/DR**2
      DEX=DEXP(-RR2)
      ARGR=AL*(RHO-R1)/DR
      ARGZ=AL*Z/DR
C
      SR=DSIN(ARGR)
      CR=DCOS(ARGR)
      S2R=2.D0*SR*CR
      C2R=CR**2-SR**2
      SZ=DSIN(ARGZ)
      CZ=DCOS(ARGZ)
      S2Z=2.D0*SZ*CZ
      C2Z=CZ**2-SZ**2
C
      SUMRHO=A1+A2*SR+A3*CR+A4*CZ+A5*S2R+A6*C2R+A7*C2Z
      BRACKRHO=(1.D0+DEX*SUMRHO)
      RHO_AST=RHO*BRACKRHO
C
      SUMZ=B1+B2*SR+B3*CR+B4*CZ
     *+B5*S2R+B6*C2R+B7*C2Z
      BRACKZ=(1.D0+DEX*SUMZ)
      Z_AST  =Z  *BRACKZ
C    
      DRHOAST_DRHO=BRACKRHO-RHO*DEX*SUMRHO*2.D0/DR**2*(RHO-R1)
     * +RHO*DEX*(A2*CR-A3*SR+2.D0*A5*C2R-2.D0*A6*S2R)*AL/DR
      DRHOAST_DZ=-RHO*DEX*2.D0*Z/DR**2*SUMRHO-RHO*DEX*(A4*DSIN(ARGZ)
     * +2.D0*A7*S2Z)*AL/DR
      DZAST_DRHO=-Z*2.D0*(RHO-R1)/DR**2*DEX*SUMZ+Z*DEX*(B2*CR-B3*SR
     * +2.D0*B5*C2R-2.D0*B6*S2R)*AL/DR
      DZAST_DZ=BRACKZ-Z*DEX*2.D0*Z/DR**2*SUMZ-Z*DEX*(B4*DSIN(ARGZ)
     * +2.D0*B7*S2Z)*AL/DR
C
      X_AST  =RHO_AST*X/RHO  
      Y_AST  =RHO_AST*Y/RHO
C
C     THE FIRST (OUTER) LOOP:
C     
      CALL SPREAD_LOOP_B (R1,D1,X_AST,Y_AST,Z_AST,BX_AST,BY_AST,BZ_AST)
C
C      IF (RHO_AST.LT.1.D-9) RHO_AST=1.D-9   !   TO PREVENT DIVISION BY 0 AT Z-AXIS
C
      BRHO_AST=(BX_AST*X_AST+BY_AST*Y_AST)/RHO_AST
C
      BRHO_S=RHO_AST/RHO*(DZAST_DZ*BRHO_AST-DRHOAST_DZ*BZ_AST)
      BPHI_S=0.D0
C
      BX1= BRHO_S*X/RHO
      BY1= BRHO_S*Y/RHO
      BZ1=RHO_AST/RHO*(-DZAST_DRHO*BRHO_AST+DRHOAST_DRHO*BZ_AST)
C
C     THE SECOND (INNER) LOOP:
C     
      CALL SPREAD_LOOP_B (R2,D2,X,Y,Z,BX2,BY2,BZ2)
C      
      BX=F1*BX1+F2*BX2
      BY=F1*BY1+F2*BY2
      BZ=F1*BZ1+F2*BZ2
C
      RETURN 
      END
C
C==========================================================================
C
      SUBROUTINE PRCS_SHLD (X,Y,Z,BZIMF,EPS,SCALE,FX,FY,FZ)
C
C   CALCULATES COMPONENTS OF THE PRCS SHIELDING FIELD VECTOR AT ANY GIVEN POSITION X,Y,Z
C
C      INPUT:  X,Y,Z           - POSITION VECTOR (KSM FOR SATURN OR GSM FOR EARTH)
C              BZIMF           - IMF BZ 
C              EPS             - EXPONENT OF R IN THE RADIAL STRETCH FUNCTION (RANGE:  0.7 - 0.9)
C              SCALE           - OVERALL SCALE OF THE MODEL RING CURRENT       
C              (variation ranges of each of the above 3 parameters are specified in DATA statements below)
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=400,NTOT=410)
C
       DIMENSION A(NTOT)
       DATA A/  
     *  67.092921D0  ,  13.270625D0  ,  34.938796D0  , -53.019097D0  ,
     * -54.744225D0  , -142.90710D0  ,  35.185473D0  ,  99.929273D0  ,
     * -127.46201D0  , -7.0264969D0  , -3.2734459D0  , -275.99991D0  ,
     * -3.8623252D0  , -12.664719D0  , -72.253859D0  ,  116.35440D0  ,
     * -52.411422D0  ,  54.432181D0  ,  17.371851D0  ,  7.3617241D0  ,
     * -164.38164D0  ,  165.84226D0  , -711.83825D0  ,  113.36296D0  ,
     *  109.42072D0  , -929.51327D0  , -110.82952D0  ,  434.17477D0  ,
     *  288.03817D0  ,  6.1706649D0  ,  9.7718739D0  , -286.24759D0  ,
     * -14.015289D0  , -2.1008396D0  ,  126.05752D0  ,  282.59248D0  ,
     *  142.69314D0  , -123.66749D0  , -31.802267D0  , -15.640632D0  ,
     *  238.58112D0  , -254.66810D0  ,  1257.9457D0  , -112.44743D0  ,
     * -59.230558D0  ,  1924.9509D0  ,  148.90583D0  , -694.01152D0  ,
     * -493.48477D0  ,  56.923070D0  ,  1.4569275D0  ,  667.42887D0  ,
     * -41.747721D0  ,  104.37560D0  ,  3.7149568D0  , -485.23713D0  ,
     * -343.89426D0  ,  191.90338D0  ,  24.901160D0  ,  24.788761D0  ,
     *  351.70223D0  ,  140.88694D0  ,  2425.7980D0  , -56.891302D0  ,
     *  16.668083D0  ,  4094.5574D0  ,  29.902185D0  ,  273.67164D0  ,
     * -58.838142D0  , -178.49893D0  , -22.536087D0  ,  2163.2304D0  ,
     *  44.363115D0  , -248.24211D0  , -51.199528D0  ,  160.71389D0  ,
     * -36.753723D0  ,  20.010564D0  ,  10.209530D0  , -30.461139D0  ,
     * -555.52696D0  ,  15.720341D0  , -3010.1245D0  ,  155.78289D0  ,
     *  9.4652458D0  , -4483.3727D0  , -70.346002D0  ,  147.03345D0  ,
     *  432.46840D0  ,  121.64489D0  ,  20.893073D0  , -2048.0019D0  ,
     * -18.382855D0  ,  152.15268D0  ,  40.726342D0  ,  130.76979D0  ,
     *  296.10647D0  , -73.339947D0  , -15.670729D0  ,  12.525128D0  ,
     * -148.40733D0  , -45.025323D0  , -187.75145D0  ,  212.25210D0  ,
     *  127.38564D0  ,  333.89531D0  , -57.797716D0  , -293.64770D0  ,
     *  297.52639D0  ,  26.268864D0  ,  13.531224D0  ,  474.23566D0  ,
     * -40.318562D0  ,  29.233667D0  ,  198.49372D0  , -312.27491D0  ,
     *  28.741175D0  , -52.658778D0  , -40.219051D0  , -39.185930D0  ,
     *  412.91117D0  , -394.01179D0  ,  1737.5872D0  , -382.88757D0  ,
     * -269.96948D0  ,  2181.4441D0  ,  244.20319D0  , -1056.5669D0  ,
     * -726.82929D0  , -49.985499D0  , -35.686778D0  ,  765.27272D0  ,
     *  67.791703D0  , -28.973495D0  , -357.33382D0  , -688.56771D0  ,
     * -278.32442D0  ,  251.40576D0  ,  69.814287D0  ,  59.423655D0  ,
     * -634.02416D0  ,  619.71045D0  , -3408.0255D0  ,  406.68820D0  ,
     *  180.48161D0  , -5000.3553D0  , -384.92336D0  ,  1812.8664D0  ,
     *  1341.3575D0  , -54.334433D0  ,  29.524610D0  , -1819.0054D0  ,
     *  51.508838D0  , -145.15971D0  ,  111.73354D0  ,  1286.9907D0  ,
     *  866.98973D0  , -459.83948D0  , -51.909339D0  , -64.481897D0  ,
     * -555.23446D0  , -319.93675D0  , -3875.3579D0  , -6.3196170D0  ,
     * -66.827324D0  , -6650.5678D0  ,  16.409902D0  , -737.03112D0  ,
     * -188.99819D0  ,  311.36561D0  ,  21.378528D0  , -3596.4971D0  ,
     * -78.343923D0  ,  444.51971D0  ,  40.793612D0  , -468.43967D0  ,
     * -121.06795D0  ,  37.939083D0  , -20.634841D0  ,  63.934436D0  ,
     *  1062.0842D0  , -36.231464D0  ,  5640.0796D0  , -280.63113D0  ,
     * -13.761969D0  ,  8320.7541D0  ,  114.27974D0  , -287.43254D0  ,
     * -772.14426D0  , -229.88876D0  , -35.823631D0  ,  3780.9988D0  ,
     *  35.001138D0  , -288.58457D0  , -70.257472D0  , -251.88512D0  ,
     * -527.38381D0  ,  118.33572D0  ,  31.073828D0  , -24.603263D0  ,
     *  157.95496D0  ,  51.574378D0  ,  69.465668D0  , -271.30470D0  ,
     * -106.37902D0  , -432.71333D0  , -40.600744D0  ,  299.96534D0  ,
     * -181.99764D0  , -15.791806D0  , -17.181854D0  , -316.28675D0  ,
     *  62.613729D0  , -7.4909394D0  , -182.02324D0  ,  314.49164D0  ,
     *  143.26692D0  , -60.560446D0  ,  35.185922D0  ,  69.169120D0  ,
     * -364.95751D0  ,  335.52941D0  , -1258.0890D0  ,  444.73661D0  ,
     *  225.99453D0  , -1381.2258D0  , -112.63644D0  ,  921.79308D0  ,
     *  564.95350D0  ,  53.532936D0  ,  40.431394D0  , -510.20127D0  ,
     * -93.203882D0  ,  31.149961D0  ,  335.70634D0  ,  592.42622D0  ,
     *  63.162396D0  , -102.91484D0  , -55.945556D0  , -88.240443D0  ,
     *  589.43660D0  , -550.21307D0  ,  2931.5956D0  , -479.17049D0  ,
     * -182.91345D0  ,  3969.1557D0  ,  289.24339D0  , -1703.2794D0  ,
     * -1202.2664D0  , -15.543649D0  , -50.112319D0  ,  1426.0228D0  ,
     *  6.8537302D0  ,  41.780365D0  , -197.25998D0  , -1216.6085D0  ,
     * -677.78276D0  ,  326.53483D0  ,  38.915142D0  ,  68.212804D0  ,
     *  190.15509D0  ,  270.59525D0  ,  1576.6995D0  ,  143.22388D0  ,
     *  83.065752D0  ,  2980.7788D0  , -72.020515D0  ,  746.70456D0  ,
     *  439.51349D0  , -156.07429D0  ,  10.068961D0  ,  1747.1771D0  ,
     *  35.646634D0  , -236.18671D0  ,  44.925005D0  ,  508.14470D0  ,
     *  261.56079D0  , -85.193561D0  ,  12.283103D0  , -47.461691D0  ,
     * -625.55547D0  ,  21.651348D0  , -3248.9452D0  ,  144.15061D0  ,
     *  1.4834306D0  , -4752.9412D0  , -50.012899D0  ,  155.94362D0  ,
     *  396.74042D0  ,  134.22233D0  ,  16.761398D0  , -2153.6872D0  ,
     * -19.913973D0  ,  169.75862D0  ,  30.967640D0  ,  135.65216D0  ,
     *  273.18686D0  , -51.520718D0  , -19.612647D0  ,  15.786038D0  ,
     * -62.447098D0  , -17.317582D0  ,  87.732421D0  ,  115.16453D0  ,
     *  38.086845D0  ,  237.52137D0  ,  83.035632D0  , -107.97703D0  ,
     *  5.8965002D0  , -4.8979104D0  ,  6.6369115D0  ,  58.059233D0  ,
     * -22.700805D0  , -15.486878D0  ,  62.355929D0  , -123.77219D0  ,
     * -129.11985D0  ,  90.434149D0  , -10.945588D0  , -39.232726D0  ,
     *  115.41409D0  , -113.53586D0  ,  194.56040D0  , -173.39925D0  ,
     * -69.224744D0  ,  160.44437D0  , -51.477195D0  , -301.11182D0  ,
     * -105.44010D0  , -6.7595220D0  , -12.648877D0  ,  113.10476D0  ,
     *  35.664527D0  ,  8.0166560D0  , -107.38371D0  , -176.73792D0  ,
     *  89.043697D0  , -56.968075D0  ,  17.048559D0  ,  46.443231D0  ,
     * -185.87547D0  ,  188.86931D0  , -757.84943D0  ,  179.22860D0  ,
     *  60.823743D0  , -937.96674D0  , -35.645236D0  ,  585.02278D0  ,
     *  331.76126D0  ,  10.288079D0  ,  17.843489D0  , -340.74042D0  ,
     * -14.476700D0  , -6.5628946D0  ,  79.762834D0  ,  408.86717D0  ,
     *  139.10040D0  , -40.785905D0  , -11.651632D0  , -29.496072D0  ,
     *  8.2459758D0  , -92.382966D0  , -134.30925D0  , -76.270449D0  ,
     * -30.817667D0  , -390.23429D0  ,  19.971322D0  , -280.24610D0  ,
     * -180.38338D0  ,  24.392052D0  , -8.2361709D0  , -275.73457D0  ,
     * -2.7358093D0  ,  41.981227D0  , -31.387494D0  , -195.23967D0  ,
     * -96.673039D0  ,  22.050945D0  , -1.9334181D0  ,  14.189961D0  ,
     *  120.58276D0  , -1.2455097D0  ,  619.68902D0  , -20.326637D0  ,
     *  2.2146868D0  ,  906.01420D0  ,  6.5408356D0  , -17.214191D0  ,
     * -59.380466D0  , -26.169239D0  , -1.9754983D0  ,  411.94515D0  ,
     *  3.4939633D0  , -33.610588D0  , -2.2437028D0  , -16.302879D0  ,
     * -43.122770D0  ,  6.7976165D0  ,  4.2049134D0  , -3.7087596D0  ,
     * 0.21978915D-01, 0.49592622D-01,-0.36118249D-04,-0.58714729D-02,
     * 0.42928510D-02, 0.60047340D-02,-0.29960545D-01,-0.38787015D-01,
     * 0.47028426D-03,-0.36136501D-02/
C
      DATA BZIMF_LOW,BZIMF_HIGH /-10.D0,10.D0/
      DATA SCALE_LOW,SCALE_HIGH /0.5D0,1.5D0/
      DATA EPS_LOW,EPS_HIGH /-1.0D0,1.0D0/

        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   !  NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FBZ21=DABS(FBZ)
        FSCA  =(SCALE*2.D0-SCALE_HIGH-SCALE_LOW)/(SCALE_HIGH-SCALE_LOW)
        FEPS =(EPS*2.D0-EPS_HIGH-EPS_LOW)/(EPS_HIGH-EPS_LOW)
C
        A0=A(NLIN+1)+A(NLIN+3)*FBZ+A(NLIN+5)*FBZ21+A(NLIN+7)*FSCA
     *  +A(NLIN+9)*FEPS  
        B0=A(NLIN+2)+A(NLIN+4)*FBZ+A(NLIN+6)*FBZ21+A(NLIN+8)*FSCA
     *   +A(NLIN+10)*FEPS                                        
C
        FX=0.D0
        FY=0.D0
        FZ=0.D0
C
        IND=1
C
        DO 1 I=1,4 
        DO 1 K=1,5

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FSCA
     *     +A(IND+3) *FEPS  
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FSCA**2
     *     +A(IND+6) *FEPS**2
     *     +A(IND+7) *FBZ*FSCA
     *     +A(IND+8) *FSCA*FEPS
     *     +A(IND+9) *FBZ*FEPS
     *     +A(IND+10)*FBZ*FBZ21
     *     +A(IND+11)*FSCA**3
     *     +A(IND+12)*FEPS**3
     *     +A(IND+13)*FBZ*FSCA*FEPS
     *     +A(IND+14)*FBZ21*FSCA
     *     +A(IND+15)*FBZ*FSCA**2
     *     +A(IND+16)*FSCA**2*FEPS
     *     +A(IND+17)*FSCA*FEPS**2
     *     +A(IND+18)*FBZ*FEPS**2
     *     +A(IND+19)*FBZ21*FEPS
C
         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ

         IND=IND+20

  1      CONTINUE
C
         RETURN   
         END
C
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C
      SUBROUTINE R1_FAC_R (Theta00,DTheta00,PS,X,Y,Z,BX,BY,BZ)  ! THE SUFFIX "R" CORRESPONDS TO REGULAR MODE OF R1 FACs
C
c   INPUTS:  X,Y,Z   -  GSM position of the observation point (RE),
c            Theta00 - colatitude of R1 zone at noon (radians)
c            DTheta00 - the colatitude increment from noon to midnight
c
c   OUTPUT: BX,BY,BZ - field GSM components (nT)
c
      IMPLICIT REAL*8 (A-H,O-Z)
C
      DIMENSION RR(15)
      COMMON /XYZD/ CURDPHI,DD(15),SP(25),CP(25),XXN(15,25),YYN(15,25),
     *  ZZN(15,25),XXS(15,25),YYS(15,25),ZZS(15,25)
C
      SAVE
      DATA RR / 0.D0,1.D0,1.9529D0,3.0653D0,4.4573D0,6.0765D0,7.9188D0, 
     *10.0630D0,12.8026D0,17.2532D0,32.2532D0,47.2532D0,62.2532D0,
     *77.2532D0, 92.2532D0/
C
      DATA RH0/8.D0/,ALPHA/4.D0/,PI/3.14159265359D0/
      DATA D0/0.08D0/
      DATA N/3/             !  designated in the present version of the 2015 paper as "nu"
      DATA MW/25/    !   number of wires, spanning 360-degs of the R1 FAC oval
      DATA T0,DT0,PS0 /3*10.D0/
C-----------------------------------------------------------------------------------------------------
      IF (Theta00.NE.T0.OR.DTheta00.NE.DT0.OR.PS.NE.PS0) 
     *  THEN                                              !   INITIALIZE ARRAYS ON FIRST CALL (OR IN CASE OF CHANGE
C                                                               IN THE ABOVE FOUR PARAMETERS)
      T0=Theta00
      DT0=DTheta00
      PS0=PS
C
      DPHI=6.283185307D0/MW  ! AZIMUTHAL SUMMATION STEPSIZE IN RADIANS
      CURDPHI=DPHI/2.0D0     ! PEAK CURRENT (PER THE SUMMATION "WIRE" CORRESPONDING TO THAT PEAK),
C                       ASSUMING THAT THE TOTAL INFLOWING CURRENT FOR THE M=1 MODE EQUALS 1 MEGAAMP
C
      CC=1000.D0/63.712D0   ! SO THAT WITH INPUT I IN MA AND DISTANCES IN RE,
C                              THE OUTPUT B IS IN NANOTESLA  (1RE = 6371.2 km)
       DO I=1,15
        DD(I)=D0*RR(I)*DSQRT(RR(I))    !  HALF-THICKNESS OF THE WIRE GROWS WITH DISTANCE AS R^(3/2)
c       DD(I)=D0*RR(I)                  !  HALF-THICKNESS OF THE WIRE GROWS WITH DISTANCE AS R
       ENDDO
C
       SPS=DSIN(PS)
       CPS=DCOS(PS)

      DO 1 K=1,MW     !   SUMMATION OVER MW WIRES
       PHI=(K-0.5D0)*DPHI
       CP(K)=DCOS(PHI)
       SP(K)=DSIN(PHI)

       Theta0N=Theta00+DTheta00*DSIN(PHI*0.5d0)**2 -0.07D0*PS*CP(K) 
       Theta0S=(PI-Theta00)-DTheta00*DSIN(PHI*0.5d0)**2-0.07D0*PS*CP(K) 

C   the footpoint colatitudes Theta0N and Theta0S are functions of the azimuthal angle PHI and of the tilt angle
C                                                                 (see Tsyganenko, Space Sci.Rev.,1990, Fig.52)
         DO 2 I=1,15                                                
          R=RR(I)
          F=R**N+1.D0/(DSIN(Theta0N))**(2*N)-1.0D0
          FN=F**(1.0D0/DFLOAT(N))
          STHETA=DSQRT(R/FN)
          RHOSM=R*STHETA
          IF (I.EQ.1) THEN
           THETA_S=DASIN(STHETA)
          ELSE
           H=RH0*SPS/CPS*((1.D0+(RHOSM/RH0)**ALPHA)**(1.D0/ALPHA)-1.D0)  !  DEVIATION OF THE [TS.ET AL.2015] EQ.SHEET FROM SM EQUATOR
           THETA_S=DASIN(STHETA)+H/R*STHETA
          ENDIF
          RHOSM_S=R*DSIN(THETA_S)
          XXN(I,K)=RHOSM_S*CP(K)
          YYN(I,K)=RHOSM_S*SP(K)
          ZZN(I,K)=R*DCOS(THETA_S)

          F=R**N+1.D0/(DSIN(Theta0S))**(2*N)-1.0D0
          FN=F**(1.0D0/DFLOAT(N))
          STHETA=DSQRT(R/FN)
          RHOSM=R*STHETA
          IF (I.EQ.1) THEN
           THETA_S=PI-DASIN(STHETA)
          ELSE
           H=RH0*SPS/CPS*((1.D0+(RHOSM/RH0)**ALPHA)**(1.D0/ALPHA)-1.D0)  !  DEVIATION OF THE [TS.ET AL.2015] EQ.SHEET FROM SM EQUATOR
           THETA_S=PI-DASIN(STHETA)+H/R*STHETA
          ENDIF
          RHOSM_S=R*DSIN(THETA_S)
          XXS(I,K)=RHOSM_S*CP(K)
          YYS(I,K)=RHOSM_S*SP(K)
          ZZS(I,K)=R*DCOS(THETA_S)
  2    CONTINUE
  1    CONTINUE
C
      ENDIF    !   END OF THE RE-INITIALIZATION OF ARRAYS DUE TO CHANGED VALUES OF Theta00 OR DTheta00
C---------------------------------------------------------------------------------------------------------
C
      BXSM=0.D0
      BYSM=0.D0
      BZSM=0.D0
C
      XSM=X*CPS-Z*SPS
      YSM=Y              
      ZSM=Z*CPS+X*SPS

      DO 40 K=1,MW  !   MW WIRES, SPACED BY 360/MW-DEGREE INTERVALS OF SM LONGITUDE
C
C  FIRST, CALCULATE CONTRIBUTION FROM THE NORTHERN WIRE:
C
         L=1
         X1= XXN(L,K)
         Y1= YYN(L,K)
         Z1= ZZN(L,K)
         D1= DD(L)
c
         HX=0.D0
         HY=0.D0
         HZ=0.D0

         DO 3 I=2,15   !   15 SEGMENTS IN EACH WIRE, REGARDLESS OF LONGITUDE
c
          L=L+1
          X2=XXN(L,K)
          Y2=YYN(L,K)
          Z2=ZZN(L,K)
          D2=DD(L)

        X1X2=X1-X2
        Y1Y2=Y1-Y2
        Z1Z2=Z1-Z2
        XX1=XSM-X1
        YY1=YSM-Y1
        ZZ1=ZSM-Z1
        D2D1=D2-D1

        A=X1X2**2+Y1Y2**2+Z1Z2**2+D2D1**2
        B=2.D0*(XX1*X1X2+YY1*Y1Y2+ZZ1*Z1Z2+D1*D2D1)
        C=XX1**2+YY1**2+ZZ1**2+D1**2

       FACTOR=2.D0*CC*(DSQRT(1.D0/(A+B+C))/(2.D0*A+B
     *+2.D0*DSQRT(A*(A+B+C)))-DSQRT(1.D0/C)/(B+2.D0*DSQRT(A*C)))

       HX=HX+FACTOR*(Y1*(ZSM-Z2)-YSM*Z1Z2-Y2*ZZ1)
       HY=HY+FACTOR*(Z1*(XSM-X2)-ZSM*X1X2-Z2*XX1)
       HZ=HZ+FACTOR*(X1*(YSM-Y2)-XSM*Y1Y2-X2*YY1)
C
         X1=X2
         Y1=Y2
         Z1=Z2
   3     D1=D2
C
       S=SP(K)    !  modulates the current with SIN(PHI) (regular R1 mode, with FAC peaks at dawn and dusk)
c       S=CP(K)    !  modulates the current with COS(PHI) (N-M R1 mode, with FAC peaks at noon and midnight)
       BXSM=BXSM+HX*S
       BYSM=BYSM+HY*S
       BZSM=BZSM+HZ*S
C
C  NOW DO THE SAME FOR THE SOUTHERN WIRE:
C
         L=1
         X1= XXS(L,K)
         Y1= YYS(L,K)
         Z1= ZZS(L,K)
         D1= DD(L)
c
         HX=0.D0
         HY=0.D0
         HZ=0.D0

         DO 4 I=2,15   !   15 SEGMENTS IN EACH WIRE, REGARDLESS OF LONGITUDE
c
          L=L+1
          X2=XXS(L,K)
          Y2=YYS(L,K)
          Z2=ZZS(L,K)
          D2=DD(L)

        X1X2=X1-X2
        Y1Y2=Y1-Y2
        Z1Z2=Z1-Z2
        XX1=XSM-X1
        YY1=YSM-Y1
        ZZ1=ZSM-Z1
        D2D1=D2-D1

        A=X1X2**2+Y1Y2**2+Z1Z2**2+D2D1**2
        B=2.D0*(XX1*X1X2+YY1*Y1Y2+ZZ1*Z1Z2+D1*D2D1)
        C=XX1**2+YY1**2+ZZ1**2+D1**2

       FACTOR=2.D0*CC*(DSQRT(1.D0/(A+B+C))/(2.D0*A+B
     *+2.D0*DSQRT(A*(A+B+C)))-DSQRT(1.D0/C)/(B+2.D0*DSQRT(A*C)))

       HX=HX+FACTOR*(Y1*(ZSM-Z2)-YSM*Z1Z2-Y2*ZZ1)
       HY=HY+FACTOR*(Z1*(XSM-X2)-ZSM*X1X2-Z2*XX1)
       HZ=HZ+FACTOR*(X1*(YSM-Y2)-XSM*Y1Y2-X2*YY1)
C
         X1=X2
         Y1=Y2
         Z1=Z2
   4     D1=D2
C
        S=SP(K)    !  modulates the current with SIN(PHI) (regular R1 mode, with FAC peaks at dawn and dusk)
c       S=CP(K)    !  modulates the current with COS(PHI) (N-M R1 mode, with FAC peaks at noon and midnight)
       BXSM=BXSM+HX*S
       BYSM=BYSM+HY*S
       BZSM=BZSM+HZ*S

  40  CONTINUE
C
      BX=(BXSM*CPS+BZSM*SPS)*CURDPHI
      BY= BYSM  *CURDPHI
      BZ=(BZSM*CPS-BXSM*SPS)*CURDPHI
C
      RETURN
      END
C
C--------------------------------------------------------------------------------------------------------------------
C
      SUBROUTINE R1_FAC_A (PS,X,Y,Z,BX,BY,BZ)  !     THE SUFFIX "A" CORRESPONDS TO TILT-ANTISYMMETRIC MODE OF R1 FACs            
C                                              !               (that is, downward at NORTHERN DAWN, and SOUTHERN DUSK
c                                                                       upward at NORTHERN DUSK, and SOUTHERN DAWN 
c                                              ! must be called after R1_FAC_C (where the common arrays are prepared)
c
c   This is N-S antisymmetric mode, i.e., Jx(-z)=-Jx(z), Jy(-z)=-Jy(z), and Jz(-z)=Jz(z)
C
c   INPUTS: X,Y,Z -  GSM position of the observation point (RE),
c
c   OUTPUT: BX,BY,BZ - field GSM components (nT)
c
      IMPLICIT REAL*8 (A-H,O-Z)
C
      COMMON /XYZD/ CURDPHI,DD(15),SP(25),CP(25),XXN(15,25),YYN(15,25),
     *  ZZN(15,25),XXS(15,25),YYS(15,25),ZZS(15,25)
      DATA MW/25/    !   number of wires, spanning 360-degs of the R1 FAC oval
      DATA CC/15.69563D0/  !  CC=1000.D0/63.712D0  SO THAT, WITH I IN MA AND DISTANCES IN RE, 
C                               THE OUTPUT IS IN NT (1RE = 6371.2 km)
      SPS=DSIN(PS)
      CPS=DCOS(PS)
C
      BXSM=0.D0
      BYSM=0.D0
      BZSM=0.D0
C
      XSM=X*CPS-Z*SPS
      YSM=Y              
      ZSM=Z*CPS+X*SPS

      DO 40 K=1,MW  !   MW WIRES, SPACED BY 360/MW-DEGREE INTERVALS OF SM LONGITUDE
C
C  FIRST, CALCULATE CONTRIBUTION FROM THE NORTHERN WIRE:
C
         L=1
         X1= XXN(L,K)
         Y1= YYN(L,K)
         Z1= ZZN(L,K)
         D1= DD(L)
c
         HX=0.D0
         HY=0.D0
         HZ=0.D0

         DO 3 I=2,15   !   15 SEGMENTS IN EACH WIRE, REGARDLESS OF LONGITUDE
c
          L=L+1
          X2=XXN(L,K)
          Y2=YYN(L,K)
          Z2=ZZN(L,K)
          D2=DD(L)

        X1X2=X1-X2
        Y1Y2=Y1-Y2
        Z1Z2=Z1-Z2
        XX1=XSM-X1
        YY1=YSM-Y1
        ZZ1=ZSM-Z1
        D2D1=D2-D1

        A=X1X2**2+Y1Y2**2+Z1Z2**2+D2D1**2
        B=2.D0*(XX1*X1X2+YY1*Y1Y2+ZZ1*Z1Z2+D1*D2D1)
        C=XX1**2+YY1**2+ZZ1**2+D1**2

       FACTOR=2.D0*CC*(DSQRT(1.D0/(A+B+C))/(2.D0*A+B
     *+2.D0*DSQRT(A*(A+B+C)))-DSQRT(1.D0/C)/(B+2.D0*DSQRT(A*C)))

       HX=HX+FACTOR*(Y1*(ZSM-Z2)-YSM*Z1Z2-Y2*ZZ1)
       HY=HY+FACTOR*(Z1*(XSM-X2)-ZSM*X1X2-Z2*XX1)
       HZ=HZ+FACTOR*(X1*(YSM-Y2)-XSM*Y1Y2-X2*YY1)
C
         X1=X2
         Y1=Y2
         Z1=Z2
   3     D1=D2
C
       S=SP(K)    !  modulates the current with SIN(PHI) (regular R1 mode, with FAC peaks at dawn and dusk)
c      S=CP(K)    !  modulates the current with COS(PHI) (N-M R1 mode, with FAC peaks at noon and midnight)
       BXSM=BXSM+HX*S
       BYSM=BYSM+HY*S
       BZSM=BZSM+HZ*S
C
C  NOW DO THE SAME FOR THE SOUTHERN WIRE:
C
         L=1
         X1= XXS(L,K)
         Y1= YYS(L,K)
         Z1= ZZS(L,K)
         D1= DD(L)
c
         HX=0.D0
         HY=0.D0
         HZ=0.D0

         DO 4 I=2,15   !   15 SEGMENTS IN EACH WIRE, REGARDLESS OF LONGITUDE
c
          L=L+1
          X2=XXS(L,K)
          Y2=YYS(L,K)
          Z2=ZZS(L,K)
          D2=DD(L)

        X1X2=X1-X2
        Y1Y2=Y1-Y2
        Z1Z2=Z1-Z2
        XX1=XSM-X1
        YY1=YSM-Y1
        ZZ1=ZSM-Z1
        D2D1=D2-D1

        A=X1X2**2+Y1Y2**2+Z1Z2**2+D2D1**2
        B=2.D0*(XX1*X1X2+YY1*Y1Y2+ZZ1*Z1Z2+D1*D2D1)
        C=XX1**2+YY1**2+ZZ1**2+D1**2

       FACTOR=-2.D0*CC*(DSQRT(1.D0/(A+B+C))/(2.D0*A+B              !   here is the only difference from the C-mode - the southern
     *+2.D0*DSQRT(A*(A+B+C)))-DSQRT(1.D0/C)/(B+2.D0*DSQRT(A*C)))   !   currents have opposite polarity, hence the "minus" sign 

       HX=HX+FACTOR*(Y1*(ZSM-Z2)-YSM*Z1Z2-Y2*ZZ1)
       HY=HY+FACTOR*(Z1*(XSM-X2)-ZSM*X1X2-Z2*XX1)
       HZ=HZ+FACTOR*(X1*(YSM-Y2)-XSM*Y1Y2-X2*YY1)
C
         X1=X2
         Y1=Y2
         Z1=Z2
   4     D1=D2
C
       S=SP(K)    !  modulates the current with SIN(PHI) (regular R1 mode, with FAC peaks at dawn and dusk)
c      S=CP(K)    !  modulates the current with COS(PHI) (N-M R1 mode, with FAC peaks at noon and midnight)
       BXSM=BXSM+HX*S
       BYSM=BYSM+HY*S
       BZSM=BZSM+HZ*S

  40  CONTINUE
C
      BX=(BXSM*CPS+BZSM*SPS)*CURDPHI
      BY= BYSM  *CURDPHI
      BZ=(BZSM*CPS-BXSM*SPS)*CURDPHI
C
      RETURN
      END
C
C===============================================================================================================
C
      SUBROUTINE R1_R_SHLD (X,Y,Z,PSI,BZIMF,TH,DTH,FX,FY,FZ)  
C
C   CALCULATES COMPONENTS OF THE SHIELDING FIELD OF THE REGULAR MODE (HENCE "R") OF THE R1 FAC SYSTEM 
C
C   INPUT:  X,Y,Z           - POSITION VECTOR (GSM)
C           PSI             - dipole tilt angle (rads)                                                            
C           BZIMF           - IMF BZ 
C           TH              - Theta00 - noon colatitude of R1 FAC oval
C           DTH             - DTheta00 increment of colatitude between noon and midnight        
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=600,NTOT=620)
C
        DIMENSION A(NTOT)
        DATA A/
     * -195.37843D0  ,  4.0415972D0  ,  4.7113514D0  , -3.8744123D0  ,
     * -40.288346D0  , -44.933970D0  , -34.771299D0  , -4.1392825D0  ,
     * -9.1949407D0  ,  3.4375187D0  , -14.690003D0  , -1.8278654D0  ,
     *  9.4635223D0  ,  2.1688159D0  ,  4.4106812D0  , -14.075658D0  ,
     * -1.2132597D0  , -3.5920627D0  , -6.8408246D0  ,-0.50219606D-01,
     *  35.895236D0  , -10.532599D0  , -11.343871D0  , -1.4318168D0  ,
     *  39.820113D0  ,  41.722271D0  ,  34.113998D0  ,  6.8919814D0  ,
     *  10.032531D0  , -3.2120097D0  , -36.498065D0  ,  6.9859034D0  ,
     * -15.980445D0  , -7.4590678D0  , -38.228081D0  , -27.749225D0  ,
     * -17.878699D0  , -2.3552159D0  ,  1.9604603D0  , -1.8444788D0  ,
     * -37.316416D0  , 0.25552844D0  ,  14.258028D0  ,  4.6653077D0  ,
     * -7.4586208D0  , -5.5528448D0  , -10.755532D0  , -2.7538461D0  ,
     *  1.1774179D0  , 0.11173789D0  ,  68.567426D0  , -12.392400D0  ,
     * -7.0841859D0  , 0.30683078D0  ,  21.324141D0  ,  3.9179089D0  ,
     *  14.375375D0  , -1.8236683D0  , -6.6669880D0  ,-0.22665402D0  ,
     * -76.185215D0  ,  13.953954D0  , -19.777938D0  , -11.825662D0  ,
     * -41.817962D0  , -7.2223298D0  , -26.783314D0  , 0.18609890D0  ,
     *  12.613857D0  ,  1.2118887D0  ,  10.144822D0  ,  2.5573858D0  ,
     *  15.731561D0  ,  2.7489696D0  ,  1.9420971D0  ,  3.8564871D0  ,
     *-0.90946907D-01, -2.8324736D0  , 0.12687770D0  , 0.57948952D0  ,
     *  29.978512D0  , -8.2864692D0  , -13.840230D0  , -2.7516834D0  ,
     *  5.8150059D0  , -6.3457287D0  ,  4.8497116D0  , 0.87199975D0  ,
     * -5.7967093D0  , 0.21239467D0  , -47.806040D0  ,  1.8699726D0  ,
     *  2.6479197D0  , 0.54729405D-01, -8.9777607D0  ,  2.9553405D0  ,
     * -5.9843176D0  ,  4.6168902D0  ,  2.7046737D0  , 0.53103741D0  ,
     *  18.580036D0  , -6.0476852D0  ,  3.1427676D0  ,  6.8490488D0  ,
     *  27.053706D0  ,  20.283200D0  ,  15.715324D0  , 0.56613706D0  ,
     *  2.1623511D0  , 0.33961763D0  ,  32.100088D0  , -4.8866117D0  ,
     *  12.924253D0  ,  5.8920024D0  ,  26.630711D0  ,  18.238599D0  ,
     *  14.861890D0  , -1.2331960D0  , 0.86930894D0  , 0.71662766D0  ,
     *  54.826855D0  , 0.18389778D0  , -4.9380264D0  , 0.23768713D0  ,
     *  22.957604D0  ,  26.044418D0  ,  20.478867D0  ,  3.8723309D0  ,
     *  5.8427036D0  , -2.1745528D0  , -19.453570D0  ,  8.1466283D0  ,
     * -7.4827892D0  , -3.5858094D0  , -17.503040D0  , -5.2623141D0  ,
     * -7.1810672D0  , 0.45213642D0  ,  4.6693435D0  ,-0.93436551D0  ,
     * -3.8183290D0  , 0.96060501D0  ,  18.627278D0  ,  7.3076167D0  ,
     * 0.69197192D0  , -11.809902D0  , -5.1586410D0  , -3.2005277D0  ,
     * -7.0936441D0  , 0.94416264D0  ,  47.301815D0  , -8.0711451D0  ,
     * -4.5154071D0  ,  1.6425972D0  ,  22.686540D0  ,  8.5703158D0  ,
     *  14.055783D0  ,  1.0379988D0  , -5.1312304D0  , 0.10898832D0  ,
     *  13.112895D0  ,  4.1428441D0  , -11.163097D0  , -2.8483741D0  ,
     *  7.6506500D0  ,  17.113210D0  ,  8.0514543D0  ,  2.2710154D0  ,
     *  7.9103326D0  , 0.39080202D0  , -2.5606897D0  ,  4.9691135D0  ,
     *  12.623043D0  ,  2.7801199D0  ,-0.62472878D0  ,  1.0660781D0  ,
     * -1.1269195D0  , -1.5641197D0  ,  1.0650766D0  , 0.39323638D0  ,
     *  40.935585D0  , -2.1769484D0  ,  4.8806390D0  ,  4.0958495D0  ,
     *  19.533863D0  ,  6.5828492D0  ,  12.716765D0  , 0.62841618D0  ,
     * -2.7156363D0  , 0.37483101D0  , -2.1719868D0  , 0.24281508D0  ,
     * -1.4238354D0  , 0.53363149D0  ,  3.3997936D0  ,  4.3179975D0  ,
     *  3.0070405D0  ,  1.9244954D0  , 0.97303874D0  , 0.16622487D0  ,
     *  56.249846D0  , 0.99759795D0  , -9.0170971D0  ,-0.22892834D-02,
     *  2.4332286D0  ,  13.505042D0  ,  2.7273094D0  , 0.85286147D0  ,
     *  6.2476052D0  ,  1.1668279D0  ,  21.425502D0  , -3.5858604D0  ,
     *  11.366853D0  ,  4.4198128D0  ,  23.833542D0  ,  31.437872D0  ,
     *  12.800404D0  ,  1.5921705D0  ,  7.2102793D0  ,  2.0118020D0  ,
     * -13.326060D0  ,  13.516557D0  , -5.5521307D0  , -7.6348338D0  ,
     * -43.517188D0  , -30.914650D0  , -26.136892D0  , -1.5207673D0  ,
     * -4.0960436D0  , 0.39359664D0  , -50.347672D0  ,  8.6261305D0  ,
     * -3.6101806D0  , -3.0502790D0  , -15.744930D0  ,  4.7731851D0  ,
     * -10.537163D0  ,  4.0519305D0  ,  7.2375283D0  , 0.24705012D0  ,
     * -10.015663D0  , -4.9107503D0  ,  17.049900D0  ,  5.0112081D0  ,
     * -13.155088D0  , -39.355624D0  , -13.154626D0  , -4.9301854D0  ,
     * -20.182232D0  ,  1.4104714D0  , -18.380964D0  , -4.9058427D0  ,
     * -6.3258538D0  , 0.14983791D0  ,  5.4076778D0  , 0.81244368D0  ,
     * 0.38398144D0  ,  4.1942780D0  , -3.2145544D0  , 0.28927988D0  ,
     *  73.923797D0  , -15.147222D0  , -8.8879035D0  ,  2.6011473D0  ,
     *  44.018005D0  ,  31.574038D0  ,  35.293598D0  ,  3.4923259D0  ,
     *  1.1570867D0  , -2.2002833D0  , -44.188137D0  ,  5.3891850D0  ,
     *  3.5639276D0  ,-0.49887963D0  , -19.871951D0  , -14.308017D0  ,
     * -13.614183D0  , -1.3195984D0  ,  2.0342390D0  ,-0.35502973D0  ,
     * -67.697354D0  ,  9.4404628D0  ,  18.362139D0  ,  1.0058470D0  ,
     * -27.102740D0  , -11.391561D0  , -21.357225D0  , -2.7836197D0  ,
     *  2.6675338D0  ,-0.53227508D0  ,  52.798195D0  , -4.6978395D0  ,
     * -11.966006D0  , -2.6336658D0  ,  8.5526304D0  , -3.3786951D0  ,
     *  6.4437157D0  , -4.3903650D0  , -2.9090069D0  ,-0.90292220D0  ,
     * -51.538154D0  ,  1.7151844D0  ,  4.3185658D0  , -3.8777125D0  ,
     * -15.329524D0  , -6.4979235D0  , -12.819149D0  , -8.1285503D0  ,
     *  4.9989575D0  , -6.1421902D0  , -8.9984145D0  , 0.27577879D0  ,
     * -2.8189280D0  , -4.6639381D0  , -14.429005D0  ,-0.39993961D-01,
     *  3.8626173D0  , -7.4937793D0  ,  5.7972079D0  , -7.6863202D0  ,
     * -37.592792D0  , -6.2852011D0  ,  7.3073436D0  ,  3.8185854D0  ,
     *  7.4141533D0  , -2.4605615D0  ,  2.2687465D0  , 0.91067867D0  ,
     *  5.0105457D0  ,  3.8264403D0  , -18.693893D0  , -4.0241736D0  ,
     * -4.8465493D0  , -1.4068423D0  , -3.9087303D0  , -8.4383115D0  ,
     * 0.60251699D0  , 0.80978245D0  ,  5.0777901D0  , 0.67118076D0  ,
     * -30.341660D0  ,-0.21426767D0  ,  7.3199788D0  , -1.3941479D0  ,
     * -12.654811D0  , -9.2811407D0  , -5.7703032D0  , 0.46057307D0  ,
     *  3.1177503D0  ,  3.2379333D0  ,  14.136453D0  , -2.8220574D0  ,
     *  12.958732D0  ,  3.5413501D0  ,  16.501701D0  ,  1.1323695D0  ,
     *-0.80629852D0  ,  2.5160334D0  , -4.0292649D0  ,  4.3201633D0  ,
     * -2.6913222D0  ,  5.6773483D0  , -73.398314D0  , -21.778012D0  ,
     *  3.4835716D0  ,  19.225979D0  ,  8.8561544D0  ,-0.89693700D0  ,
     *  2.9567284D0  , -2.7231488D0  ,  4.9163078D0  ,  7.9785416D0  ,
     * -23.316471D0  , -4.1985871D0  ,  19.694066D0  ,  17.706385D0  ,
     *  5.8257647D0  , 0.29198249D0  ,-0.14646616D0  ,  1.1244238D0  ,
     *  19.327001D0  ,  4.7062149D0  , -24.146253D0  , -9.7951966D0  ,
     *  18.288684D0  , -5.8423771D0  ,  3.6263372D0  ,  2.5412824D0  ,
     * -11.347229D0  ,  6.0260470D0  ,  14.541660D0  ,  2.7592937D0  ,
     * -27.738786D0  , -9.2360547D0  ,  10.107713D0  ,  4.0731198D0  ,
     *  9.4694512D0  ,  1.1507385D0  , -2.6198258D0  , 0.89095188D0  ,
     *  94.338058D0  ,  7.9935917D0  , -15.264231D0  , 0.41608016D0  ,
     *  26.384307D0  ,  7.1399193D0  ,  11.056054D0  ,  8.7971599D0  ,
     * -15.077341D0  ,  4.8890058D0  ,  44.628653D0  , 0.91180419D0  ,
     * -1.2572589D0  , 0.15109582D0  ,  10.876046D0  , 0.10168736D0  ,
     *  3.0639823D0  ,  3.9365562D0  , -13.209234D0  ,  4.8321838D0  ,
     *  44.402404D0  ,  2.2815647D0  , -8.1351925D0  ,  3.6496146D0  ,
     *  18.732453D0  ,  15.596899D0  ,  12.468623D0  ,  4.1239651D0  ,
     * -5.7539145D0  ,-0.18156353D0  , -3.5160004D0  ,  2.3564493D0  ,
     * -11.657260D0  , -1.4399875D0  , -10.262749D0  , -4.2290701D0  ,
     * -4.5272052D0  ,  3.4277391D0  , -2.7895621D0  , -1.0477540D0  ,
     *  15.971975D0  ,-0.81653628D0  ,  40.538470D0  ,  11.911440D0  ,
     * -12.472785D0  , -3.1337690D0  , -5.1231272D0  , -1.3678346D0  ,
     * 0.96612263D0  , -2.2779325D0  ,  7.4829803D0  , -3.1057415D0  ,
     *  24.358413D0  ,  6.6695230D0  , -8.8908910D0  , -4.6806062D0  ,
     * -8.2125543D0  , 0.34567928D-02, -2.9490625D0  ,-0.93474576D0  ,
     *  6.5922005D0  , -5.6909256D0  ,  23.711355D0  ,  10.454826D0  ,
     * -11.151205D0  ,  5.2714039D0  , -1.5349970D0  , -2.0364751D0  ,
     *  8.2713571D0  , -6.0314579D0  , -23.066997D0  , -1.9055690D0  ,
     *  24.809892D0  ,  7.4519983D0  , -17.303990D0  , -4.9698872D0  ,
     * -8.6692266D0  , -2.7916504D0  ,  5.0007343D0  , -2.2756433D0  ,
     * -10.912519D0  , -9.7835613D0  ,  72.566538D0  ,  23.908773D0  ,
     * -20.388305D0  , -13.698632D0  , -10.901799D0  , -2.6220785D0  ,
     *  6.4224279D0  , -4.2336067D0  , -25.153578D0  , -7.1228482D0  ,
     *  37.272253D0  ,  9.8549369D0  , -25.831480D0  , -14.359420D0  ,
     * -11.771693D0  , -2.2791917D0  ,  5.5600220D0  , -3.4272112D0  ,
     * -31.659134D0  , -8.8201062D0  ,  19.030242D0  ,  2.7338837D0  ,
     * -24.644335D0  , -12.060517D0  , -8.5081081D0  , -3.3034870D0  ,
     *  11.625792D0  ,-0.38816515D0  , -18.187538D0  , -4.6322807D0  ,
     *  29.019947D0  ,  8.6686018D0  ,  7.6660369D0  ,  3.5585597D0  ,
     * -3.4752827D0  , 0.39124437D0  ,  6.5746802D0  ,  4.0903724D0  ,
     * -9.1816355D0  ,  3.2634813D0  , -33.646079D0  , -10.304648D0  ,
     *  4.7289115D0  ,  9.3336558D0  ,  4.7529946D0  , -3.7218258D0  ,
     * -1.8341768D0  , -3.8550480D0  , -13.279896D0  ,  7.6325649D0  ,
     * -31.025743D0  , -7.8032047D0  , -1.2848641D0  ,  8.9106132D0  ,
     *  2.8055825D0  ,-0.55142165D0  ,  3.7200418D0  , -4.4192516D0  ,
     *  2.5672564D0  ,  3.3466923D0  ,  10.302698D0  ,-0.79022668D0  ,
     *  1.9119924D0  , -11.663306D0  , -5.9006022D0  , -1.5455202D0  ,
     * -7.2920701D0  ,  1.9717978D0  ,  18.660507D0  , 0.22939795D0  ,
     * -11.639152D0  , -4.7419391D0  , 0.92337001D0  ,  1.0312834D0  ,
     *  6.2241722D0  , -1.8675722D0  , -2.5994488D0  , -2.2088180D0  ,
     *  5.6729598D0  , -1.9163577D0  ,  6.9429642D0  ,  2.5975492D0  ,
     *  15.154817D0  , -8.6517013D0  , 0.58815928D0  ,  5.7410250D0  ,
     * -4.5560073D0  ,  6.0609943D0  , -5.1303529D0  , -3.0157456D0  ,
     * -1.4544771D0  , -2.1681495D0  , -2.5465926D0  , -9.9109914D0  ,
     *  4.5132911D0  , 0.34353320D-01,-0.52332625D0  ,  2.6309107D0  ,
     * -9.6387115D0  ,  5.4870403D0  , -37.550227D0  , -11.837462D0  ,
     *  1.2337675D0  ,  15.068695D0  ,  5.6790059D0  ,-0.82180690D0  ,
     *  2.3215222D0  , -1.0103627D0  ,  15.441083D0  ,  3.6796557D0  ,
     * -12.236272D0  , -1.7153865D0  ,  15.169027D0  ,  9.7082687D0  ,
     *  2.1467077D0  ,  1.9135164D0  , -3.2289422D0  ,  1.8556578D0  ,
     * 0.13279370D-01, 0.94441644D-04, 0.86814291D-03, 0.11507320D-02,
     * 0.61436755D-03, 0.28783052D-01,-0.17508923D-02, 0.15285429D-02,
     * 0.67371711D-04, 0.97976664D-03, 0.20882977D-01,-0.32760591D-03,
     * 0.12917751D-02, 0.65106370D-04, 0.22257649D-03, 0.38003076D-01,
     *-0.13947321D-02, 0.22317608D-02, 0.12923790D-02, 0.20669744D-02/
C
        DATA PSI_LOW,PSI_HIGH,BZIMF_LOW,BZIMF_HIGH,T_LOW,T_HIGH,DT_LOW,
     *   DT_HIGH /-0.6D0,0.6D0,-7.D0,7.D0,0.13000D0,0.300000D0,
     *   0.1100000D0,0.2500000D0/
C
        FPS=(PSI*2.D0-PSI_HIGH-PSI_LOW)/(PSI_HIGH-PSI_LOW)               !  ALL NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FPS2=FPS**2
        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   
        FBZ21=DABS(FBZ) 
        FTH  =(TH*2.D0-T_HIGH-T_LOW)/(T_HIGH-T_LOW)
        FDTH =(DTH*2.D0-DT_HIGH-DT_LOW)/(DT_HIGH-DT_LOW)

        A0=A(NLIN+1)+A(NLIN+2)*FBZ+A(NLIN+3)*FBZ21
     *   +A(NLIN+4)*FTH+A(NLIN+5)*FDTH
        B0=A(NLIN+6)+A(NLIN+7)*FBZ+A(NLIN+8)*FBZ21
     *   +A(NLIN+9)*FTH+A(NLIN+10)*FDTH

        FX=0.D0
        FY=0.D0
        FZ=0.D0

        IND=-19

        DO 1 I=1,3 
        DO 1 K=1,5 

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         IND=IND+20

         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FTH
     *     +A(IND+3) *FDTH 
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FTH**2
     *     +A(IND+6) *FDTH**2
     *     +A(IND+7) *FBZ*FTH 
     *     +A(IND+8) *FTH *FDTH
     *     +A(IND+9) *FBZ*FDTH
     *     +A(IND+10)*FPS2
     *     +A(IND+11)*FBZ*FPS2
     *     +A(IND+12)*FTH*FPS2
     *     +A(IND+13)*FDTH*FPS2 
     *     +A(IND+14)*FBZ21*FPS2
     *     +A(IND+15)*FTH**2*FPS2
     *     +A(IND+16)*FDTH**2*FPS2
     *     +A(IND+17)*FBZ*FTH*FPS2 
     *     +A(IND+18)*FTH *FDTH*FPS2
     *     +A(IND+19)*FBZ*FDTH*FPS2
C
         FX=FX+AI*BX
         FY=FY+AI*BY
         FZ=FZ+AI*BZ
C
  1      CONTINUE
C
C  - ----------       NOW PARALLEL COMPONENT:
C
        A1=A(NLIN+11)+A(NLIN+12)*FBZ+A(NLIN+13)*FBZ21
     *   +A(NLIN+14)*FTH+A(NLIN+15)*FDTH
        B1=A(NLIN+16)+A(NLIN+17)*FBZ+A(NLIN+18)*FBZ21
     *   +A(NLIN+19)*FTH+A(NLIN+20)*FDTH

        DO 2 I=1,3 
        DO 2 K=1,5 

         A_I=A1*I
         B_K=B1*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*CBZ
         BY=-A_I*EX*SAY*CBZ
         BZ=-B_K*EX*CAY*SBZ

         IND=IND+20

         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FTH
     *     +A(IND+3) *FDTH 
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FTH**2
     *     +A(IND+6) *FDTH**2
     *     +A(IND+7) *FBZ*FTH 
     *     +A(IND+8) *FTH *FDTH
     *     +A(IND+9) *FBZ*FDTH
     *     +A(IND+10)*FPS2
     *     +A(IND+11)*FBZ*FPS2
     *     +A(IND+12)*FTH*FPS2
     *     +A(IND+13)*FDTH*FPS2 
     *     +A(IND+14)*FBZ21*FPS2
     *     +A(IND+15)*FTH**2*FPS2
     *     +A(IND+16)*FDTH**2*FPS2
     *     +A(IND+17)*FBZ*FTH*FPS2 
     *     +A(IND+18)*FTH *FDTH*FPS2
     *     +A(IND+19)*FBZ*FDTH*FPS2

         FX=FX+AI*BX*FPS
         FY=FY+AI*BY*FPS
         FZ=FZ+AI*BZ*FPS

  2      CONTINUE

         RETURN   
C
         END
C
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C
      SUBROUTINE R1_A_SHLD (X,Y,Z,PSI,BZIMF,TH,DTH,FX,FY,FZ)
C
C   CALCULATES COMPONENTS OF THE SHIELDING FIELD OF THE ANTISYMMETRIC MODE (HENCE "A") OF THE R1 FAC SYSTEM 
C
C   INPUT:  X,Y,Z           - POSITION VECTOR (GSM)
C           PSI             - dipole tilt angle (rads)                                                            
C           BZIMF           - IMF BZ 
C           TH              - Theta00 - noon colatitude of R1 FAC oval
C           DTH             - DTheta00 increment of colatitude between noon and midnight        
C
C      OUTPUT: FX,FY,FZ        - TOTAL SHIELDING FIELD COMPONENTS
C
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
        IMPLICIT  REAL * 8  (A - H, O - Z)
C
        PARAMETER (NLIN=600,NTOT=620)
C
        DIMENSION A(NTOT)
        DATA A/
     * -15.271347D0  ,  1.6455582D0  ,  10.794049D0  ,  6.0306370D0  ,
     * -3.7796018D0  , -2.8885203D0  , -2.9401536D0  ,-0.11517207D-01,
     *-0.95736669D0  , 0.14868986D0  , -4.9341796D0  ,-0.39183480D0  ,
     *  8.3385044D0  ,  4.1259764D0  ,-0.19308437D0  , 0.75095926D0  ,
     *-0.66509934D0  ,-0.33958269D0  ,-0.21049905D0  , 0.54950447D0  ,
     * -20.245188D0  ,  1.2002129D0  ,  8.6501459D0  ,  5.0327667D0  ,
     * -3.9696399D0  ,-0.97747152D-01, -2.5714432D0  , 0.34820790D0  ,
     *  1.3263432D0  ,-0.10239796D0  , -5.8123136D0  ,  1.5406234D0  ,
     *  2.1156842D0  ,  2.6777544D0  , 0.51059055D0  ,  2.1020729D0  ,
     * 0.41322944D0  ,  1.2779064D0  , 0.49081057D0  , 0.34228743D0  ,
     * -17.817429D0  ,  1.5597554D0  , -5.1901082D0  , -1.2148744D0  ,
     * -2.4417820D0  ,  5.6969302D0  ,-0.12102459D0  ,  1.2019761D0  ,
     *  3.9745854D0  ,-0.46030737D0  ,-0.90364498D0  , 0.60152450D-01,
     * -2.4436798D0  , 0.10173255D0  ,  3.4493795D0  ,  6.2586263D0  ,
     *  2.3222067D0  , 0.31817308D0  ,  1.8608735D0  ,-0.10522218D0  ,
     * -6.8820843D0  ,-0.27314442D0  , -13.906148D0  , -5.2933769D0  ,
     *  1.5731330D0  ,  6.7126454D0  ,  2.1538384D0  ,  1.4827180D0  ,
     *  3.9447130D0  ,-0.65058900D0  ,  1.6468335D0  ,-0.35812963D0  ,
     * -6.9564424D0  , -2.9201997D0  ,  3.8314242D0  ,  8.0418680D0  ,
     *  3.3335992D0  , 0.87916089D0  ,  2.6853615D0  , 0.10966281D0  ,
     *  7.2656848D0  ,-0.23581093D0  , 0.58898660D0  ,  1.0101912D0  ,
     * 0.48098264D0  , -6.5340398D0  ,-0.48577992D0  ,-0.16852617D0  ,
     * -2.7049354D0  , 0.34960232D0  ,-0.58969991D0  ,-0.93116760D-01,
     *  6.5205225D0  ,  1.3625373D0  , -3.0528431D0  , -2.2322458D0  ,
     * -2.4510668D0  ,-0.88218763D0  , -1.5160863D0  , 0.50509015D0  ,
     * -3.7700492D0  , 0.83926659D0  , 0.93972517D0  , 0.90487308D0  ,
     *-0.56041950D0  ,-0.68487029D0  ,-0.52442145D0  ,-0.80175281D-01,
     *-0.35872245D0  ,-0.13644167D0  ,-0.74689715D0  ,-0.69208022D0  ,
     *  2.3492683D0  , 0.72703004D0  , 0.25468799D0  ,  1.1596298D0  ,
     * 0.63014265D-01,-0.44609039D0  , 0.35440889D0  , 0.18849753D0  ,
     * -3.9282029D0  ,-0.27465231D0  ,  2.5988569D0  ,  1.2623058D0  ,
     *-0.35005605D0  ,-0.19941512D0  ,-0.38035664D0  ,-0.16283901D-01,
     * 0.25995450D0  ,-0.63761202D-01, -1.4549721D0  , 0.80756960D0  ,
     * -1.0589640D0  , 0.10273842D0  ,-0.15865484D0  , 0.15646497D0  ,
     * 0.12046765D0  , 0.88456828D0  , 0.27310353D-01, 0.12749888D0  ,
     * -3.5400198D0  , 0.52936534D0  , -1.2706701D0  ,-0.44412937D0  ,
     *-0.84449367D0  ,  1.4171252D0  , 0.10826550D-01, 0.28098975D0  ,
     * 0.88322759D0  , 0.22113253D-01, 0.56657521D0  ,-0.29464927D0  ,
     * 0.28566963D0  , 0.31361521D0  , 0.79652763D0  , 0.89884955D0  ,
     * 0.21814103D0  ,-0.35984978D0  , 0.52874298D-01,-0.10309188D0  ,
     *-0.98187577D0  ,-0.12658854D0  , -2.6852877D0  ,-0.94813949D0  ,
     * 0.45426445D0  ,  1.0947740D0  , 0.35495148D0  , 0.22988848D0  ,
     * 0.65912711D0  ,-0.19000311D0  ,-0.12128984D0  , 0.62978988D-01,
     * -1.3626784D0  ,-0.63676349D0  , 0.64950842D0  ,  1.6687608D0  ,
     * 0.70220448D0  , 0.25646153D0  , 0.64548394D0  , 0.28869747D-01,
     *  1.2766155D0  , 0.80824494D-02, 0.34169410D0  , 0.26469269D0  ,
     * 0.11028794D-01, -1.3307173D0  ,-0.14910077D0  ,-0.76727833D-01,
     *-0.55949775D0  , 0.51985948D-01,-0.13283266D0  ,-0.11194929D-01,
     *  1.3195809D0  , 0.31139764D0  ,-0.63577909D0  ,-0.54538963D0  ,
     *-0.53904450D0  ,-0.20820291D0  ,-0.34271554D0  , 0.74115945D-01,
     *  14.744764D0  ,-0.53145884D0  , -14.471161D0  , -7.1582553D0  ,
     *  4.4989763D0  ,  2.5618190D0  ,  3.2795746D0  ,-0.14166576D0  ,
     * 0.43619356D0  ,-0.54915407D0  ,  5.8260177D0  , -1.2283109D0  ,
     * -6.9738932D0  , -4.6145226D0  , 0.81796983D0  ,  1.5809610D0  ,
     *  1.1281105D0  ,-0.58986145D0  ,  1.1530497D0  ,-0.36273720D0  ,
     *  22.759656D0  , -2.7101406D0  , -6.9004168D0  , -4.7322580D0  ,
     *  5.4513842D0  ,-0.61918332D0  ,  3.1242857D0  ,-0.58928693D0  ,
     * -1.6025682D0  , 0.37484820D-01,  5.5337346D0  ,-0.40620304D0  ,
     * -5.9935864D0  , -3.9932245D0  , -1.3928743D0  , -3.2375316D0  ,
     *-0.45053899D0  , 0.25856228D0  ,-0.80998403D0  ,-0.20042797D0  ,
     *  19.902912D0  , -1.1681881D0  ,  5.4888067D0  , 0.96777133D0  ,
     *  1.6895499D0  , -5.7814532D0  , 0.17962567D0  , -1.2088116D0  ,
     * -4.2593062D0  , 0.84851590D0  ,  2.8949595D0  ,-0.88022408D0  ,
     *  4.9435841D0  , 0.75208268D0  , -3.6514418D0  , -8.0624743D0  ,
     * -3.2861891D0  , -1.4477105D0  , -2.9595274D0  ,-0.77666524D-01,
     *  8.6846728D0  , 0.82202688D-01,  15.852551D0  ,  6.2380180D0  ,
     * -1.3994554D0  , -8.1068881D0  , -2.5738740D0  , -1.7677688D0  ,
     * -4.7093335D0  , 0.61532335D0  , -2.9996223D0  , 0.73196267D0  ,
     *  7.8040331D0  ,  3.1332764D0  , -4.5546231D0  , -8.8255868D0  ,
     * -3.5743152D0  ,-0.70465782D0  , -2.6879628D0  ,-0.67545690D-01,
     * -8.6095912D0  , 0.27881259D0  ,-0.72853789D0  , -1.2551575D0  ,
     *-0.56442154D0  ,  7.8232208D0  , 0.64723331D0  , 0.24016248D0  ,
     *  3.2422148D0  ,-0.36691553D0  , 0.96618466D0  , 0.23334320D-01,
     * -7.8388137D0  , -1.6157130D0  ,  3.6688026D0  ,  2.6166092D0  ,
     *  2.9028760D0  ,  1.0456438D0  ,  1.7553525D0  ,-0.56899779D0  ,
     *  28.658100D0  ,  1.3748015D0  , -19.699179D0  , -10.363964D0  ,
     *  6.7302157D0  ,  4.8852361D0  ,  3.9664115D0  ,  1.1124231D0  ,
     * 0.23591186D-01,-0.38879137D-01,  1.6086627D0  ,  1.7900439D0  ,
     * 0.32760757D0  ,-0.95989632D0  , -3.2103181D0  , -3.0728502D0  ,
     * -2.0623461D0  ,-0.41712777D0  , -1.3065570D0  , 0.45390196D0  ,
     *  16.918680D0  ,  2.5778391D0  , -7.9210732D0  , -5.5431623D0  ,
     *  1.0937879D0  ,  1.0697655D0  ,-0.44334859D0  ,-0.56039369D0  ,
     * 0.16106746D0  ,-0.14101467D0  , 0.32124323D0  , 0.34956103D0  ,
     * -5.9686606D0  , -2.0614439D0  , -2.4685552D0  , -3.9290120D0  ,
     * -1.1793542D0  ,-0.34293127D0  , -1.9829915D0  ,-0.27968113D0  ,
     *  17.206384D0  , 0.60588211D-01, -2.9892193D0  , -1.8801090D0  ,
     *  3.2879687D0  , -1.9402996D0  , 0.57042872D0  ,-0.53939826D0  ,
     * -3.3469087D0  ,-0.13274444D0  , -6.2240681D0  ,  1.8355204D0  ,
     *  2.1284208D0  ,-0.47472016D0  , -3.4904048D0  ,-0.16264498D0  ,
     * -2.4523527D0  , 0.46724794D-01, 0.22944981D0  ,-0.17774443D0  ,
     *  11.058877D0  , -1.5369009D0  ,  3.0285393D0  , 0.89442778D0  ,
     *-0.35247281D0  , -5.2151919D0  ,-0.73866222D0  ,-0.19786357D-01,
     * -3.5246426D0  ,-0.21293137D0  , -8.9984138D0  ,-0.52127875D0  ,
     *-0.72286490D0  ,-0.38615685D0  , -3.6457241D0  ,-0.65314502D-01,
     * -3.1311006D0  , 0.53364877D0  , 0.96775472D0  ,-0.38957320D-01,
     *  7.3142401D0  , -1.4937863D0  ,  6.6840662D0  ,  3.1210862D0  ,
     * -2.1008612D0  , -3.0756274D0  , -2.1320448D0  ,-0.72635450D0  ,
     * -2.4932463D0  ,-0.35244573D0  , -2.9446736D0  ,-0.45751048D0  ,
     *  1.2736560D0  , 0.72853475D0  , -5.8463491D0  , -6.6366170D0  ,
     * -3.4866725D0  ,-0.51077400D0  , -1.4522607D0  , 0.12547474D-01,
     * -22.271917D0  , -4.3006521D0  ,  14.693966D0  ,  10.707686D0  ,
     * -6.5674574D0  , -3.0274418D0  , -3.0829797D0  ,-0.10423623D0  ,
     *  1.6436359D0  ,-0.96382116D-01, -1.0802533D0  , -1.1004701D0  ,
     *  9.6740785D0  ,  5.3088323D0  ,  2.2878260D0  ,  3.3496046D0  ,
     *  1.3450454D0  , 0.62216024D0  , 0.64914279D0  ,-0.63041422D-01,
     * -22.569474D0  , -2.1015809D0  ,  9.8015664D0  ,  6.3442363D0  ,
     * -5.6177933D0  , 0.41512994D0  , -1.9005245D0  , 0.38357375D-01,
     *  3.9667645D0  , 0.36235727D0  ,  3.9240307D0  , -2.8495313D0  ,
     * 0.31571409D0  ,  1.9687928D0  ,  4.9592455D0  ,  2.0991413D0  ,
     *  3.7139562D0  ,-0.13699646D0  ,-0.40288973D0  ,-0.37876500D-01,
     * -17.246650D0  ,-0.63639505D0  , 0.81774006D0  ,  2.1439765D0  ,
     * -1.4099567D0  ,  1.8831939D0  , 0.46124140D0  , 0.75292234D0  ,
     *  2.6316429D0  , 0.46756760D0  ,  2.6403981D0  , 0.36476495D0  ,
     *  4.7251422D0  ,  1.6857628D0  ,  4.3796450D0  ,  4.6533856D0  ,
     *  3.1774415D0  , 0.27099773D-02,  1.1402869D0  , 0.21424338D0  ,
     * -16.129484D0  ,  1.1223483D0  , -2.4800571D0  ,-0.80199872D0  ,
     *-0.31533894D0  ,  2.9684968D0  ,  1.3154297D0  , 0.63822906D0  ,
     *  3.5418912D0  , 0.34736259D0  ,  5.6013686D0  ,-0.39980334D0  ,
     * -2.8014159D0  ,-0.44095632D0  ,  7.0921026D0  ,  6.4916702D0  ,
     *  4.5484625D0  , 0.56539747D0  ,  1.4776117D0  ,-0.41901881D-01,
     * -13.257860D0  ,  2.9923037D0  , -9.2499795D0  , -4.5923265D0  ,
     *  3.2409752D0  ,  6.8435096D0  ,  2.9923162D0  , 0.69521579D0  ,
     *  4.5134991D0  , 0.40443866D0  ,  10.292041D0  , 0.64830330D0  ,
     * -3.2106312D0  , -1.3201682D0  ,  8.6475305D0  ,  6.0254521D0  ,
     *  5.6590543D0  , 0.12529398D0  , 0.84212776D0  , 0.98205483D-01,
     *  10.514875D0  ,  2.9950672D0  , -1.6466923D0  , -2.7019114D0  ,
     *  2.0803189D0  , -3.3957506D0  , 0.60163500D0  ,-0.97476044D0  ,
     * -3.8814902D0  ,-0.43541934D-01, -4.7895029D0  ,  2.1570065D0  ,
     * -4.1647884D0  , -2.8612442D0  ,-0.66695727D0  ,  2.6819667D0  ,
     * -1.4084441D0  ,  1.1161515D0  ,  1.7538392D0  ,-0.46419258D0  ,
     *  4.5374642D0  ,-0.37759033D0  , 0.49917414D0  ,-0.95048593D0  ,
     *  1.6217139D0  , 0.82512846D0  ,  1.1817115D0  , 0.14621015D0  ,
     *-0.18874313D0  ,-0.12607646D0  ,  1.1376532D0  , -2.0373894D0  ,
     * -4.4027318D0  , -1.6545852D0  , 0.26318020D0  , -2.1844168D0  ,
     *-0.17155302D0  ,-0.56049358D0  ,-0.42054812D0  , 0.15966903D0  ,
     *  2.8251980D0  , 0.53641612D-01, -2.9961682D0  , -1.3370245D0  ,
     * 0.48834767D0  , 0.91620855D0  ,-0.16793073D0  , 0.63239697D0  ,
     *-0.52617606D0  ,-0.19742096D0  , 0.41812179D0  , 0.99606778D0  ,
     *  4.1724508D0  , 0.77979356D0  , -3.7570530D0  , -4.6878614D0  ,
     * -2.2182782D0  ,-0.78568775D0  ,-0.78714011D0  , 0.47883871D0  ,
     *  4.6188524D0  , 0.43323190D0  ,  2.2791890D0  , 0.51306074D0  ,
     *-0.66258366D-01, 0.50118903D0  ,-0.91648888D0  , -1.0542866D0  ,
     *-0.28370795D0  ,-0.20909786D0  ,  3.3100384D0  ,-0.40019597D-01,
     * -2.1903989D0  ,-0.18467609D0  , -2.1639749D0  , -4.1573028D0  ,
     *-0.91172148D0  ,-0.15371192D0  , -1.7381971D0  ,-0.28526189D0  ,
     *  5.8237338D0  , -1.4409654D0  ,  2.1910925D0  ,  1.4120539D0  ,
     * -1.0705890D0  , -3.3078473D0  ,-0.82434613D0  , 0.86620485D-01,
     * -1.9265913D0  ,-0.50649182D-01, -6.9711352D0  ,-0.11068921D0  ,
     *  2.8955611D0  , 0.73126502D0  , -3.0110206D0  ,-0.20011953D0  ,
     * -2.2363997D0  , 0.13554367D0  , 0.35218065D0  ,-0.26202084D-01,
     * 0.93470292D-02, 0.46516521D-04, 0.51572626D-03,-0.12140839D-02,
     * 0.13652830D-04, 0.67482845D-01,-0.43183269D-02, 0.10169675D-02,
     * 0.14055113D-04, 0.87104557D-03, 0.26922377D-01,-0.10044979D-02,
     * 0.14457557D-02, 0.60191484D-04, 0.20030915D-03, 0.54593113D-01,
     *-0.14994350D-02, 0.25290933D-02, 0.23759393D-02, 0.17313238D-02/
C
        DATA PSI_LOW,PSI_HIGH,BZIMF_LOW,BZIMF_HIGH,T_LOW,T_HIGH,DT_LOW,
     *   DT_HIGH /-0.6D0,0.6D0,-7.D0,7.D0,0.13000D0,0.300000D0,
     *   0.1100000D0,0.2500000D0/
C
        FPS=(PSI*2.D0-PSI_HIGH-PSI_LOW)/(PSI_HIGH-PSI_LOW)               !  ALL NORMALIZED PARAMETERS VARY BETWEEN -1 AND +1
        FPS2=FPS**2
        FBZ=(BZIMF*2.D0-BZIMF_HIGH-BZIMF_LOW)/(BZIMF_HIGH-BZIMF_LOW)   
        FBZ21=DABS(FBZ) 
        FTH  =(TH*2.D0-T_HIGH-T_LOW)/(T_HIGH-T_LOW)
        FDTH =(DTH*2.D0-DT_HIGH-DT_LOW)/(DT_HIGH-DT_LOW)

        A0=A(NLIN+1)+A(NLIN+2)*FBZ+A(NLIN+3)*FBZ21
     *   +A(NLIN+4)*FTH+A(NLIN+5)*FDTH
        B0=A(NLIN+6)+A(NLIN+7)*FBZ+A(NLIN+8)*FBZ21
     *   +A(NLIN+9)*FTH+A(NLIN+10)*FDTH

        FX=0.D0
        FY=0.D0
        FZ=0.D0

        IND=-19

        DO 1 I=1,3 
        DO 1 K=1,5 

         A_I=A0*I
         B_K=B0*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*SBZ
         BY=-A_I*EX*SAY*SBZ
         BZ= B_K*EX*CAY*CBZ

         IND=IND+20

         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FTH
     *     +A(IND+3) *FDTH 
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FTH**2
     *     +A(IND+6) *FDTH**2
     *     +A(IND+7) *FBZ*FTH 
     *     +A(IND+8) *FTH *FDTH
     *     +A(IND+9) *FBZ*FDTH
     *     +A(IND+10)*FPS2
     *     +A(IND+11)*FBZ*FPS2
     *     +A(IND+12)*FTH*FPS2
     *     +A(IND+13)*FDTH*FPS2 
     *     +A(IND+14)*FBZ21*FPS2
     *     +A(IND+15)*FTH**2*FPS2
     *     +A(IND+16)*FDTH**2*FPS2
     *     +A(IND+17)*FBZ*FTH*FPS2 
     *     +A(IND+18)*FTH *FDTH*FPS2
     *     +A(IND+19)*FBZ*FDTH*FPS2
C
         FX=FX+AI*BX*FPS
         FY=FY+AI*BY*FPS
         FZ=FZ+AI*BZ*FPS
C
  1      CONTINUE
C
C  - ----------       NOW PARALLEL COMPONENT:
C
        A1=A(NLIN+11)+A(NLIN+12)*FBZ+A(NLIN+13)*FBZ21
     *   +A(NLIN+14)*FTH+A(NLIN+15)*FDTH
        B1=A(NLIN+16)+A(NLIN+17)*FBZ+A(NLIN+18)*FBZ21
     *   +A(NLIN+19)*FTH+A(NLIN+20)*FDTH

        DO 2 I=1,3 
        DO 2 K=1,5 

         A_I=A1*I
         B_K=B1*K
         XK=DSQRT(A_I**2+B_K**2)
         EX=DEXP(XK*X)
         CAY=DCOS(A_I*Y)
         SAY=DSIN(A_I*Y)
         CBZ=DCOS(B_K*Z)
         SBZ=DSIN(B_K*Z)
         BX=XK*EX*CAY*CBZ
         BY=-A_I*EX*SAY*CBZ
         BZ=-B_K*EX*CAY*SBZ

         IND=IND+20
C
         AI=A(IND)
     *     +A(IND+1) *FBZ
     *     +A(IND+2) *FTH
     *     +A(IND+3) *FDTH 
     *     +A(IND+4) *FBZ21
     *     +A(IND+5) *FTH**2
     *     +A(IND+6) *FDTH**2
     *     +A(IND+7) *FBZ*FTH 
     *     +A(IND+8) *FTH *FDTH
     *     +A(IND+9) *FBZ*FDTH
     *     +A(IND+10)*FPS2
     *     +A(IND+11)*FBZ*FPS2
     *     +A(IND+12)*FTH*FPS2
     *     +A(IND+13)*FDTH*FPS2 
     *     +A(IND+14)*FBZ21*FPS2
     *     +A(IND+15)*FTH**2*FPS2
     *     +A(IND+16)*FDTH**2*FPS2
     *     +A(IND+17)*FBZ*FTH*FPS2 
     *     +A(IND+18)*FTH *FDTH*FPS2
     *     +A(IND+19)*FBZ*FDTH*FPS2
C
         FX=FX+AI*BX  
         FY=FY+AI*BY  
         FZ=FZ+AI*BZ  

  2      CONTINUE

         RETURN   
C
         END
C

C===================================================================================
c
       SUBROUTINE DIPOLE(PS,X,Y,Z,BX,BY,BZ)  ! USED ONLY IN THE DIPOLE_SHIELD SUBROUTINE AS AN IMAGE SOURCE 
C
C  CALCULATES GSM COMPONENTS OF GEODIPOLE FIELD WITH THE DIPOLE MOMENT
C  CORRESPONDING TO THE EPOCH OF 1980.
C------------INPUT PARAMETERS:
C   PS - GEODIPOLE TILT ANGLE IN RADIANS, X,Y,Z - GSM COORDINATES IN RE
C------------OUTPUT PARAMETERS:
C   BX,BY,BZ - FIELD COMPONENTS IN GSM SYSTEM, IN NANOTESLA.
C
C
C     AUTHOR: N. A. TSYGANENKO
C
      IMPLICIT REAL*8 (A-H,O-Z)

      DATA M,PSI/0,5.D0/
      SAVE M,PSI,SPS,CPS
      IF(M.EQ.1.AND.DABS(PS-PSI).LT.1.D-5) GOTO 1
      SPS=DSIN(PS)
      CPS=DCOS(PS)
      PSI=PS
      M=1
  1   P=X**2
      U=Z**2
      V=3.D0*Z*X
      T=Y**2
      Q=30574.D0/DSQRT(P+T+U)**5
      BX=Q*((T+U-2.D0*P)*SPS-V*CPS)
      BY=-3.D0*Y*Q*(X*SPS+Z*CPS)
      BZ=Q*((P+T-2.D0*U)*CPS-V*SPS)
      RETURN
      END

C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
